#!/bin/bash 
# Description: Run repeated simulations while varying DATA.INI parameters (FD=free draining, HI= w/ hydrodynamic interactions)

######################### Sec. 0) SET UP PARAMETERS

ini_master="INI_files/DATA.INI.master" # INI file (master)
ini_trap="INI_files/DATA.INI.trap" # (entropic trapping)
ini_pipe="INI_files/DATA.INI.pipe" # (pipe simulations)
ini_vaf="INI_files/DATA.INI.vaf" # (velocity autocorrelation fcn simulations)
ini_wall="INI_files/DATA.INI.wall" # (wall interactions simulations)

ini_tracker="DATA.INI" # Tracker's parameters (to be overwritten as needed)
ini_script="Utils/runsims.ini" # script parameters (e.g. N)
ini_matlab="Utils/runsims.m" # parameters for Matlab
qsub_script="Utils/qsub.sh"
root=`dirname $0`/.. # Tracker's home directory

# Check whether running locally or on vulcan cluster
# NOTE: An alternative (more likely to succeed) way of running on vulcan is mentioned
#       below with simulation III) and IV) (see ON VULCAN). Use for other simulations if needed.
if [ `uname -n` == "vulcan" ]
then
# vulcan cluster
tracker="qsub $qsub_script"
wait="Utils/wait.sh"
#wait=""
else
# local desktop
tracker="./Tracker"
wait=""
fi

export TRACKER_HOME=$root
cd $root

# Read in simulation params
relaxation=( `sed -n 's/^relaxation=\(.*\).*/\1/p' $ini_script` ) # Boolean values (1=YES, 0=NO)
scaling=( `sed -n 's/^scaling=\(.*\).*/\1/p' $ini_script` ) #
confinement=( `sed -n 's/^confinement=\(.*\).*/\1/p' $ini_script` ) #
trap=( `sed -n 's/^trap=\(.*\).*/\1/p' $ini_script` ) #
HI=( `sed -n 's/^HI=\(.*\).*/\1/p' $ini_script` ) # (hydrodynamic interactions)
vaf=( `sed -n 's/^vaf=\(.*\).*/\1/p' $ini_script` ) #
N=( `sed -n 's/^N=.*\[\(.*\)\].*/\1/p' $ini_script` ) # fill in beads array (extract string between '[' and ']')
H_array=( `sed -n 's/^H_array=.*\[\(.*\)\].*/\1/p' $ini_script` ) # fill in height array
Hpdf=( `sed -n 's/^Hpdf=\(.*\).*/\1/p' $ini_script` ) # for which chain length to calculate pdf P(y) vs y ...
pressure=( `sed -n 's/^pressure=.*\[\(.*\)\].*/\1/p' $ini_script` ) # ... and for which Peclet numbers?
Ntrap=( `sed -n 's/^Ntrap=.*\[\(.*\)\].*/\1/p' $ini_script` ) # fill in beads array
Vtrap=( `sed -n 's/^Vtrap=.*\[\(.*\)\].*/\1/p' $ini_script` ) # fill in voltages array
Zeta=( `sed -n 's/^Zeta=.*\[\(.*\)\].*/\1/p' $ini_script` )

######################### Sec. I) BULK SIMULATIONS

if [ $relaxation = 1 ]
then
######## Subsec. a) Polymer Relaxation
cp $ini_master $ini_tracker # select desired ini file
sed -i 	-e 's/Stretch =.*/Stretch = 1/' \
	-e 's/Mode =.*/Mode = 2/' \
	-e 's/BOND =.*/BOND = 16000/' \
	-e 's/dt =.*/dt = 1e-5/' \
	-e 's/equil_frac =.*/equil_frac = 0/' $ini_tracker
for i in ${N[@]}
do
echo "Running Relaxation, FD, N=$i..."
sed -i  -e 's/NoOfParticles.*/NoOfParticles = '$i'/' \
	-e 's/Momentum_transfer.*/Momentum_transfer = 0/' \
	-e 's/outputSuffix.*/outputSuffix = R_FD_'$i'/' $ini_tracker \
	&& $tracker && $wait # FD
if [ $HI = 1 ]
then
echo "Running Relaxation, HI, N=$i..." 
sed -i  -e 's/Momentum_transfer.*/Momentum_transfer = 1/' \
	-e 's/outputSuffix.*/outputSuffix = R_'$i'/' $ini_tracker \
	&& $tracker && $wait # HI
fi
done
fi

if [ $scaling = 1 ]
then
####### Subsec. b) Polymer Scaling Laws
cp $ini_master $ini_tracker # select desired ini file
for i in ${N[@]}
do
echo "Running Scaling, FD, N=$i..."
sed -i  -e 's/NoOfParticles.*/NoOfParticles = '$i'/' \
	-e 's/Momentum_transfer.*/Momentum_transfer = 0/' \
	-e 's/friction_corr.*/friction_corr = 1/' \
	-e 's/outputSuffix.*/outputSuffix = S_FD_'$i'/' $ini_tracker \
	&& $tracker && $wait # FD
if [ $HI = 1 ]
then
echo "Running Scaling, HI, N=$i..."
sed -i 	-e 's/Momentum_transfer.*/Momentum_transfer = 1/' \
	-e 's/friction_corr.*/friction_corr = 0.1/' \
	-e 's/outputSuffix.*/outputSuffix = S_'$i'/' $ini_tracker \
	&& $tracker && $wait # HI
fi
done
fi


if [ $confinement = 1 ]
then
######################### Sec. II) CONFINEMENT SIMULATIONS
cp $ini_master $ini_tracker # select desired ini file
sed -i	-e 's/Wall =.*/Wall = 1/' \
        -e 's/Stretch =.*/Stretch = 1/' \
        -e 's/Mode =.*/Mode = 2/' \
        -e 's/Export_vmd =.*/Export_vmd = 1/' \
        -e 's/BOND =.*/BOND = 4000/' $ini_tracker
for h in ${H_array[@]} # loop over heights [microns]
do
echo "Running Confinement, FD, H=$h..."
sed -i  -e 's/height.*=.*/height = '$h'e-6/' \
	-e 's/Momentum_transfer.*/Momentum_transfer = 0/' \
	-e 's/outputSuffix.*/outputSuffix = C_FD_'$h'/' $ini_tracker \
	&& $tracker && $wait # FD
if [ $HI = 1 ]
then
echo "Running Confinement, HI, H=$h..."
sed -i	-e 's/Momentum_transfer.*/Momentum_transfer = 1/' \
	-e 's/outputSuffix.*/outputSuffix = C_'$h'/' $ini_tracker \
	&& $tracker && $wait # HI
fi

########### Sec. II-VMD) Process VMD files
# create a TCL script for VMD
cat > ${root}/Output/init_C_${h}.tcl <<+
# Initialize VMD & show dynamics
# Petr Hotmar
mol load psf $root/Output/CHAIN_C_$h.psf
mol addfile $root/Output/HISTORY_C_$h.dlpolyhist step 1000 waitfor all
mol modstyle all all CPK
# geometry height [microns] (should reflect the status of DATA.INI)
source $root/Grids/pipe3D.tcl
pipe3D $h
#rotate x by -45
#rotate y by -45
#scale to 0.07
#rock x by 0.08
#animate forward
#package require vmdmovie
#mkdir $root/VMD_movie
#::MovieMaker::makemovie $root/VMD_movie tracker ppmtompeg snapshot 10
+
###########
done

# ### Vary Peclet for height=Hpdf
# for p in ${pressure[@]} # loop over Pe
# do
# sed -i -e 's/height.*=.*/height = '$Hpdf'e-6/' \
#         -e 's/Pressure_driven.*=.*/Pressure_driven = 1/' \
# 	-e 's/pressure.*=.*/pressure = -'$p'/' \
# 	-e 's/outputSuffix.*/outputSuffix = C_P'$p'/' $ini_tracker \
# 	&& $tracker
# done

fi


if [ $trap = 1 ]
then
######################### Sec. III) ENTROPIC TRAP SIMULATIONS
cp $ini_pipe $ini_tracker # select desired ini file

for i in ${Ntrap[@]} # loop over N
do
for v in ${Vtrap[@]} # loop over V
do
echo "Running Entropic Trap, FD, N=$i, V=$v..."
sed -i  -e 's/NoOfParticles.*/NoOfParticles = '$i'/' \
        -e 's/a_inflow1.*/a_inflow1 = '$v'/' \
	-e 's/outputSuffix.*/outputSuffix = Tn'$i'v'$v'/' $ini_tracker \
	&& $tracker && $wait 
#	&& cp $ini_tracker ${ini_tracker}.n${i}v${v} \
#	&& sed 's/Tracker_static.*/Tracker_static -ini=DATA.INI.n'$i'v'$v' > Tracker.output.$JOB_ID /' < $qsub_script > ${qsub_script}.n${i}v${v} \
#	&& qsub ${qsub_script}.n${i}v${v}
### ON VULCAN: Uncomment above 3 lines and comment out '&& $tracker && $wait'

done
done
fi


if [ $vaf = 1 ]
then
######################### Sec. IV) LLNS, VAF tests, temperature check
cp $ini_vaf $ini_tracker # select desired ini file

for z in ${Zeta[@]} # loop over zeta
do
echo "Running LLNS, VAF tests, temperature check, zeta_corr=$z..."
sed -i  \
        -e 's/NoOfSims.*/NoOfSims = 100/' \
        -e 's/NoOfTimeSteps =.*/NoOftimeSteps = 100/' \
        -e 's/height.*/height = 100e-6/' \
        -e 's/outputSuffix.*/outputSuffix = Z'$z'/' $ini_tracker \
        -e 's/friction_corr.*/friction_corr = '$z'/' \
	&& $tracker && $wait 
#	&& cp $ini_tracker ${ini_tracker}.z${z} \
#	&& sed 's/Tracker_static.*/Tracker_static -ini=DATA.INI.z'$z' > Tracker.output.$JOB_ID /' < $qsub_script > ${qsub_script}.z${z} \
#	&& qsub ${qsub_script}.z${z}
### ON VULCAN: Uncomment above 3 lines and comment out '&& $tracker && $wait'
done
fi

######################### Sec. V) EXPORT PARAMETERS FOR MATLAB
# Processhdf.m may need some parameters from Tracker's INI file and some script parameters, so save them in $ini_matlab
sed 's/#/%/g' $ini_tracker | grep -v '^\[' | grep -v '^name' | grep -v '^outputSuffix' > $ini_matlab # process ini file ...
cat $ini_script >> $ini_matlab # ... and append script params
