/*
  ===================================================================================================
  Description:
  Hybrid particle-continuum solver (Lagrangian particle tracking) for polymer/colloidal dynamics
  Supports 2D & 3D geometries; serial version
  Version 4.0

  Requires: 
  1) modified interpPoints routine (Overture)
  2) modified MD fortran files (Dlpoly)
  3) modified ini parser (iniParser)
  4) modified ANN (Approximate Nearest Nieghbor Library)

  Input files:  
  1) DATA.INI (input parameters)
  2) CONFIG / CFGMIN file (coordinate file) (CONTROL and FIELD generated by the program)

  Output files (Output/): 
  1) solutions (.show & .hdf files)
  2) dlpoly output files (OUTPUT,REVCON,REVIVE)
  3) (optional) VMD files (HISTORY, .PSF)

  Notes: Poisson-Boltzmann (elst field) and linearized Navier-Stokes (elosmotic velocity) 
  eqns solved using central finite difference approximations. Navier-Stokes solved using 
  Henshaw's pressure damping method, particle trajectories integrated with r-RESPA.
  Implemented in both 2nd and 4th order accuracies (corresponding HDF grid required).
  Currently 6 contributions to the total force acting on a particle: 
  * Elphoretic & Dielphoretic, Stokes drag & Random (Gaussian noise), Molecular Dynamics forces,
  Wall Repulsion *
  DLPOLY routines called in 3 phases (Initialize & Equilibrate, Evaluate forces, Terminate) to
  determine molecular dynamics forces. Only dry runs supported in the dlpoly interface (no
  dynamics, only force evaluation).
  Both TRACERS (no flow disturbance) and COLLOIDS/POLYMERS (momentum flux transfer) modeled.
  Suuported flow fields: electroosmotic, shear flow, pressure-driven flow
  Also supported unsteady treatment: Method of False Transients (MOFT):
  * Only supports 2nd order accuracy (4th order not supported by Neumann BC)
  * Recommend building grids with EXPLICIT interpolation to speed up computations
  * Currently supports geometries with up to 2 inlets and 2 outlets
  ===================================================================================================
*/
#ifdef USE_MPI
#include "mpi.h"        // For parallelization of sample paths
#endif
#include "Overture.h"  
#include "CompositeGridOperators.h"
#include "Oges.h"
#include "OGPolyFunction.h"
#include "display.h"
#include "Ogshow.h"
#include "interpPoints.h"
#include "PlotStuff.h"
#include "MappedGridOperators.h"
#include "HDF_DataBase.h"
#include <iostream.h>
#include <cmath>
#include <iomanip>      // For setw()
#include <ctime>        // For time()
#include <cstdlib>      // C++ standard general utilities (for srand() and rand())
#include <stdlib.h>     // C standard general utilities (for Quick_sort())
#include <fstream>      // File I/O
#include <string>       // For string manipulation
#include <A++.h>        // A++ array manipulation
#include <vector>       // vector template
#include <algorithm>
#include <map>          // map template
#include <deque>        // deque template
#include <queue>        // queue template
#include "iniparser.h"  // parser's header
#include <ANN/ANN.h>	// ANN's header
#include "sarw.h"       // sarw's header
#include "Tracker.h"    // Tracker's header
#include <sys/stat.h>   // for FileExists
// #include "Regrid.h"     // For AMR
// #include "ErrorEstimator.h"
// #include "InterpolateRefinements.h"
// #include "Ogen.h"
// #include "ParentChildSiblingInfo.h"
// #include "ParallelUtility.h"
// #include "App.h"
// #include "LoadBalancer.h"
// #include "GridStatistics.h"

int main(int argc, char *argv[])
{
  using namespace Data_ini;  // use data.ini vars
  using namespace Main_misc; // use misc vars
  using namespace PP;

  Overture::start(argc,argv);  // initialize Overture
  cout << "\nUSE: Tracker [-ini=<ini_file>] [-solver=[yale][harwell][slap][petsc][mg]]\n";
  cout << "NOTE: The selected order of accuracy must be supported by the HDF grid file.\n ";
  cout << "      Disabled momentum transfer option runs a free-draining code (no HIs).\n ";
  
  int len=0;  
  aString nameOfINIFile="DATA.INI"; // default INI file
  int SolverType=OgesParameters::yale; aString Solver="yale"; // default solver
  if( argc > 1 ) // loop over command line arguments
    { 
      for( i=1; i<argc; i++ )
	{
	  aString arg = argv[i];
	  if( len=arg.matches("-solver=") )
	    {
	      Solver=arg(len,arg.length()-1);
	      if( Solver=="yale" )
		SolverType=OgesParameters::yale;
	      else if( Solver=="harwell" )
		SolverType=OgesParameters::harwell;
	      else if( Solver=="petsc" || Solver=="PETSc" )
		SolverType=OgesParameters::PETSc;
	      else if( Solver=="slap" || Solver=="SLAP" )
		SolverType=OgesParameters::SLAP;
	      else if( Solver=="mg" || Solver=="multigrid" )
		SolverType=OgesParameters::multigrid;
	      else
		{
		  printf("Unknown Solver=%s \n",(const char*)Solver);
		  throw "error";
		}
	    }
	  else if( len=arg.matches("-ini=") )
	    {
	      nameOfINIFile=arg(len,arg.length()-1);
	    }
	}
    }
    
  time_START=getCPU(); // start the overall timer
  time_INI=time_START; // start the timer for initialization phase

  int rank=0, size=1; // defaults
#ifdef USE_MPI
  MPI::Init(argc,argv);
  size=MPI::COMM_WORLD.Get_size(); // number of processes
  rank=MPI::COMM_WORLD.Get_rank(); // process ID
  printf("I'm process no. %i out of %i.", rank, size);
#endif    

  import_INI(nameOfINIFile,rank); // read in input parameters

  //=========================================================================================
  //  Load grid
  //=========================================================================================
 
  cout << "\n *****************************************************************\n";
  cout << " Loading & updating grid: " << nameOfOGFile << "\n";
  cout << " *****************************************************************\n\n";
  
  getFromADataBase(cg,nameOfOGFile);
  cg.update(MappedGrid::THEvertex | MappedGrid::THEvertexBoundaryNormal); // compute these on the grid, not more
  Interpolant interpolant(cg); // allow interpolation on this grid

  import_grid(nameOfOGFile); // NOTE: This routine has to be extended for each new HDF grid! 

  // System dimension (also corresponds to the number of velocity components (e.g. u,v) and el. field strength components (e.g. E_x, E_y))
  Dim = cg.numberOfDimensions(); // Determine dimension of the grid
  cout << "\nGrid dimension: " << Dim << endl;

  // Domain bounds
  realArray domain_min(Dim),domain_max(Dim);
  getDomainBounds(domain_min,domain_max);

  //=========================================================================================
  //    Calculate derived parameters
  //=========================================================================================
  cout << "\n *****************************************************************\n";
  cout << " Processing INI parameters...\n";
  cout << " *****************************************************************\n\n";

  double mu_p=(charge_p*PROTON)/(6*Pi*eta*rad_p); // particle electrophoretic mobility; units of charge = [proton charge]

  // FORCE PREFACTORS etc.
  double 
    FRICTION=6*Pi*rad_p*eta*friction_corr, // Stokes friction constant (tuning parameter) [kg/s]
    EP=6*Pi*mu_p*eta*rad_p, // Elphoretic constant (=surface charge on particle)
    DEP=-Pi*dep_corr*epsilon*rad_p*rad_p*rad_p, // Dielphoretic constant
    DEP_Q=DEP*rad_p*rad_p/3./3.; // Dielphoretic constant for quadrupole
  Diff=KB*temp/FRICTION; // Stokes-Einstein diffusion coefficient for a monomer [m2/s] (assumes spherical bead & free solution)
  
  // *** SCALES FOR NON-DIMENSIONALIZATION --- START ***
  // 0) ELECTRIC POTENTIAL scales  (available: zeta wall potential for intrinsic field, max(phi) for applied field)
  const int NoOfBCs=4; // support max of 2 inflows and 2 outflows
  int minmax = 1; // 0~min, 1~max
  double voltages[NoOfBCs]= {a_inflow1, a_inflow2, a_outflow1, a_outflow2}; // voltages applied at ports
  i_wall!=0. ? phi_i_0=abs(i_wall) : phi_i_0=1.;; // set to zeta potential @ wall (or set to 1 if i_wall=0)
  phi_a_0=abs(find_minmax(voltages,NoOfBCs,minmax)); // set to max(phi_applied)

  // 1) LENGTH scales (available: system height & length, particle radius (colloid) & hydrodynm. radius (polymer, for ideal chain: 3/8*sqrt(pi)*Rgyr), diffusive length scale D/u_0thermal, spring extensibility q0 (for polymers))
  // Note that height was chosen for scaling the 'separator' geometry => other scales do not capture dynamics in the geometry
  switch (LengthScale) // choose
    {
    case 1: L=height;   cout << "Length scale: height of geometry\n"; break; // height an appropriate scale for confined geometry
    case 2: L=rad_p;    cout << "Length scale: bead radius\n"; break;
    case 3: L=q0;       cout << "Length scale: max spring extensibility\n"; break;
    case 4: L=sqrt(q0*bk/6.); cout << "Length scale: radius of gyration of ideal chain\n"; break;
    case 5: L=Diff/sqrt(KB*temp/mass_p); cout << "Length scale: diffusional\n"; break; // u_0t*t_r
    default: cout << "UNKNOWN length scale!\n";
    }
  E_0=phi_a_0/L; // el. field strength scale depends on length scale [V/m]

  // 2) TIME scales (available: colloid relaxation (velocity decay), viscous (=> HI interactions), diffusive (=> colloid/polymer diffusion), polymer relaxation (FRICTION/4./ks), ?)
  double 
    t_md=sqrt(mass_p*L*L/KB/temp),
    t_r=mass_p/FRICTION, 
    t_v=L*L*rho/eta, // note that t_v approximates t_r if particle density ~ solvent density
    t_d=L*L/Diff;
  switch (TimeScale) // choose
    {
    case 1: Tau=t_r; cout << "Time scale: relaxation\n"; break; 
    case 2: Tau=t_v; cout << "Time scale: viscous\n"; break; 
    case 3: Tau=t_d; cout << "Time scale: diffusive\n"; break; 
    case 4: Tau=t_md; cout << "Time scale: molecular dynamics\n"; break; 
    default: cout << "UNKNOWN time scale!\n";
    }

  // 3) FORCE scales (available: thermal)
  double     
    f_0t=KB*temp/L; // thermal
  switch (ForceScale) // choose
    {
    case 1: f_0=f_0t;  cout << "Force scale: thermal\n"; break; 
    default: cout << "UNKNOWN force scale!\n";
    }
  //f_0=mass_p*L/Tau/Tau; // temporary! for testing only!
  // NOTE: To have CONSISTENT scaling, the number of scales must be the same as the number of fundamental dimensional units represented by the data (=3),
  //       i.e. all the following scales should be used only for rescaling!

  // 4) VELOCITY scales
  u_0=L/Tau; // 'natural' (convective) velocity
  double
    u_0hs=abs(epsilon*i_wall*E_0/eta), // Helmholtz-Smoluchowski slip velocity, 
    u_0d=Diff/rad_p, // L/t_d => diffusive velocity
    u_0t=sqrt(KB*temp/mass_p), // thermal velocity
    u_0s=abs(shear_rate*height), // shear velocity
    u_0f=KB*temp*Tau/mass_p/L; // to get F0=KB*T/L

  // 5) PRESSURE scale
  p_0=u_0*eta/L; // 'natural' pressure scale

  // 6) FRICTION COEFF. scale
  zeta_0=f_0/u_0;
  double Zeta=FRICTION/zeta_0, Inv_zeta=1./Zeta;

  // 7) MASS scale
  mass_0=f_0*Tau*Tau/L;
  double Mass=mass_p/mass_0;

// //   // *** Ahlrich's experiments *** (START)
//   // redefine Mass & Zeta for Ahlrich's experiments (otherwise comment out!)
//   Mass=1; 
//   Zeta=Mass/dt*Tau*friction_corr;
//   FRICTION=zeta_0*Zeta; // temporary, assumes t_r, t_d, u_0d Inv_zeta not used!
//   Diff=KB*temp/FRICTION; // temporary, assumes t_r, t_d and u_0d not used
//   printf("\nLong-time tail prefactor B = (1/1)2 * (N*m/rho) * (pi*nu)^(-3/2) = %e", 1./12.*KB*temp*sqrt(rho)*pow(Pi*eta,-3./2.));
// //   // *** Ahlrich's experiments *** (END)


  // *** SCALES FOR NON-DIMENSIONALIZATION --- END ***
  bool Quadrupole=0;

  // Debye length
  double ksqr=L*L/debye/debye;

  // BC's for Poisson-Boltzmann
  i_wall/=phi_i_0; i_wall2/=phi_i_0; // non-dimensionalize
  a_inflow1/=phi_a_0; a_inflow2/=phi_a_0; a_outflow1/=phi_a_0; a_outflow2/=phi_a_0;

  // BC's for Navier-Stokes
  //BC BC_NS = {0,0,0,0,0}; // initialize: no pressure & no slip @ walls, unless...
  if (Pressure_driven) BC_NS.p_inflow2=BC_NS.p_inflow1=pressure / p_0; // set up pressure gradient (and non-dimensionalize)
  if (Shear_driven) BC_NS.v_wall=shear_rate*height / u_0; // set up a sliding wall (and non-dimensionalize)

  // Define some time-related properties
  const double
    dtT_nondim=dtT/Tau, // non-dimensionalized time step for tracers
    dt_nondim=dt/Tau, // non-dimensionalized time step for colloids
    dt_d_nondim=dt_d/Tau,
    dt_u_nondim=dt_u/Tau,
    dt_v_nondim=dt_v/t_v,
    Sc=eta/rho/Diff, // Schmidt number = momentum diff / mass diff (if large, use of force-based coupling (Langevin eqn) justified)
    Pe=L*u_0/Diff, // Peclet number = Re*Sc (if small, hydrodynamic interactions important)
  
    // Define proportionality constants for the forces & non-dimensionalize
    Fep=(Electrophoretic)*EP*E_0/f_0,  // prefactors for electric forces (elphoretic, dielphoretic)
    Fdep=(Dielectrophoretic)*DEP*E_0*E_0/L/f_0,
    Fdep_q=(Quadrupole)*DEP_Q*E_0*E_0/L/L/L/f_0,
    Fmd=(0.001/AVOG*1e-10*1e24)/f_0, // prefactor for MD forces from DLPOLY ( force units: [Da*A*ps] = 1.67e-13 [N] )
    NS_rescale=f_0*Tau/eta/L/L, // rescale net dimensionless force (f_hat) from particle dynamics to continuum dynamics (Navier-Stokes)

    // Factors for rescaling positions/velocities 
    Position_rescale=1e-10/L,  // CONFIG positions in Angstroms = 1e-10 m => rescale/nondim to Overture format
    Velocity_rescale=1e2/u_0;  // CONFIG velocities in Angstroms/ps = 1e2 m/s => rescale/nondim to Overture format

  // Results from polymer theory of ideal chains (Rubinstein) - bulk equilibrium values
  const double
    polymer_size=q0*pow(double(NoOfParticles), .5), // assume ideal chain with the reciprocal of fractal dimension = 0.5
    mseed=NoOfParticles*q0*q0,
    msrgyr=mseed/6.,
    tau_rouse=FRICTION*q0*q0*NoOfParticles*NoOfParticles/(6.*PI*PI*KB*temp),
    tau_zimm=0.163*eta*pow(polymer_size,3.)/(KB*temp),
    diff_rouse=KB*temp/(NoOfParticles*FRICTION),
    diff_zimm=0.196*KB*temp/(eta*polymer_size);

  // r-RESPA params
  const double
    respa0=.5*dt_nondim/Mass,
    respa1=N1*respa0,
    respa2=N2*respa1,
    respa3=N3*respa2;

  // Params for bead-wall repulsion potential (SI units)
  // 1) Jendrejack
  double A_w=Aw*KB*temp; delta_w=bk*sqrt(q0/bk)/2; // adjusted for entropic trap geometry (Jen: A_w/KB/temp = 25, default: 40)
  // 2) 9-3, 10-4
  if (Wall == 2 or Wall == 3)
    {
      A_wall=0.4*KB*temp*8*Pi*0.74; 
      B_wall=0.3e-9;  // assume PDMS (eps=1kJ/mol=0.4kT, sigma=0.3e-9m; literature: eps=1kT)
    }
  // 3) Lennard-Jones
  else if (Wall == 4)
    { // Matysiak's params
      A_wall=16/0.9*vdw_epsilon/(0.1*AVOG); // dlpoly -> SI units
      B_wall=vdw_sigma*1e-10; // dlpoly -> SI units
    }
  // Non-dimensionalize
  A_w/=(f_0*L); A_wall/=(f_0*L); delta_w/=L; B_wall/=L; // non-dimensionalize relevant quantities (A_w has units of energy)
  C_wall=A_w/(3.*bk/L*delta_w*delta_w); // wall repulsion prefactor
  printf("\nWall potentials:\nJENDREJACK:\t [A_w,delta_w,C_wall]=[%e,%e,%e] \n9-3:\t\t [A_wall,B_wall]=[%e,%e]\n", A_w, delta_w, C_wall, A_wall, B_wall);
    
  // take background grid as representative for spacing determination, assume equal spacings in all coordinate directions
  spacing=cg[0].gridSpacing(axis1);
  damping=1./spacing/spacing;    // damping factor for pressure-Poisson eqn ~ mu/h^2, h=grid spacing
  printf("\nBackground grid spacing = [%f,%f,%f]\n", cg[0].gridSpacing(axis1), cg[0].gridSpacing(axis2), cg[0].gridSpacing(axis3));
  cout << "Damping term for Navier-Stokes: " << damping << endl;

  // show & hdf files for eikonal, poisson-boltzmann & navier-stokes eqns
  aString 
    eikShow="Output/eik.show",
    pbShow="Output/pb.show",
    nsShow="Output/ns.show",

    eikHdf="Output/eik.hdf",
    pbHdf="Output/pb.hdf",
    nsHdf="Output/ns.hdf";
  

  const double
    a_ll=NS_rescale * sqrt(2*KB*temp*eta/(dt*spacing*spacing*spacing*L*L*L)) *L*L/f_0, // coeffs of LLNS stochastic tensor 'S'
    b_ll=sqrt(4./3);

#ifdef USE_MPI
if(rank==0) // only root does I/O...
{
#endif 
  // *** PRINT-OUT A SUMMARY ***
  cout << " \nPrinting out some important user-defined parameters...";
  cout << "\n ****************** PREFACTORS ***********************************";
  cout << "\nNondim timestep: " << dt_nondim;
  cout << "\nStokes friction: " << FRICTION;
  cout << "\nElphoretic force constant: " << EP;
  cout << "\nDielphoretic force constant: " << DEP;
  cout << "\nFep: " << Fep;
  cout << "\nFdep: " << Fdep;
  cout << "\nFdep_q: " << Fdep_q;
  cout << "\nFmd: " << Fmd;
  cout << "\nRescale factor for Navier-Stokes: " << NS_rescale;
  cout << "\n ****************** DIMENSIONLESS NUMBERS ************************";
  cout << "\nReynolds number: " << t_v/Tau;
  cout << "\nSchmidt number: " << Sc;
  cout << "\nPeclet number: " << Pe;
  cout << "\nStokes domain number: " << t_r/Tau; 
  cout << "\nStokes-Einsten diffusivity: " << Diff;
  cout << "\n ****************** CHOSEN SCALES ********************************";
  cout << "\nLENGTH: " << L;
  cout << "\nVELOCITY: " << u_0;
  cout << "\nTIME: " << Tau;
  cout << "\nFORCE: " << f_0;
  cout << "\nMASS: " << Mass;
  cout << "\nPRESSURE: " << p_0;
  cout << "\nZETA: " << Zeta;
  cout << "\nINTRINSIC POTENTIAL: " << phi_i_0;
  cout << "\nAPPLIED POTENTIAL: " << phi_a_0;
  cout << "\n ****************** POLYMER PHYSICS THEORY (SI units) ************";
  cout << "\nMSEED:              " << setw(10) << mseed;
  cout << "\nMSRGYR:             " << setw(10) << msrgyr;
  cout << "\nDiffusion coefficient (Rouse):" << setw(10) << diff_rouse;
  cout << "\nDiffusion coefficient (Zimm): " << setw(10) << diff_zimm;
  cout << "\nRelax. time (Rouse):" << setw(10) << tau_rouse;
  cout << "\nRelax. time (Zimm): " << setw(10) << tau_zimm;
  cout << "\n ************************** OTHER ********************************";
  cout << "\nMu_p: " << mu_p;
  cout << "\nKsqr: " << ksqr;
  cout << endl;
#ifdef USE_MPI
}
#endif    

  //=========================================================================================
  //    Define some arrays common to all solvers
  //=========================================================================================
  cout << "\n *****************************************************************\n";
  cout << " Initializations...\n";
  cout << " *****************************************************************\n\n";
  cout << "Declaring & initializing arrays...\n";
 
  // General Overture params
  R1D=Range(1,Dim); // Range for components, Un, n=1..Dim (e.g. velocity components)
  CompositeGridOperators op(cg); // create differential operators
  op.setOrderOfAccuracy(OrderOfAccuracy);
  BoundaryConditionParameters bcParams;
  Oges solver( cg );             // create a solver
  solver.set(OgesParameters::THEsolverType,SolverType);
  real tolerance=1.e-8; int iluLevels=10, maxIterations=1;
  if( solver.isSolverIterative() ) 
    {
      solver.setCommandLineArguments( argc,argv );
      solver.set(OgesParameters::THEsolverMethod,OgesParameters::generalizedMinimalResidual);
      //solver.set(OgesParameters::THEsolverMethod,OgesParameters::biConjugateGradientStabilized);
      //solver.set(OgesParameters::THEsolverMethod,OgesParameters::biConjugateGradient);
      //solver.set(OgesParameters::THEnumberOfIncompleteLULevels,iluLevels);
      //solver.set(OgesParameters::THEpreconditioner,OgesParameters::incompleteLUPreconditioner);
      //solver.set(OgesParameters::THErelativeTolerance,max(tolerance,REAL_EPSILON*10.));
      //solver.set(OgesParameters::THEmaximumNumberOfIterations,maxIterations);
      //Oges::debug=3;
    } 
  printf("\n === Solver:\n %s\n =====\n",(const char*)solver.parameters.getSolverName());
  
  PlotStuff pi(FALSE,"Visualization");      // create a PlotStuff object
  if(Plot_intermediate || Plot_final) PlotStuff pi(TRUE,"Visualization");
  PlotStuffParameters psp;         // This object is used to change plotting parameters

  int numberOfComponents1 = 2; // phi intrinsic & phi applied (scalar fields)
  int width=OrderOfAccuracy+1; // 4th order accuracy => 5-point stencil
  int numberOfGhostLines=OrderOfAccuracy/2;

  realCompositeGridFunction D(cg,all,all,all,1), Dold(cg,all,all,all,1), Dd(cg,all,all,all,Dim);
  Dd=0; D=Dold=0;
  D.setOperators(op);                                 

  // Grid Functions for steady & unsteady treatments, noOfComponents2=Dim+1 (=> pressure included)
  realCompositeGridFunction u2(cg,all,all,all,Dim+1), u2old(cg,all,all,all,Dim+1), f2(cg,all,all,all,Dim+1), f3(cg,all,all,all,Dim+1),
   tmp1(cg,all,all,all,Dim+1),tmp2(cg,all,all,all,Dim+1),tmp3(cg,all,all,all,Dim+1);
  u2=u2old=f2=tmp1=tmp2=tmp3=0.; 
  u2.setOperators(op);
  f2.setOperators(op);
  f3.setOperators(op);
  tmp1.setOperators(op);
  tmp2.setOperators(op);
  tmp3.setOperators(op);  

  realCompositeGridFunction  E(cg,all,all,all,Dim), Ei(cg,all,all,all,Dim), grad_E_sq(cg,all,all,all,Dim), E_sq(cg);
  E=Ei=grad_E_sq=0.; E_sq=0.; // initialize to zero

  realCompositeGridFunction div_force_Transfered(cg), 
    u_fluid_base(cg,all,all,all,Dim), 
    u_fluid(cg,all,all,all,Dim), 
    up(cg,all,all,all,Dim),
    force_Transfered(cg,all,all,all,Dim);
  force_Transfered=0;
  
  realCompositeGridFunction u(cg,all,all,all,numberOfComponents1), uold(cg,all,all,all,numberOfComponents1),
   tmp11(cg,all,all,all,numberOfComponents1),tmp22(cg,all,all,all,numberOfComponents1),tmp33(cg,all,all,all,numberOfComponents1);
  u=uold=tmp11=tmp22=tmp33=0.; // initialize
  u.setOperators(op);                             // associate with a grid function
  tmp11.setOperators(op);
  tmp22.setOperators(op);
  tmp33.setOperators(op); 

  doubleArray Fwall(NoOfParticles,Dim); Fwall=0;

  realCompositeGridFunction v(cg,all,all,all,Dim+1), p(cg), p_base(cg), f(cg,all,all,all,Dim+1);
  v=0; p=0; f=0; // initialize
  v.setOperators(op); // associate operators with a grid function
  p.setOperators(op);
  f.setOperators(op);
  force_Transfered.setOperators(op);

  // Set up an array for LLNS
  realCompositeGridFunction force_LL(cg,all,all,all,6);
  force_LL.setOperators(op);
  force_LL=0.;

  // Set up particle dynamics arrays: position, velocity, electric field...
  doubleArray 
    position_tracer(NoOfParticles,Dim), 
    positionToInterpolate(NoOfParticles,Dim), 
    u_fluid_interpolated(NoOfParticles,Dim),
    forceToExtrapolate(NoOfParticles,Dim), 
    E_interpolated(NoOfParticles,Dim),
    grad_E_sq_interpolated(NoOfParticles,Dim),
    Q_interpolated(NoOfParticles,Dim);

  position_tracer=0.;
  positionToInterpolate=u_fluid_interpolated=forceToExtrapolate=E_interpolated=grad_E_sq_interpolated=Q_interpolated=0.;

  doubleArray position(NoOfParticles,Dim), position_ini(NoOfParticles,Dim),
    velocity(NoOfParticles,Dim), velocity_ini(NoOfParticles,Dim),
    force(NoOfParticles,Dim),
    Random(NoOfParticles,Dim);
  doubleArray eev(Dim), com(Dim), vcom(Dim); // end-to-end vector; position, velocity of center of center of mass
  velocity_ini=0; Random=0;
  eev=com=vcom=0;

  // Set up post-processing arrays for magnitudes of forces
  doubleArray Zeta_at_Tf(NoOfParticles),
    Fep_at_Tf(NoOfParticles),
    Fdep_at_Tf(NoOfParticles),
    Frnd_at_Tf(NoOfParticles),
    Fmd_at_Tf(NoOfParticles),
    Fw_at_Tf(NoOfParticles);
  double Zeta_max,Fep_max,Fdep_max,Frnd_max,Fmd_max,Fw_max;

  //realCompositeGridFunction Q(cg,all,all,all,Dim); Q=0.; // *** DEP QUADRUPOLE

  // Coefficient matrix (CM) for Navier-Stokes
  realCompositeGridFunction coeff2;

  //=========================================================================================
  //    ppAlloc - allocate polymer properties arrays
  //=========================================================================================
  // simulation = equilibration + production
  NoOfEquilSteps=equil_frac*NoOfTimeSteps;
  NoOfProdSteps=(NoOfTimeSteps-NoOfEquilSteps)/ExportFreq; 

  int NPPD=0; PPDrows=0;
  int NPPS=0; PPSrows=0;
  int NPPSIM=0; PPSIMrows=0;

  NT0=ceil( NoOfProdSteps*nof ); // ratio of time steps over time origins
  if (!NT0)
    {
      printf("\nNT0 cannot be 0. Check your inputs (NoOfTimeSteps, equil_frac, ExportFreq). Aborting now...");
      abort();
    }
  NorgT=NoOfProdSteps/NT0; // # of time origins
  printf("\nParams for correlation fcns: ND=%i, NT=%i, NT0=%i, NorgT=%i\n",Dim,NoOfProdSteps,NT0,NorgT);

  // Fill in indices for dynamic (time-dependent) polymer properties
  if(Msrgyr)  {row_msrgyr=NPPD;   PPDrows(i_msrgyr)=NPPD+1;  NPPD++;} // increment index (NPPD+1) b/c matlab arrays start from 1
  if(Mseed)   {row_mseed=NPPD;    PPDrows(i_mseed)=NPPD+1;   NPPD++;}
  if(Stretch) {row_stretch=NPPD;  PPDrows(i_stretch)=NPPD+1; NPPD++;}
  if(MsdVaf)  {row_msd=NPPD;      PPDrows(i_msd)=NPPD+1;     NPPD++;
               row_vaf=NPPD;      PPDrows(i_vaf)=NPPD+1;     NPPD++;}
  if(Com)     {row_com=NPPD;      PPDrows(i_com)=NPPD+1;     NPPD+=Dim;}

  // Fill in indices for static (time-independent) polymer properties
  if(MsdVaf)  {row_diff_msd=NPPS; PPSrows(i_diff_msd)=NPPS+1;NPPS++;
               row_diff_vaf=NPPS; PPSrows(i_diff_vaf)=NPPS+1;NPPS++;}
  if(Com)     {row_vcom_avg=NPPS; PPSrows(i_vcom_avg)=NPPS+1;NPPS+=Dim;}

  if(1)       {row_transloc=NPPSIM; PPSIMrows(i_transloc)=NPPSIM+1;NPPSIM++;}

  doubleArray PPD(NPPD,NoOfProdSteps), PPDsum(NPPD,NoOfProdSteps), PPSsum(NPPS), PPS(NPPS), PPSIM(NPPSIM,NoOfSims), PPSIM2(NPPSIM,NoOfSims), DiffSims(NoOfSims), DiffSims2 (NoOfSims);  // polymer properties arrays - dynamic (matrix), static (column vector)

  PPD=0; PPDsum=0; PPS=0; PPSsum=0; PPSIM=0; PPSIM2=0; DiffSims=0; DiffSims2=0.;
  doubleArray x0(Dim,NorgT), v0(Dim,NorgT), norm(NoOfProdSteps), N0(NorgT); // correlation fcn arrays
  
  doubleArray time(NoOfProdSteps);
  ForProdSteps(prodstep) // fill in time array
    time(prodstep)=prodstep*dt;

  //=========================================================================================
  //    Set up a interface structures
  //=========================================================================================

  // 1) C++ / Fortran interface (compiler & hardware specific!) & initialize dlpoly
  // Define anonymous external structure to share data (through direct access) with Fortran
#pragma pack(2)
  extern struct {
    double rescale; // SCALING FACTOR FOR POSITION CONVERSION
    int pts, // NUMBER OF PARTICLES
      dim,   // DIMENSION
      steps; // NUMBER OF TIMESTEPS
  } shared_data_; // trailing underscore compiler-dependent
#pragma pack()

  // Initialize the structure
  shared_data_.rescale=Position_rescale;
  shared_data_.pts=NoOfParticles;
  shared_data_.dim=Dim;
  shared_data_.steps=NoOfTimeSteps;

  // 2) SARW interface
  // Initialize sarw structure to DATA.INI params (scale = bond/L, bond in angstroms)
  sarw_data dat =
    {
      NoOfParticles, BOND, L, BOND*1e-10/L, com_ref_x, com_ref_y, com_ref_z,
      Mode
    };

  // 3) Data for forceEval() (SI units)
  fe fed;
  fed.f_MS=0.5*KB*temp/bk; // Marko-Siggia prefactor
  fed.f_H=3*KB*temp/bk; // Hookean spring constant
  fed.v_EV=v_ev;
  fed.Nks=q0/bk;
  fed.Rs2=fed.Nks*bk*bk;
  fed.vdw_e=vdw_epsilon/(0.1*AVOG); 
  fed.vdw_s=vdw_sigma*1e-10; // dlpoly -> SI units
  fed.exp_EV=-9./(2*fed.Rs2); 
  fed.f_EV1=-2*fed.exp_EV * .5*KB*temp*fed.v_EV*fed.Nks*fed.Nks*pow(9./(2*PI*fed.Rs2),3./2.); // Jendrejack
  fed.f_EV2=24*fed.vdw_e/fed.vdw_s; // Lennard-Jones
  fed.lb=0.714e-9; // Bjerrum length [m] for water @ 298K
  fed.f_ES=-charge_p*charge_p*fed.lb*KB*temp;

  fed.k_B=2736*vdw_epsilon/(0.1*AVOG)/rad_p; // bond and angle potentials
  fed.r0=rad_p;
  fed.k_A=8*vdw_epsilon/(0.1*AVOG);
  fed.theta0=PI;

  //=========================================================================================
  //    Generate and load CONFIG / CFGMIN (initial polymer configuration), generate FIELD 
  //    and CONTROL
  //=========================================================================================
  
  // Generate
#ifdef USE_MPI
if(rank==0) // only root does I/O...
{
#endif 
  cout << "\nGenerating CONFIG file...\n";
  if(Mode==2) // 2D zig-zag...
    zigzag_(&NoOfParticles,&BOND,&ALPHA,&THETA); // call Fortran subroutine (zigzag.f90)
  else // ...or 3D self-avoiding random walk
    run_Sarw(dat); // run SARW
#ifdef USE_MPI
}
 MPI::COMM_WORLD.Barrier(); // ...the rest will wait until root finishes I/O
#endif 

  // Import initial positions and velocities from CFGMIN or CONFIG file
  // NOTE!: Alternatively, initial particle velocities can be determined from fluid velocity field
  // end electric field in the 'colloid dynamics' section below

  import_CONFIG(position, velocity); // import position and velocity from CONFIG
  shift_position(position, Position_rescale); // shift position

#ifdef USE_MPI
MPI::COMM_WORLD.Barrier(); // Makes sure everybody finished importing CONFIG before root starts changing it
if(rank==0) // only root does I/O...
{
#endif
  write_position_to_CONFIG(position, "CONFIG"); // overwrite config with shifted position
#ifdef USE_MPI
}
#endif 

  rescale_position(position, velocity, position_ini, Position_rescale, Velocity_rescale); // rescale position to Overture units

#ifdef USE_MPI
MPI::COMM_WORLD.Barrier(); // Makes sure everybody finished importing CONFIG before root starts changing it
if(rank==0) // only root does I/O...
{
#endif 
  int NoOfWallAtoms=0;
  if (Wall==4) // discrete atoms
    NoOfWallAtoms = add_walls_to_CONFIG("Walls/ATOMS.XYZ." + outputSuffix, "CONFIG", Position_rescale); // add wall atoms for dlpoly
  export_FIELD_CONTROL("FIELD", "CONTROL",NoOfWallAtoms); // write dlpoly input files
#ifdef USE_MPI
}
 MPI::COMM_WORLD.Barrier(); // ...the rest will wait until root finishes I/O
#endif 

#ifdef USE_DLPOLY
  //position.setBase(1); // set Fortran array indexing
  if(Molecular_dynamics)
    {
      cout << "\nInitializing Dlpoly...(if crash occurs here, examine OUTPUT for details)\n";
      dlpoly_initialize_(); // DLPOLY: Initialize and Equilibrate (phase 1 of 3) CONFIG -> CFGMIN
    }
#else
  if(Wall==4)
    {
      cout << "Discrete walls (Wall=4) require DLPOLY support! Use -DUSE_DLPOLY in your Makefile. Aborting now.\n";
      abort();
    }
#endif 

  time_INI=getCPU()-time_INI;

  //=========================================================================================
  //    Solve for wall distance (Eikonal eqn) or load the solution, if available
  //=========================================================================================

  time_EIK=getCPU();
  if (Wall)
    {
      bool File_Exists=FileExists(eikHdf); // solved already & saved in HDF?
      HDF_DataBase db;

      if(File_Exists) // yes
	{
	  cout << "\nIMPORTING wall distance & its derivatives from " << eikHdf << " ...\n";
	  db.mount(eikHdf,"R"); // mount, read-only
	  D.get(db, "Wall distance"); // load distance & its derivatives from HDF
	  Dd.get(db, "Wall distance derivatives"); 
	}
      else // no
	{
	  cout << "\n *****************************************************************\n";
	  cout << " Solving for wall distance function...\n";
	  cout << " *****************************************************************\n\n";
 
	  db.mount(eikHdf,"I"); // mount, init
	  Ogshow eikshow( eikShow ); // create a show file
 
	  D=1.;                                   // initial condition
	  real viscosity=.1;                      // parameters
	  int save_freq=50;
  
	  cout << "Solution will be saved in " << eikShow << " every " << save_freq << " steps."
	       << "\nUsing Lax-Friedrichs, artificial viscosity = " << viscosity;
	  printf("\nStriving for MAX ERROR = %e with TIME STEP = %e...\n\n", eps_d, dt_d);

	  t=0.;
	  for( ft_step=1; ft_step<=StepMax; ft_step++ )
	    {

	      if( (ft_step-1) % save_freq == 0 )  // save solution every save_freq steps
		{
		  eikshow.startFrame();              // start a new frame
		  eikshow.saveComment(0,sPrintF(buffer,"Eikonal eqn solution %i, \nt=%e",ft_step-1,t));   // comment 0 (shown on plot)
		  eikshow.saveSolution( D );         // save the current grid function
		}

	      Dold=D;
	      D+=dt_d_nondim*( -D.x()*D.x() - D.y()*D.y() -D.z()*D.z() + viscosity*(D.xx() + D.yy() + D.zz()) + 1.); // take a time step with Euler's method
	      t+=dt_d_nondim;
	      D.interpolate(); // interpolate
      
	      // apply BCs
	      D.applyBoundaryCondition(0,BCTypes::dirichlet,noSlipWall,0.);
	      D.applyBoundaryCondition(0,BCTypes::dirichlet,noSlipWall2,0.);
	      D.applyBoundaryCondition(0,BCTypes::neumann,inflow1,0.);
	      D.applyBoundaryCondition(0,BCTypes::neumann,outflow1,0.);
	      D.finishBoundaryConditions();
      
	      if( ft_step % ErrorFreq == 0)
		if ( (error_d=max(fabs(D-Dold))) > eps_d) // continue if above tolerance
		  printf("\nMax dudt: %e (Eikonal solver step: %i). Continuing...", error_d, ft_step);
		else // break if within tolerance
		  break;    
	    }
  
	  if (ft_step==StepMax)
	    printf("\nReached StepMax = %i and error still not within tolerance!\n", StepMax);

	  cout << "\nMax(D)=" << max(fabs(D)) << ", Min(D)=" << min(fabs(D));

	  eikshow.close(); // close showfile

	  // set ghostlines to 0, otherwise derivatives will be wrong near the boundaries
	  ForGrids(grid)
	  {
	    MappedGrid & mg = cg[grid];
	    ForBoundary(side,axis)
	      {
		for(int ghostline=1; ghostline<=numberOfGhostLines; ghostline++) // loop over ghostlines
		  {
		    getGhostIndex(mg.gridIndexRange(),side,axis,Ig1,Ig2,Ig3,ghostline); // get index
		    D[grid](Ig1,Ig2,Ig3) = 0.; // set to 0
		  }
	      }
	  }
	  D.interpolate();
	  Dd=D.grad(); // Dd=(Dx,Dy,Dz)

 	  // save wall distance & its derivatives to HDF
 	  cout << "\nEXPORTING wall distance & its derivatives to " << eikHdf << " ...\n";
	  D.put(db, "Wall distance");
	  Dd.put(db, "Wall distance derivatives");
	}
      db.unmount();

    }
  time_EIK=getCPU()-time_EIK;

  //=========================================================================================
  //    Set up wall distance kd-tree
  //=========================================================================================

  //----------------------------------------------------------------------
  // Uses ANN (Approximate Nearest Nieghbor Library),
  // Copyright (c) 1997-2005 University of Maryland and Sunil Arya and
  // David Mount.  All Rights Reserved.
  //----------------------------------------------------------------------

  // Parameters
  int			nPts=0;					// actual number of data points = # of lines in the file (to be incremented)
  int                   Pts=0;                                  // reset counter
  ANNpointArray		dataPts;				// data points
  ANNkd_tree*		kdTree;					// search structure
  
  nPts=getNumberOfPoints(cg);                                   // total # grid points
  dataPts = annAllocPts(nPts, Dim);			        // allocate space for data points, i.e. [x y z] coordinates
  doubleArray Dwall(nPts, Dim+1); Dwall=0;                      // for storing [D Dx Dy Dz]

  if (Wall)
    {
      ForGrids(grid)
      {
	MappedGrid & mg = cg[grid];
	getIndex(mg.indexRange(),I1,I2,I3); 
	For3I_Interior(i1,i2,i3,I1,I2,I3) // interior only
	  //For3I(i1,i2,i3,I1,I2,I3) // interior  + boundaries
	  {
	    Dwall(Pts,0)=D[grid](i1,i2,i3);                     // fill in D [D]
	    ForDims(dim)
	    {
	      Dwall(Pts,dim+1)=Dd[grid](i1,i2,i3,dim);          // fill in D derivatives [Dx Dy Dz]
	      dataPts[Pts][dim]=mg.vertex()(i1,i2,i3,dim);      // fill in vertex coordinates [x y z]
	    }
	    Pts++;                                              // increment counter
	  }
      }
      
      cout << "\nSetting up wall distance kd-tree...\n";
      // Build the kd-tree
      kdTree = new ANNkd_tree(					// build search structure
			      dataPts,				// the data points
			      nPts,				// number of points
			      Dim);				// dimension of space
    }

  //=========================================================================================
  //    Set up the Poisson-Boltzmann solver
  //=========================================================================================
  
  time_PB=getCPU();
  if (Electric_field)
    {
      bool File_Exists=FileExists(pbHdf); // solved already & saved in HDF?
      HDF_DataBase db;
      
      if(File_Exists) // yes
	{
	  cout << "\nIMPORTING P-B solution from " << pbHdf << " ...\n";
	  db.mount(pbHdf,"R"); // mount, read-only
	  u.get(db, "P-B solution"); // load from HDF
	}
      else // no
	{
	  db.mount(pbHdf,"I"); // mount, init

      if (!False_transients) // 1) solve by coefficient matrices
	{
	  cout << " \nSetting up the Poisson-Boltzmann solver and applying boundary conditions...\n";
	  // create a grid function to hold the coefficients (coeff1)
	  int stencilSize1=int( pow(width,cg.numberOfDimensions())+1);  // add 1 for interpolation equations
	  int stencilDimension1=stencilSize1*SQR(numberOfComponents1);
	  realCompositeGridFunction coeff1(cg,stencilDimension1,all,all,all); 
	  // make this grid function a coefficient matrix:
	  coeff1.setIsACoefficientMatrix(TRUE,stencilSize1,numberOfGhostLines,numberOfComponents1);
	  coeff1=0.; // initialize to zero
	  // create grid functions to hold the solution and RHS (u,f1) 
	  realCompositeGridFunction f1(cg,all,all,all,numberOfComponents1);

	  op.setNumberOfComponentsForCoefficients(numberOfComponents1);
	  op.setStencilSize(stencilSize1);
	  op.setOrderOfAccuracy(OrderOfAccuracy);
	  u.setOperators(op);           // associate differential operators with u
	  coeff1.setOperators(op);
 
	  // Solve a system of equations for (u_0,u_1) = (phi_Intrinsic, phi_Applied) = (u,v)
	  //     u_xx + u_yy - ksqr u = 0 
	  //     v_xx + v_yy          = 0

	  Overture::printMemoryUsage("Before filling coefficient matrix");
	  // fill in coefficients for the interior points
	  coeff1=op.laplacianCoefficients(eq1,ic)-ksqr*op.identityCoefficients(eq1,ic)+op.laplacianCoefficients(eq2,ac);
	  Overture::printMemoryUsage("After filling coefficient matrix");

	  // fill in coefficients for the boundary conditions
	  coeff1.applyBoundaryConditionCoefficients(eq1,ic,dirichlet,  noSlipWall);
	  coeff1.applyBoundaryConditionCoefficients(eq1,ic,extrapolate,noSlipWall); // extrap 1st ghost line
	  coeff1.applyBoundaryConditionCoefficients(eq1,ic,dirichlet,  noSlipWall2);
	  coeff1.applyBoundaryConditionCoefficients(eq1,ic,extrapolate,noSlipWall2); // extrap 1st ghost line
	  coeff1.applyBoundaryConditionCoefficients(eq1,ic,neumann,  inflow2);
	  coeff1.applyBoundaryConditionCoefficients(eq1,ic,neumann,  inflow1);
	  coeff1.applyBoundaryConditionCoefficients(eq1,ic,neumann,  outflow1);
	  coeff1.applyBoundaryConditionCoefficients(eq1,ic,neumann,  outflow2);
  
	  coeff1.applyBoundaryConditionCoefficients(eq2,ac,neumann,    noSlipWall);
	  coeff1.applyBoundaryConditionCoefficients(eq2,ac,neumann,    noSlipWall2);
	  coeff1.applyBoundaryConditionCoefficients(eq2,ac,dirichlet,  inflow2);
	  coeff1.applyBoundaryConditionCoefficients(eq2,ac,extrapolate,inflow2);
	  coeff1.applyBoundaryConditionCoefficients(eq2,ac,dirichlet,  inflow1);
	  coeff1.applyBoundaryConditionCoefficients(eq2,ac,extrapolate,inflow1);
	  coeff1.applyBoundaryConditionCoefficients(eq2,ac,dirichlet,  outflow1);
	  coeff1.applyBoundaryConditionCoefficients(eq2,ac,extrapolate,outflow1);
	  coeff1.applyBoundaryConditionCoefficients(eq2,ac,dirichlet,  outflow2);
	  coeff1.applyBoundaryConditionCoefficients(eq2,ac,extrapolate,outflow2);

	  if( OrderOfAccuracy==4 )
	    {
	      bcParams.ghostLineToAssign=2; // assign BC also to the 2nd ghost line
	      coeff1.applyBoundaryConditionCoefficients(eq1,ic,extrapolate,allBoundaries,bcParams); // extrap 2nd ghost line
	      coeff1.applyBoundaryConditionCoefficients(eq2,ac,extrapolate,allBoundaries,bcParams); // extrap 2nd ghost line
	    }

	  // fix the corners
	  coeff1.finishBoundaryConditions();
   
	  //=========================================================================================
	  //    Solve Poisson-Boltzmann equation
	  //=========================================================================================
	  cout << "\n *****************************************************************\n";
	  cout << " Solving for electrostatic field...\n";
	  cout << " *****************************************************************\n\n";
 
	  solver.setCoefficientArray( coeff1 );   // supply coefficients
  
	  // assign the right hand sides
	  ForGrids(grid) // loop over the component grids
	  {
	    MappedGrid & mg = cg[grid]; // make a reference (alias) to save typing
	    getIndex(mg.indexRange(),I1,I2,I3);  
	    // RHS of interior points
	    f1[grid](I1,I2,I3,ic)=0.;
	    f1[grid](I1,I2,I3,ac)=0.;

	    // RHS of boundary conditions
	    ForBoundary(side,axis)
	      {
		if( mg.boundaryCondition()(side,axis) > 0  )
		  {
		    getBoundaryIndex(mg.gridIndexRange(),side,axis,Ib1,Ib2,Ib3);
		    getGhostIndex(mg.gridIndexRange(),side,axis,Ig1,Ig2,Ig3);
		    if( mg.boundaryCondition()(side,axis)==inflow2 )
		      {
			// Note: For Neumann BC: interior's RHS (getGhostIndex(...,-1)) extended to the boundary (getBoundaryIndex)
			f1[grid](Ig1,Ig2,Ig3,ic)=0.; // Neumann BC: fill in BC's RHS on the ghostlines
			f1[grid](Ib1,Ib2,Ib3,ac)=a_inflow2;
		      }
		    if( mg.boundaryCondition()(side,axis)==inflow1 )
		      {			
			f1[grid](Ig1,Ig2,Ig3,ic)=0.; // Neumann BC: fill in BC's RHS on the ghostlines
			f1[grid](Ib1,Ib2,Ib3,ac)=a_inflow1;
		      }
		    else if( mg.boundaryCondition()(side,axis)==outflow1)
		      {
			f1[grid](Ig1,Ig2,Ig3,ic)=0.;  // Neumann BC: fill in BC's RHS on the ghostlines
			f1[grid](Ib1,Ib2,Ib3,ac)=a_outflow1; // Dirichlet BC
		      }
		    else if( mg.boundaryCondition()(side,axis)==outflow2)
		      {
			f1[grid](Ig1,Ig2,Ig3,ic)=0.;  // Neumann BC's: fill in BC's RHS on the ghostlines
			f1[grid](Ib1,Ib2,Ib3,ac)=a_outflow2; // Dirichlet BC
		      }
		    else if( mg.boundaryCondition()(side,axis)==noSlipWall || mg.boundaryCondition()(side,axis)==noSlipWall2)
		      {
			f1[grid](Ib1,Ib2,Ib3,ic)=i_wall;  // Dirichlet BC
			f1[grid](Ig1,Ig2,Ig3,ac)=0.;  // Neumann BC's: fill in BC's RHS on the ghostlines
		      }
		    else if( mg.boundaryCondition()(side,axis)==noSlipWall3)
		      {
			f1[grid](Ib1,Ib2,Ib3,ic)=i_wall2;  // Dirichlet BC
			f1[grid](Ig1,Ig2,Ig3,ac)=0.;  // Neumann BC's: fill in BC's RHS on the ghostlines
		      }
		    if( OrderOfAccuracy==4 )
		      {
			getGhostIndex(mg.gridIndexRange(),side,axis,Ig21,Ig22,Ig23,2); // Index's for 2nd ghostline
			if((mg.boundaryCondition()(side,axis)==inflow2)||(mg.boundaryCondition()(side,axis)==inflow1))
			  {
			    f1[grid](Ig21,Ig22,Ig23,ic)=0.; // Neumann BC's: fill in BC's RHS on the 2nd ghostline
			  }
			else if((mg.boundaryCondition()(side,axis)==outflow1)||(mg.boundaryCondition()(side,axis)==outflow2))
			  {
			    f1[grid](Ig21,Ig22,Ig23,ic)=0.;  // Neumann BC's: fill in BC's RHS on the 2nd ghostline
			  }
			else if( mg.boundaryCondition()(side,axis)==noSlipWall || mg.boundaryCondition()(side,axis)==noSlipWall2 || mg.boundaryCondition()(side,axis)==noSlipWall3)
			  {
			    f1[grid](Ig21,Ig22,Ig23,ac)=0.;  // Neumann BC's: fill in BC's RHS on the 2nd ghostline
			  }
		      }
		  }
	      }
	  }
	  cout << "Waiting for solver to finish...\n";
	  u=0.;  // for interative solvers.
	  solver.solve( u,f1 );   // solve the equations
	  cout << "Solver finished.\n";

	}

      else // 2) solve by false transients

	{
	  //  ***********************************************
	  cout << "\n *****************************************************************\n";
	  cout << " Solving Poisson-Boltzmann by false transients...\n";
	  cout << " *****************************************************************\n";

	  // Apply initial conditions
	  ForGrids(grid) // loop over the component grids
	  {
	    MappedGrid & mg = cg[grid]; // make a reference (alias) to save typing
	    getIndex(mg.indexRange(),I1,I2,I3);
	    u[grid](I1,I2,I3)=0.5; uold[grid](I1,I2,I3)=0.5;
	  }

	  printf("\nStriving for MAX ERROR = %e with TIME STEP = %e...\n", eps_u, dt_u);

	  t=0.; Deque.clear();

	  for( ft_step=1; ft_step<=StepMax; ft_step++ )
	    {
	      // Take a time step (Euler, Runge-Kutta, Multistep)
	      euler(t,dt_u_nondim,u,tmp11);
	      //rungeKutta4(t,dt_u_nondim,u,tmp11,tmp22,tmp33);
	      //linearMultiStep(t,dt_u_nondim,u,tmp11,tmp22,tmp33);
	      
	      // Update error (based on du/dt -> 0) every ErrorFreq
	      if( ft_step % ErrorFreq == 0)
		if ( (error_u=max(fabs(tmp11))) > eps_u) // continue if above tolerance
		  printf("\nMax dudt: %e (PB solver step: %i). Continuing...", error_u, ft_step);
// 		else if (NaN(tmp11)) 
// 		  abort(); // abort if NaN encountered (algorithm diverges)
		else // break if within tolerance
		  break;
	    }
	  
	  if (ft_step==StepMax)
	    printf("\nReached StepMax = %i and error still not within tolerance!\n", StepMax);
	  
	  if (Plot_intermediate) // plot intermediate results
	    {
	      pi.erase();
	      psp.set(GI_TOP_LABEL,sPrintF(buffer,"P-B solution"));  // set title
	      PlotIt::contour(pi, u(R0),psp );
	      pi.erase();
	      PlotIt::contour(pi, u(R1),psp );
	    }
	  
	  // ************************************************
	} // end if(!False_transients)

      if (Plot_intermediate) // plot intermediate results
	{
	  pi.erase();
	  psp.set(GI_TOP_LABEL,sPrintF(buffer,"P-B solution"));  // set title
	  PlotIt::contour(pi, u(R0),psp );
	  pi.erase();
	  PlotIt::contour(pi, u(R1),psp );
	}

 	  // save to HDF
 	  cout << "\nEXPORTING P-B solution to " << pbHdf << " ...\n";
	  u.put(db, "P-B solution");
	}
      db.unmount();
      
      //=========================================================================================
      //    Compute electrostatic quantities
      //=========================================================================================
      // Compute derivatives for calculation of forces, work only with the applied field, intrinsic field accounted for through the slip velocity
      //  Work with the entire grid rather than looping over component grids (equivalent)
      cout << "\nCalculating electric field quantities..." << endl;
      
      Ei=-u(R0).grad(); 
      E=-u(R1).grad(); // el. field strength E = -grad(phi_a)
      E.interpolate(); Ei.interpolate(); // interpolate values at interpolation points
      
      // Compute the magnitudes of 'E' vector fields (sqrt works only on mapped grid functions => loop over component grids)
      ForGrids(grid) // loop over the component grids
	ForDims(i)
	E_sq[grid] += SQR(E[grid](all,all,all,i));
      
      E_sq.setOperators(op); 
      E_sq.interpolate();     // interpolate E^2
      grad_E_sq=E_sq.grad();  // gradient of the squared magnitude of E
      grad_E_sq.interpolate();// interpolate grad(E^2)
      
      cout << "\n *** E & grad(E^2) ***";
      cout << "\n QUANTITY  " << setw(10) << "MIN" << setw(10) <<  "MAX";
      cout << "\n E:        " << setw(10) <<  min(fabs(E))  << setw(10) <<  max(fabs(E));
      cout << "\n GRAD_E_SQ:" << setw(10) <<  min(fabs(grad_E_sq))  << setw(10) <<  max(fabs(grad_E_sq)) << endl;

//       eval_Quadrupole(E,Q); // *** DEP QUADRUPOLE

      // Compute el. body force that  appears in Navier-Stokes for electroosmotic flow
      if (Electroosmotic)
	{
	  ForGrids(grid)
	  {
	    MappedGrid & mg = cg[grid];
	    getIndex(mg.indexRange(),I1,I2,I3); // loop over INTERIOR points
	    for(i=1; i<Dim+1; i++) 
	      {
		// Process all velocity components (uv, vc, wc), use Ex, Ey, Ez
		f2[grid](I1,I2,I3,i) = -ksqr*u_0hs/u_0*u[grid](I1,I2,I3,0)*E[grid](I1,I2,I3,i-1); // F = kappa^2 * phi_i * grad (phi_a) = - kappa^2 * phi_i * E
		// Process pressure
		f2[grid](I1,I2,I3,pc) -= ksqr*u_0hs/u_0*Ei[grid](I1,I2,I3,i-1)*E[grid](I1,I2,I3,i-1); // -div(F), div(F) = div(phi_i*grad(phi_a))=grad(phi_i)*grad(phi_a)+phi_i*div(grad(phi_a))
	      }
	  }
	}
      
    } // end if(Electric_field)
  time_PB=getCPU()-time_PB;
  
  time_NS=getCPU();


  if (Electroosmotic || Shear_driven || Pressure_driven || Momentum_transfer )
    {
      bool File_Exists=FileExists(nsHdf); // solved already & saved in HDF?
      HDF_DataBase db;
      
 //      if(File_Exists) // yes
// 	{
// 	  cout << "\nIMPORTING N-S solution from " << nsHdf << " ...\n";
// 	  db.mount(nsHdf,"R"); // mount, read-only
// 	  u2.get(db, "N-S solution"); // load from HDF
// 	}
//       else // no
// 	{

      if (!False_transients) // 1) solve by coefficient matrices
	{
	  //=========================================================================================
	  //    Set up the steady-state linearized incompressible Navier-Stokes (Stokes flow) solver
	  //=========================================================================================

	  CM_NS_Build(coeff2); // build standard coefficient matrix for Navier Stokes

	  //=========================================================================================
	  //    Solve Navier-Stokes equation
	  //=========================================================================================

	  cout << "\n *****************************************************************\n";
	  cout << " Solving for unperturbed velocity and pressure fields...\n";
	  cout << " *****************************************************************\n\n";

	  // initialize velocity
	  u2(R1D)=-u_0hs/u_0*E; // initialize velocity components to Helmhlotz-Smoluchowski slip velocity

	  // evaluate & assign the right-hand side of Navier-Stokes
	  ForGrids(grid) // loop over the component grids
	  {
	    MappedGrid & mg = cg[grid];
	    getIndex(mg.indexRange(),I1,I2,I3);
	    // BOUNDARIES
	    ForBoundary(side,axis)
	      {
		if( mg.boundaryCondition()(side,axis) > 0  )
		  {
		    getBoundaryIndex(mg.gridIndexRange(),side,axis,Ib1,Ib2,Ib3);
		    getGhostIndex(mg.gridIndexRange(),side,axis,Ig1,Ig2,Ig3,1); // ***ph032708: increase region by 1 line?
		    //	      getGhostIndex(mg.gridIndexRange(),side,axis,Ig_1,Ig_2,Ig_3,-1); // interior  line adjacent to the boundary
		    const realArray & normal  = mg.vertexBoundaryNormal(side,axis);
		    // 	      const realArray & tangent = mg.centerBoundaryTangent(side,axis);
		    // 	      const realArray & vertex  = mg.vertex();
		    switch ( mg.boundaryCondition()(side,axis) )
		      {
		      case inflow2:
			for(i=1; i<Dim+1; i++) f2[grid](Ig1,Ig2,Ig3,i)=0; // loop over velocity components (uc, vc, wc)
			f2[grid](Ib1,Ib2,Ib3,pc)=BC_NS.p_inflow2;
			break;
		      case inflow1:
			for(i=1; i<Dim+1; i++) f2[grid](Ig1,Ig2,Ig3,i)=0;
			f2[grid](Ib1,Ib2,Ib3,pc)=BC_NS.p_inflow1;
			break;
		      case outflow1:
			for(i=1; i<Dim+1; i++) f2[grid](Ig1,Ig2,Ig3,i)=0;
			f2[grid](Ib1,Ib2,Ib3,pc)=BC_NS.p_outflow1; // outlet pressure
			break;
		      case outflow2:
			for(i=1; i<Dim+1; i++) f2[grid](Ig1,Ig2,Ig3,i)=0;
			f2[grid](Ib1,Ib2,Ib3,pc)=BC_NS.p_outflow2; // outlet pressure
			break;
		      case noSlipWall:
			f2[grid](Ib1,Ib2,Ib3,uc)=-BC_NS.v_wall;
			for(i=2; i<Dim+1; i++) f2[grid](Ib1,Ib2,Ib3,i) = 0; // loop over (vc, wc)
			for(i=1; i<Dim+1; i++) f2[grid](Ig1,Ig2,Ig3,pc) += normal(Ib1,Ib2,Ib3,i-1)*f2[grid](Ib1,Ib2,Ib3,i); //=0 in MOFT;
			break;
 		      case noSlipWall2:
 			f2[grid](Ib1,Ib2,Ib3,uc)=BC_NS.v_wall;
 			for(i=2; i<Dim+1; i++) f2[grid](Ib1,Ib2,Ib3,i) = 0; // loop over (vc, wc)
			for(i=1; i<Dim+1; i++) f2[grid](Ig1,Ig2,Ig3,pc) += normal(Ib1,Ib2,Ib3,i-1)*f2[grid](Ib1,Ib2,Ib3,i);  //=0 in MOFT;
			break;
		      case noSlipWall3:
			f2[grid](Ib1,Ib2,Ib3,uc)=BC_NS.v_wall;
			for(i=2; i<Dim+1; i++) f2[grid](Ib1,Ib2,Ib3,i) = 0; // loop over (vc, wc)
			for(i=1; i<Dim+1; i++) f2[grid](Ig1,Ig2,Ig3,pc) += normal(Ib1,Ib2,Ib3,i-1)*f2[grid](Ib1,Ib2,Ib3,i);  //=0 in MOFT;
			break;
		      }

		    if( OrderOfAccuracy==4 )
		      {
			getGhostIndex(mg.gridIndexRange(),side,axis,Ig21,Ig22,Ig23,2); // Index's for 2nd ghostline
			if( (mg.boundaryCondition()(side,axis)==inflow2)||(mg.boundaryCondition()(side,axis)==inflow1) )
			  {
			    for(i=1; i<Dim+1; i++) f2[grid](Ig21,Ig22,Ig23,i)=0; // Neumann BC's: fill in BC's RHS on the 2nd ghostline
			  }
			else if( (mg.boundaryCondition()(side,axis)==outflow1)||(mg.boundaryCondition()(side,axis)==outflow2))
			  {
			    for(i=1; i<Dim+1; i++) f2[grid](Ig21,Ig22,Ig23,i)=0;
			  }
			else if( mg.boundaryCondition()(side,axis)==noSlipWall || mg.boundaryCondition()(side,axis)==noSlipWall2 || mg.boundaryCondition()(side,axis)==noSlipWall3)
			  {
			    for(i=1; i<Dim+1; i++) f2[grid](Ig21,Ig22,Ig23,pc) += normal(Ib1,Ib2,Ib3,i-1)*f2[grid](Ib1,Ib2,Ib3,i); //=0 in MOFT;
			  }
		      }
		  }
	      }
	  }

	  // Solve
	  solver.setCoefficientArray( coeff2 );   // supply coefficients to solver (Oges will keep a reference!)
	  if (Electroosmotic || Shear_driven || Pressure_driven)
	    {
	      cout << "Waiting for solver to finish...\n";
	      solver.solve( u2,f2 );   // solve the equations
	      cout << "Solver finished.\n";
	    }
	}

      else // 2) solve by false transients

	{
	  // ***********************************************
	  cout << "\n\n *****************************************************************\n";
	  cout << " Solving Navier-Stokes by false transients...\n";
	  cout << " *****************************************************************\n";

	  // Apply initial conditions
	  ForGrids(grid) // loop over the component grids
	  {
	    MappedGrid & mg = cg[grid]; // make a reference (alias) to save typing
	    getIndex(mg.indexRange(),I1,I2,I3);
	    u2[grid](I1,I2,I3)=0.; u2old[grid](I1,I2,I3)=0.;
	  }

	  printf("\nStriving for MAX ERROR = %e with TIME STEP = %e...\n", eps_v, dt_v);

	  t=0.; Deque.clear();
	  for( ft_step=1; ft_step<=StepMax; ft_step++ )
	    {
	      // Take a time step (Euler, Runge-Kutta, Multistep)
	      euler(t,dt_v_nondim,f2,u2,tmp1);
	      //rungeKutta4(t,dt_v_nondim,f2,u2,tmp1,tmp2,tmp3);
	      //linearMultiStep(t,dt_v_nondim,f2,u2,tmp1,tmp2,tmp3);

	      // Update error (based on du/dt -> 0) every ErrorFreq
	      if( ft_step % ErrorFreq == 0)
		if ( (error_v=max(fabs(tmp1))) > eps_v) // continue if above tolerance
		  printf("\nMax dvdt: %e (NS solver step: %i). Continuing...", error_v, ft_step);
	      //else if (NaN(tmp1)) // too slow, comment unless necessary 
	      //abort(); // abort if NaN encountered (algorithm diverges)
		else // break if within tolerance
		  break;
	    }
	  
	  if (ft_step==StepMax)
	    printf("\nReached StepMax = %i and error still not within tolerance!\n", StepMax);
	  
	  if (Plot_intermediate) // plot intermediate results
	    {
	      pi.erase();
	      psp.set(GI_TOP_LABEL,sPrintF(buffer,"N-S solution by false transients"));  // set title
	      PlotIt::contour(pi, v,psp );
	    }

	  // ************************************************

	} // end if(!False_transients)

// #ifdef USE_MPI
// if(rank==0) // only root does I/O...
//  {
// #endif 
//       // save to HDF
//   db.mount(nsHdf,"I"); // mount, init
//   cout << "\nEXPORTING N-S solution to " << nsHdf << " ...\n";
//   u2.put(db, "N-S solution");
//   db.unmount();

// #ifdef USE_MPI
//  }

// if(rank>0) // slaves
// {
//   MPI::COMM_WORLD.Barrier(); // ...the rest will wait until root finishes I/O
//   db.mount(nsHdf,"R"); // mount, read-only
//   u2.get(db, "N-S solution"); // load from HDF
//   db.unmount();
// #endif 

      // Reference velocity & pressure arrays
      u_fluid_base=u2(R1D); // make a shallow copy of unperturbed velocity field (for visulization), remains unchanged
      // note: shalow and deep (e.g updateToMatchGridFunction rather than '=') copies give different results
      u_fluid.reference(u2(R1D)); // permanent reference to fluid velocity field (to be periodically updated in dynamics section below)
      p_base=u2(R0); // base pressure

    } // end if(Electroosmotic || Shear_driven || Pressure_driven || Momentum_transfer)
    
  time_NS=getCPU()-time_NS;
  time_TD=getCPU();

ofstream history;
#ifdef USE_MPI
if(rank==0) // only root does I/O...
{
#endif 
  if(Export_vmd) // if VMD files requested...
    {
      export_psf("Output/CHAIN_" + outputSuffix + ".psf"); // export chain topology
      init_history("Output/HISTORY_" + outputSuffix + ".dlpolyhist",history); // initialize history
    }
#ifdef USE_MPI
}
#endif 

  if (Tracers)
    {
      //=========================================================================================
      //  1. Tracer Dynamics (for comparison with Colloid Dynamics)
      //=========================================================================================
      // Note that tracers experience only Stokes drag and electrical forces!

      cout << "\n *****************************************************************\n";
      cout << " Particle tracking (tracer particles)\n";
      cout << " *****************************************************************\n\n";
      cout << "\n Bilinear interpolation and 4th order Runge-Kutta integration...\n\n";

      doubleArray 
	positionToInterpolate_k1(NoOfParticles,Dim), 
	up_interpolated_k1(NoOfParticles,Dim),
	positionToInterpolate_k2(NoOfParticles,Dim), 
	up_interpolated_k2(NoOfParticles,Dim),
	positionToInterpolate_k3(NoOfParticles,Dim), 
	up_interpolated_k3(NoOfParticles,Dim),
	positionToInterpolate_k4(NoOfParticles,Dim), 
	up_interpolated_k4(NoOfParticles,Dim);
      
      // Determine particle velocity 
      up = u_fluid - (Electrophoretic)*mu_p*E_0/u_0 * E - (Dielectrophoretic)*dep_corr*epsilon*rad_p*rad_p*E_0*E_0/6/eta/L/u_0 * grad_E_sq; // elosm + elphoretic + dielphoretic contributions, respectively

      // 4th order Runge-Kutta integration
      position_tracer=position;

      t=0;
      int InterpT1, InterpT2, InterpT3, InterpT4; // interpolation return codes for tracers
      for (step=0; step<NoOfTimeStepsT-1; step++)
	{
	  positionToInterpolate_k1=position_tracer; // reference the current 2D position matrix of a 3D array (hard/shallow copy)
	  InterpT1=interpolatePoints(positionToInterpolate_k1,up,up_interpolated_k1, Dim); // Interpolate (u,v,w)
	  positionToInterpolate_k2=position_tracer+dtT/2*up_interpolated_k1;
	  InterpT2=interpolatePoints(positionToInterpolate_k2,up,up_interpolated_k2, Dim);
	  positionToInterpolate_k3=position_tracer+dtT/2*up_interpolated_k2;
	  InterpT3=interpolatePoints(positionToInterpolate_k2,up,up_interpolated_k3, Dim);
	  positionToInterpolate_k4=position_tracer+dtT*up_interpolated_k3;
	  InterpT4=interpolatePoints(positionToInterpolate_k2,up,up_interpolated_k4, Dim);
      
	  ForParticlesAndDims(particle,dim)  // (u,v,w) -> (x,y,z)
	    // Integrate velocity field to obtain new position
	    //position_tracer(particle,dim) += dtT_nondim/6*(up_interpolated_k1(particle,dim)+2*up_interpolated_k2(particle,dim)+2*up_interpolated_k3(particle,dim)+up_interpolated_k4(particle,dim)); // 4th order Runge-Kutta
	    position_tracer(particle,dim) += dtT_nondim*up_interpolated_k1(particle,dim); // Euler
	  t+=dtT; // advance in time
	}
    } // end if(Tracers)
  time_TD=getCPU()-time_TD;

  time_PD=getCPU();

  //=========================================================================================
  //  2. Colloidal/Polymer Dynamics
  //=========================================================================================
  // HYBRID SCHEME: Modified velocity-Verlet/Heun's predictor-corrector (particle dynamics) coupled with Navier-Stokes (continuum dynamics)

  cout << "\n *****************************************************************\n";
  cout << " Particle tracking (colloidal particles)\n";
  cout << " *****************************************************************\n";
  
  t=0.; // reset time

  doubleArray md_force(NoOfParticles,Dim),
    hydrodynamic_force(NoOfParticles,Dim), hydrodynamic_force_ini(NoOfParticles,Dim),
    nonhydrodynamic_force(NoOfParticles,Dim), nonhydrodynamic_force_ini(NoOfParticles,Dim),
    electrical_force(NoOfParticles,Dim),
    wall_force(NoOfParticles,Dim);
   double * position_pointer = position.getDataPointer(); // store a pointer to the 1st element of the array
   double * force_pointer = md_force.getDataPointer();
int InterpC_vel, InterpC_E, InterpC_grad_E_sq, InterpC_Q, InterpC_W, InterpC_D; // interpolation return codes for colloids
  md_force=hydrodynamic_force=hydrodynamic_force_ini=nonhydrodynamic_force=nonhydrodynamic_force_ini=electrical_force=wall_force=0;  // initialize to zero

  if(Momentum_transfer) // compare particle radius and grid spacing (assume cg[0] is a background (representative) grid)
    ForDims(i)
      cout << "Ratio of particle radius to the background grid spacing in the " << i << "-direction: " << rad_p/cg[0].gridSpacing(i)/L << ".\n"; // ideally should be ~ 1

  //=========================================================================================
  //    Set up interaction tensor & stochastic tensor for BD w/ preaveraged Oseen
  //=========================================================================================
  
  // 1) Set up preaveraged Oseen tensor (a symmetric, Toeplitz matrix => stored in a compact vector)
  doubleArray POseen(NoOfParticles), Sigma(NoOfParticles, NoOfParticles);
  double 
    C_Oseen=1./(pow(6*PI*PI*PI,.5)*eta*q0), // preavg Oseen constant
    Sigma_pref=pow(2.*Dim/dt_nondim, 0.5), // Sigma prefactor for uniform random numbers
 //Sigma_pref=pow(2./dt_nondim, 0.5), // Sigma prefactor for Gaussian random numbers
    Sigma_reduced=sqrt(Diff), // reduced Sigma
    POseen_0=u_0/f_0, // scale for POseen
    Sigma_0=L*pow(Tau,-0.5); // scale for Sigma
  POseen=0.; Sigma=0.; // initialize

  POseen(0)=1./FRICTION; // self-interaction
  for (particle=1; particle<NoOfParticles; particle++) // cross-interactions
    POseen(particle)=C_Oseen/sqrt(particle);

  // 2) Set up stochastic matrix (to satisfy fluctuation-dissipation)
    for (i=0; i<NoOfParticles; i++) // Cholesky decomposition
    {
      for (j=i; j<NoOfParticles; j++)
	{
	  for (SUM=KB*temp*POseen(abs(i-j)), k=i-1; k>=0; k--) SUM -= Sigma(i,k)*Sigma(j,k);
	  if (i==j) {
	    if (SUM <= 0.) cout << "\nCholesky decomposition failed. Matrix not positive definite!\n";
	    Sigma(i,i)=sqrt(SUM);
	  } else Sigma(j,i)=SUM/Sigma(i,i);
	}
    }

  // 3) Non-dimensionalize
  POseen/=POseen_0;
  Sigma*=Sigma_pref/Sigma_0; // ... and multiply by Sigma prefactor
  Sigma_reduced*=Sigma_pref/Sigma_0;

  if(Debug & 1) 
    {POseen.display("\nPOseen:"); Sigma.display("\nSigma:");}

  // 4) Set self-interaction to zero because it is included separately in the equations of motion
  POseen(0)=0.; // computational implementation uses self-interaction O(0)=0

  time_PDINI=getCPU()-time_PD;

#ifdef USE_MPI
  ForSimsMPI(sim)
#else
  ForSims(sim) // LOOP OVER REALIZATIONS (OUTER LOOP)
#endif
  {
    cout << "\nREALIZATION NO. " << sim << " OUT OF " << NoOfSims << "\n";
    time0=getCPU();

    //=========================================================================================
    //    Seed random number generator
    //=========================================================================================

    // Seed rnd (or use default seed=1)
    srand(Seed+sim); // Initialize random number generator with a seed > 1, different for each simulation
    //srand((unsigned)(time(NULL))); // Random seeding => non-repeatable

    //=========================================================================================
    //    Re-generate CONFIG (since shared resource, use Centralized MUTEX in parallel)
    //=========================================================================================
    // Note: New CONFIG needed only for random walk configurations. CONTROL and FIELD stay the same.
    //       No need to re-run dlpoly_initialize() or add wall atoms again.

//     // CRITICAL SECTION (START)
// #define CS								\
//     do{									\
//       cout << "\nRe-generating CONFIG file...\n";			\
//       run_Sarw(dat);							\
//       import_CONFIG(position, velocity);				\
//       shift_position(position, Position_rescale);                       \
//       rescale_position(position, velocity, position_ini, Position_rescale, Velocity_rescale); \
//     } while (0)
//     // CRITICAL SECTION (END)

// #define REQUEST 1
// #define REPLY 2
// #define RELEASE 3

//     if(Mode!=2) // For random walk configurations only
//       {

// #ifdef USE_MPI
// 	queue<int> Wait; // wait queue
// 	int token;
// 	MPI::Status status;
// 	if(rank==0) // IF MASTER...
// 	  {
// 	    // 1) Process [request,release] pairs from all the others
// 	    for (int w=0; w<2*size-2; w++)
// 	      {
// 		MPI::COMM_WORLD.Recv(&token, 1, MPI::INT, MPI::ANY_SOURCE, MPI::ANY_TAG, status);
	
// 		if (token==REQUEST) // if request...
// 		  {
// 		    if (Wait.empty()) // push & send reply if queue empty 
// 		      {
// 			Wait.push(status.Get_source());
// 			MPI::COMM_WORLD.Send(&token, 1, MPI::INT, status.Get_source(), REPLY);
// 		      }
// 		    else // else push
// 		      Wait.push(status.Get_source());
// 		  }
// 		else if (token==RELEASE) // if release...
// 		  {
// 		    Wait.pop(); // pop & send reply to the next in queue
// 		    MPI::COMM_WORLD.Send(&token, 1, MPI::INT, Wait.front(), REPLY);
// 		  }
// 	      }
// 	    // 2) Process yourself (work also)
// 	    CS; // run CRITICAL SECTION
// 	  }
// 	else // IF SLAVE...
// 	  {
// 	    token=REQUEST;
// 	    MPI::COMM_WORLD.Send(&token, 1, MPI::INT, 0, MPI::ANY_TAG); // send request
// 	    MPI::COMM_WORLD.Recv(&token, 1, MPI::INT, 0, REPLY, status); // if reply received...
// 	    CS; // CRITICAL SECTION
// 	    token=RELEASE;
// 	    MPI::COMM_WORLD.Send(&token, 1, MPI::INT, 0, MPI::ANY_TAG); // send release
// 	  }
// #else
// 	CS;  // run CRITICAL SECTION
// #endif

//       }

    //=========================================================================================
    //  Calculate initial velocity & forces (forces only needed for LD, though)
    //=========================================================================================
    // Note: needs to be inside the loop over realizations if CONFIG regenerated for each realization

    bool ProductionPhase=!NoOfEquilSteps;

    // Interpolate fluid velocity, E & grad_E_sq from neighboring grid points (bi/tri-linear interpolation) to initial particle positions
    positionToInterpolate=position_ini;  // initial position of the particles
    InterpC_vel=interpolatePoints(positionToInterpolate,u_fluid,u_fluid_interpolated, Dim);  // initial fluid velocity at particle positions (obtained from unperturbed field) => represents disturbance in fluid flow due to the particles
    // el. field turned off during equilibration
    if (ProductionPhase) InterpC_E=interpolatePoints(positionToInterpolate,E,E_interpolated, Dim); // electric field at particle positions
    if (ProductionPhase) InterpC_grad_E_sq=interpolatePoints(positionToInterpolate,grad_E_sq,grad_E_sq_interpolated, Dim);
    //    if (ProductionPhase) InterpC_Q=interpolatePoints(positionToInterpolate,Q,Q_interpolated, Dim); // *** DEP QUADRUPOLE

    // INITIAL VELOCITY OF PARTICLES
    // Note: Alternatively, initial velocities of particles can be imported from CONFIG / CFGMIN (see above)
    velocity_ini=0.;

//   // *** Ahlrich's experiments *** (START)
//    velocity_ini=1.; // Ahlrich's experiment
//   // *** Ahlrich's experiments *** (END)

    // 1) Set equal to initial velocity of fluid
    //velocity_ini=u_fluid_interpolated - (Electrophoretic)*mu_p*E_0/u_0 * E_interpolated - (Dielectrophoretic)*dep_corr*epsilon*rad_p*rad_p*E_0*E_0/6/eta/L/u_0 * grad_E_sq_interpolated;
    // 2) Set individual components manually (for testing)
    //velocity_ini(all,0)=1e0; // set x-component
    //velocity_ini(all,1)=0; // set y-component
    //velocity_ini(all,2)=0;  // set z-component
    // 3) Draw velocities from Maxwell-Boltzmann distribution (Gaussian with std=sqrt(kb*T/m), mean=0)
    //ForParticlesAndDims(particle,dim)
    //  velocity_ini(particle,dim)=box_muller(0,u_0t/u_0);

    // INITIAL FORCES ON PARTICLES
    if(Molecular_dynamics) // compute MD forces
      {
#ifdef USE_DLPOLY
	dlpoly_force_eval_(position_pointer, force_pointer); // DLPOLY: evaluate MD force (phase 2 of 3), shift the pointer accordingly (arrays stored in Fortran column-order-fashion)
#else
	forceEval(position,md_force,fed);
#endif
      }

    if(Wall) ann(kdTree,position,Dwall,Fwall,domain_min,domain_max,ProductionPhase); // compute wall forces
    rndEval(Random,Sigma,Sigma_reduced); // compute Brownian forces

    // 1) initial nonhydrodynamic forces
    ForParticlesAndDims(particle,dim)
      {
	nonhydrodynamic_force_ini(particle,dim) =
	  Fmd*md_force(particle,dim)
	  +Fep*E_interpolated(particle,dim)
	  +Fdep*grad_E_sq_interpolated(particle,dim)
	  +Fdep_q*Q_interpolated(particle,dim)
	  +Wall*Fwall(particle,dim); // (MD+EP+DEP+Wall contributions)
      }
	
    // 2) initial hydrodynamic forces
    hydrodynamic_force_ini(particle,dim)= 
      -Zeta*(velocity(particle,dim)-u_fluid_interpolated(particle,dim))
      +Zeta*Random(particle,dim);


    //=========================================================================================
    //    Initialize position, velocity and forces
    //=========================================================================================
    position=position_ini;
    velocity=velocity_ini;
    u_fluid=u_fluid_base;
    hydrodynamic_force=hydrodynamic_force_ini;
    nonhydrodynamic_force=nonhydrodynamic_force_ini;

    //=========================================================================================
    //    Initialize polymer properties
    //=========================================================================================
    ppInit(PPD,PPS,PPSIM,x0,v0,norm);
    int exportstep=0, TransStatus=0;

    time_REALINI+=(getCPU()-time0);
    //=========================================================================================
    //    LOOP OVER TIME STEPS
    //=========================================================================================
    cout << "\nLooping over time steps...\n";
    //    ForSteps(step)  // LOOP OVER TIME STEPS (INNER LOOP)
    step=0; while (TransStatus < 2) // *** TRANSLOCATION --- LOOP OVER TIME STEPS (INNER LOOP)
    {
      time0=getCPU();
      //cout << "\nStep: " << step+1 << " out of " << NoOfTimeSteps;
      //DrawProgressBar(100,(step+1.)/(NoOfTimeSteps-1.));
      
      ProductionPhase = (step >= NoOfEquilSteps);
      if (ProductionPhase && (step-NoOfEquilSteps) % ExportFreq == 0) // if equilibration stage over and time to export...
	{
	  ppUpdate(PPD,PPS,PPSIM,exportstep,position,velocity,x0,v0,N0,norm,TransStatus,domain_min,domain_max,com,vcom,eev); // evaluate dynamic polymer properties
	  exportstep++;
	}
      
      if (Debug & 1)
	cout << "\n[Step, TransStatus, TransTime] = [" <<  step << ", " << TransStatus << ", " << TRANSLOC(0) << "]";

      time_PPEVAL+=(getCPU()-time0);
      time0=getCPU();

      if (BD) // Brownian dynamics
	{
	  // *********************************** BEGIN: Brownian dynamics solver **********************************
	  // UPDATE POSITION
	  ForParticlesAndDims(particle,dim)
	    position(particle,dim) += dt_nondim*velocity(particle,dim);

	  // Interpolate fields
	  positionToInterpolate=position;
	  if(Debug & 1) positionToInterpolate.display("\nPositionToInterpolate: ");

	  if(Electroosmotic || Shear_driven || Pressure_driven || Momentum_transfer) 
	    InterpC_vel=interpolatePoints(positionToInterpolate,u_fluid,u_fluid_interpolated,Dim);
	  if(ProductionPhase && Electrophoretic)  // el. field turned off during equilibration
	    InterpC_E=interpolatePoints(positionToInterpolate,E,E_interpolated, Dim);
	  if(ProductionPhase && Dielectrophoretic) // el. field turned off during equilibration
	    {
	      InterpC_grad_E_sq=interpolatePoints(positionToInterpolate,grad_E_sq,grad_E_sq_interpolated, Dim);
	      //	      InterpC_Q=interpolatePoints(positionToInterpolate,Q,Q_interpolated, Dim); // *** DEP QUADRUPOLE
	    }
	  if(Wall)
	    {
	      ann(kdTree,position,Dwall,Fwall,domain_min,domain_max,ProductionPhase); // compute Fwall
	    }
	      
	  // Update MD forces
	  if(Molecular_dynamics)
	    {
	      time_DLPOLY_START=getCPU();
#ifdef USE_DLPOLY
	      dlpoly_force_eval_(position_pointer, force_pointer); // DLPOLY: evaluate MD forces (phase 2 of 3)
#else
	      forceEval(position,md_force,fed);
#endif
	      time_DLPOLY+=(getCPU()-time_DLPOLY_START);
	    }
	  
	  // Update RND forces
	  // 1) Generate rnd numbers (use Box-Muller transform: uniform->Gaussian(0,1))
	  ForParticlesAndDims(particle,dim)
	    {
	      // Set Random to '0' if random forces are to be turned off! 
	      Random(particle,dim)=ranf(); //box_muller(0,1); // ranf() or box_muller(0,1) (Gaussian requires smaller time steps!)
	      //cout << endl << Random(particle,dim); // print them out so we can check if the distribution is correct
	    }

	  // 2) Calculate Brownian velocity contribution (sigma term in BD equation), overwrite Random array to save space
	  if (Momentum_transfer && step % MTFreq != 0 && P_Oseen) // if not resolving Navier-Stokes (and P_Oseen is ON)
	    ForParticlesReverse(particle) // use reverse loop over particles so that Random array can be overwritten
	      ForDims(dim)
	      {
		Random(particle,dim)=Sigma(particle,particle)*Random(particle,dim); // diagonal term ...
		for (particle2=0; particle2<particle; particle2++)
		  Random(particle,dim)+=Sigma(particle,particle2)*Random(particle2,dim); // ... plus off-diagonal terms
	      }
	  else
	    Random*=Sigma_reduced;

	  ForParticlesAndDims(particle,dim)
	    {
	      nonhydrodynamic_force(particle,dim) =
		+Fmd*md_force(particle,dim)
		+Fep*E_interpolated(particle,dim)
		+Fdep*grad_E_sq_interpolated(particle,dim)
		+Fdep_q*Q_interpolated(particle,dim)
	        +Wall*Fwall(particle,dim); // (MD+EP+DEP+Wall contributions)

	      // UPDATE VELOCITY
	      velocity(particle,dim) = u_fluid_interpolated(particle,dim) + Random(particle,dim) + Inv_zeta*nonhydrodynamic_force(particle,dim); // reduced BD (no HI)
 	      velocity(0,dim)*=ProductionPhase; // *** TETHER 1st BEAD DURING EQUILIBRATION
	    }
	  if (Momentum_transfer && step % MTFreq != 0 && P_Oseen) // if not resolving Navier-Stokes (and P_Oseen is ON) ...
	    ForParticlesAndDims(particle,dim) // ... add cross-interactions
	      ForParticles2(particle2)
	      velocity(particle,dim) += POseen(abs(particle-particle2))*nonhydrodynamic_force(particle2,dim);

	  ForParticlesAndDims(particle,dim) //!Not needed if nonhydrodynamic force transferred instead!
	    {
	      // calculate hydrodynamic forces
	      hydrodynamic_force(particle,dim)=
		-Zeta*(velocity(particle,dim)-u_fluid_interpolated(particle,dim)) \
	      	+Zeta*Random(particle,dim);
	    }
	  if (Debug & 1)
	    {
	      cout << "\n===== DEBUG INFO: START =====";
	      Random.display("\nBrownian velocity:");
	      u_fluid_interpolated.display("\nU_fluid_interpolated:");
	      nonhydrodynamic_force.display("\nNonhydrodynamic force:");
	      hydrodynamic_force.display("\nHydrodynamic force:");
	      velocity.display("\nVelocity:");
	      cout << "\ndt_nondim: " << dt_nondim << endl;
	    }
	  
	  // *********************************** END: Brownian dynamics solver **********************************
	}
      else // Langevin dynamics
	{
	  // *********************************** BEGIN: Langevin dynamics solver **********************************
	  // r-RESPA multiple-timestep integrator (Tuckerman, 1992; pseudocode in: Molecular Modelling, Principles
	  // and applications, A.R. Leach, 2nd edition, p.364)
	  //
	  // Net force split (linear decopmosition) into slow and fast forces, in this case F=F1+F2+F3+F4, each integrated
	  // with its own time step
	  //
	  // Reduces to velocity Verlet for N1=N2=...Nn=1

	  ForParticlesAndDims(particle,dim) 
	    velocity(particle,dim) += respa0*nonhydrodynamic_force(particle,dim);
	  
	  for (int j1=0; j1<N1; j1++)
	    {
	      ForParticlesAndDims(particle,dim) 
		velocity(particle,dim) += respa1*hydrodynamic_force(particle,dim); // predict velocity for F1
	      
	      ForParticlesAndDims(particle,dim)
		position(particle,dim) += dt_nondim*velocity(particle,dim); // update position
	      
	      // Interpolate fluid velocity, E quantities & wall forces from neighboring grid points (bi/tri-linear interpolation)
	      positionToInterpolate=position;
	      if(Debug & 1) positionToInterpolate.display("\nPositionToInterpolate: ");

	      if(Electroosmotic || Shear_driven || Pressure_driven || Momentum_transfer) 
		InterpC_vel=interpolatePoints(positionToInterpolate,u_fluid,u_fluid_interpolated,Dim);
	      if(ProductionPhase && Electrophoretic) 
		InterpC_E=interpolatePoints(positionToInterpolate,E,E_interpolated, Dim);
	      if(ProductionPhase && Dielectrophoretic) 
		{
		  InterpC_grad_E_sq=interpolatePoints(positionToInterpolate,grad_E_sq,grad_E_sq_interpolated, Dim);
		  //		  InterpC_Q=interpolatePoints(positionToInterpolate,Q,Q_interpolated, Dim); // *** DEP QUADRUPOLE
		}
	      if(Wall)
		{
		  ann(kdTree,position,Dwall,Fwall,domain_min,domain_max,ProductionPhase); // compute Fwall
		}
	      rndEval(Random,Sigma,Sigma_reduced); // compute Frnd

	      ForParticlesAndDims(particle,dim)
		{
		  // calculate hydrodynamic forces (F1)
		  hydrodynamic_force(particle,dim)=
		    -Zeta*(velocity(particle,dim)-u_fluid_interpolated(particle,dim)) \
		    +Zeta*Random(particle,dim);
		  // correct velocity for F1
		  velocity(particle,dim) += respa1*hydrodynamic_force(particle,dim);
		}
	    }
	  
	  // Add pre-averaged Oseen tensor if needed
	  if (Momentum_transfer && step % MTFreq != 0 && P_Oseen) // if not resolving Navier-Stokes (and P_Oseen is ON) ...
	    ForParticlesAndDims(particle,dim) // ... add cross-interactions
	      ForParticles2(particle2)
	      nonhydrodynamic_force(particle,dim) += Zeta*POseen(abs(particle-particle2))*nonhydrodynamic_force(particle2,dim);
	  
	  // Update MD forces if needed
	  if(Molecular_dynamics)
	    {
	      time_DLPOLY_START=getCPU();
#ifdef USE_DLPOLY
	      dlpoly_force_eval_(position_pointer, force_pointer);
#else
	      forceEval(position,md_force,fed);
#endif
	      time_DLPOLY+=(getCPU()-time_DLPOLY_START);
	    }		  
	  
	  ForParticlesAndDims(particle,dim) 
	    {
	      // calculate nonhydrodynamic forces (F2)
	      nonhydrodynamic_force(particle,dim) =
		Fmd*md_force(particle,dim) +
		Fep*E_interpolated(particle,dim) + 
		Fdep*grad_E_sq_interpolated(particle,dim) +
		Fdep_q*Q_interpolated(particle,dim) +
		Wall*Fwall(particle,dim);
	      
	      // correct velocity for F2
	      velocity(particle,dim) += respa0*nonhydrodynamic_force(particle,dim);
	    }

	  if (Debug & 1)
	    {
	      cout << "\n===== DEBUG INFO: START =====";
	      Random.display("\nBrownian velocity:");
	      u_fluid_interpolated.display("\nU_fluid_interpolated:");
	      nonhydrodynamic_force.display("\nNonhydrodynamic force:");
	      hydrodynamic_force.display("\nHydrodynamic force:");
	      velocity.display("\nVelocity:");
	    }
	  // *********************************** END: Langevin dynamics solver **********************************
	}
      time_INTEG+=(getCPU()-time0);

      // *********************************** BEGIN: Momentum transfer ***************************************
      if( Momentum_transfer )
	{
	  time0=getCPU();
	  if( step % MTFreq == 0) // resolve Navier-Stokes equation
	    {
	      // 2) Transfer momentum flux (force density, i.e. stress tensor) to neighboring grid points (bi-linear weighting function)
	      force_Transfered=0.; // reinitialize the force field
 	      if (BD)  //distribute the hydrodynamic force back to fluid
		forceToExtrapolate=-hydrodynamic_force*NS_rescale;
	      //forceToExtrapolate=nonhydrodynamic_force*NS_rescale; // BD: F(hydro) = -F(nonhydro) (=> no velocity disturbance if F(nonhydro)=0), but '-' sign produces instability when HI on, why? (Probably because non-hydro force (all its component forces) should have an opposite sign.)
	      else
		forceToExtrapolate=-hydrodynamic_force*NS_rescale;
	      //	      forceToExtrapolate=-1; // *** Stokeslet test
// 	      forceToExtrapolate(1,all)=1; // *** dipole test
	      
	      // call modified interpolatePoints & save updated force field in force_Transfered
	      // Note: within the following call to interpolatePoints, forces are divided by cell volumes in interpPoints.C
	      int interp2=interpolatePoints(positionToInterpolate,forceToExtrapolate,force_Transfered,Range(0,Dim-1));	
	      //	      int interp2=interpolatePoints(positionToInterpolate,forceToExtrapolate,force_Transfered,R0); // x-component only: *** Stokeslet test or dipole test	

// 	      // ************************* LLNS : START *************************
//  	      // Simplified Landau-Lifshitz stochastic tensor (assumes uniform grid => Vc constant, cell-centered grid not needed)
// 	      // *************************
// 	      ForGrids(grid) // loop over the component grids
// 	      {
// 		MappedGrid & mg = cg[grid];
// 		getIndex(mg.indexRange(),I1,I2,I3,2);
// 		For3I(i1,i2,i3,I1,I2,I3)
// 		  for (int ind=0; ind<5; ind++) // symmetric dyad => 6 random numbers needed
// 		    force_LL[grid](i1,i2,i3,ind)=ranf();
// 		// If simulating pure LLNS (no force transfer), change "+=" -> "=" in the 3 lines below
// 		force_Transfered[grid](I1,I2,I3,0)+=a_ll*(b_ll*force_LL[grid].x(I1,I2,I3,0) +      force_LL[grid].y(I1,I2,I3,3) +      force_LL[grid].z(I1,I2,I3,4));
// 		force_Transfered[grid](I1,I2,I3,1)+=a_ll*(     force_LL[grid].x(I1,I2,I3,3) + b_ll*force_LL[grid].y(I1,I2,I3,1) +      force_LL[grid].z(I1,I2,I3,5));
// 		force_Transfered[grid](I1,I2,I3,2)+=a_ll*(     force_LL[grid].x(I1,I2,I3,4) +      force_LL[grid].y(I1,I2,I3,5) + b_ll*force_LL[grid].z(I1,I2,I3,2));
// 	      }
// 	      // ************************* LLNS : END *************************

		if (Debug & 1)
		{
		  // Check that force has been correctly distributed (for 1 point force)
		  SUM=0.;
		  forceToExtrapolate.display("\nForce to extrapolate:");
		  printf("\n(a_LL, b_LL) = (%e, %e)\n", a_ll, b_ll);
		  ForGrids(grid) // loop over the component grids
		  {
		    //force_Transfered[grid].display("\nForce transfered:");
		    MappedGrid & mg = cg[grid];
		    getIndex(mg.indexRange(),I1,I2,I3);
		    For3I(i1,i2,i3,I1,I2,I3)
		      ForDims(dim) 
		      {
			SUM+=force_Transfered[grid](i1,i2,i3,dim);
		      }
		  }
		  //cout << "\nTotal force to extrapol: " << sum(forceToExtrapolate);
		  //cout << "\nTotal force distributed: " << SUM; // should equal the sum of components of forceToExtrapolate, divided by cell volumes
		  //cout << "\nRel. error [%]:          " << abs(SUM-sum(forceToExtrapolate))/abs(sum(forceToExtrapolate)) * 100;
		  double tolerance=1e-9;
		  abs(SUM-sum(forceToExtrapolate)) < tolerance ? cout << "\nForce transfer: within tolerance\n" : cout << "\nForce transfer: NOT within tolerance\n"; // this check needs division by cell volumes in interppoints.C to be omitted
		  cout << "\n===== DEBUG INFO: END =====";
		}

	      time_MTRANSFER+=(getCPU()-time0);
	      time0=getCPU(); 
	      // *********************************** END: Momentum transfer *****************************************
	      // *********************************** BEGIN: Update RHS *****************************************
	      // apply divergence to the rhs of poisson-pressure eqn (using distributive property)
	      div_force_Transfered=force_Transfered.div(); // divergence of localized Stokes force for penalty constraint	  
	      
	      // f3 <- point forces (perturbation)
	      ForGrids(grid) 
	      {
		MappedGrid & mg = cg[grid];
		getIndex(mg.indexRange(),I1,I2,I3);
		// Process all velocity components (uv=1, vc=2, wc=3) and force components (Fx=0, Fy=1, Fz=2)
		for(i=1; i<Dim+1; i++) 
		  f3[grid](I1,I2,I3,i) = force_Transfered[grid](I1,I2,I3,i-1); // F
		// Process pressure
		f3[grid](I1,I2,I3,pc) = -div_force_Transfered[grid](I1,I2,I3); // -div(F)

	      // f3 <- unperturbed + perturbation
		f3[grid](I1,I2,I3,all) = f2[grid](I1,I2,I3,all) + f3[grid](I1,I2,I3,all);
	      }

	      // *********************************** END: Update RHS ********************************************
	      time_RHSUPDATE+=(getCPU()-time0);
	      time0=getCPU();

	      if (!False_transients) // 1) solve by coefficient matrices
		{
		  // *********************************** BEGIN: Navier-Stokes re-solver ************************************
		  // augment right-hands sides of Navier-Stokes with point forces
		  solver.solve( u2,f3 );   // re-solve the equations (note that u_fluid dynamically linked to u2)

// 		  // ***  Ahlrichs' deterministic experiment *** (START)
// 		  // Assume 1 particle, no radnom forces; use analytical Oseen tensor
// 		  bool Ahlrichs=0;
// 		  if (Ahlrichs)
// 		    {
// #define xfl x[g](I1,I2,I3)
// #define yfl y[g](I1,I2,I3)
// #define zfl z[g](I1,I2,I3)
// #define R rmag[g](I1,I2,I3)
// #define Fx hydrodynamic_force(0,0)
// #define Fy hydrodynamic_force(0,1)
// #define Fz hydrodynamic_force(0,2)
// 		      int g;
// 		      realCompositeGridFunction 
// 			x(cg),y(cg),z(cg),rmag(cg);
// 		      double
// 			xp,yp,zp;

// 		      xp=position(0,0); // primed vars
// 		      yp=position(0,1);
// 		      zp=position(0,2);

// 		      // Evaluate force
// 		      ForParticlesAndDims(particle,dim)
// 			hydrodynamic_force(particle,dim)=Zeta*(velocity(particle,dim)-u_fluid_interpolated(particle,dim));

// 		      // Calcualte velocity disturbance field
// 		      ForGrids(g)
// 		      {
// 			MappedGrid & mg = cg[g];
// 			getIndex(mg.indexRange(),I1,I2,I3);
// 			x[g](I1,I2,I3)=mg.vertex()(I1,I2,I3,0);
// 			y[g](I1,I2,I3)=mg.vertex()(I1,I2,I3,1);
// 			z[g](I1,I2,I3)=mg.vertex()(I1,I2,I3,2);
	  
// 			R=sqrt( (xfl-xp)*(xfl-xp)+(yfl-yp)*(yfl-yp)+(zfl-zp)*(zfl-zp) );
// 			u_fluid[g](I1,I2,I3,0)=1/(R*R*R)*( ((xfl-xp)*(xfl-xp)+R*R)*Fx + (xfl-xp)*(yfl-yp)*Fy       + (xfl-xp)*(zfl-zp)*Fz       ); // vx
// 			u_fluid[g](I1,I2,I3,1)=1/(R*R*R)*( (yfl-yp)*(xfl-xp)*Fx       + ((yfl-yp)*(yfl-yp)+R*R)*Fy + (yfl-yp)*(zfl-zp)*Fz       ); // vy
// 			u_fluid[g](I1,I2,I3,2)=1/(R*R*R)*( (zfl-zp)*(xfl-xp)*Fx       + (zfl-zp)*(yfl-yp)*Fy       + ((zfl-zp)*(zfl-zp)+R*R)*Fz ); // vz
// 		      }
// 		      u_fluid*=f_0/(u_0*L*8*PI*eta);

// 		      // Interpolate
// 		      positionToInterpolate=position;
// 		      int interp0_vel=interpolatePoints(positionToInterpolate,u_fluid,u_fluid_interpolated, Dim);  // update u_fluid_interpolated
// 		    }
// 		  // ***  Ahlrichs' deterministic experiment *** (END)

		  // *********************************** END: Navier-Stokes re-solver **************************************
		}

	      else // 2) solve by false transients

		{
		  // *********************************** BEGIN: Navier-Stokes re-solver by FALSE TRANSIENTS ****************
		  t=0.; Deque.clear();
// 		  // *** BACKWARD EULER - START
// 		  CM_NS_Build(coeff2,2,dt_v_nondim); // build backward-Euler coefficient matrix for Navier Stokes
// 		  // *** BACKWARD EULER - STOP

		  for( ft_step=1; ft_step<=StepMax; ft_step++ ) // steady Stokes flow
		  //for( ft_step=1; ft_step<=dt/dt_v; ft_step++ ) // unsteady Stokes flow
		    {
		      // Take a time step (Euler, Runge-Kutta, Multistep)
		      euler(t,dt_v_nondim,f3,u2,tmp3);
		      //rungeKutta4(t,dt_v_nondim,f3,u2,tmp1,tmp2,tmp3);
		      //linearMultiStep(t,dt_v_nondim,f3,u2,tmp1,tmp2,tmp3);
		      //solver.solve(u2,u2+f3); // *** BACKWARD EULER (solver keeps reference to coeff2)

// 		      //-----------------
		      // Update error (based on du/dt -> 0) every ErrorFreq
		      if( ft_step % ErrorFreq == 0)
			if ( (error_v=max(fabs(tmp3))) > eps_v) // continue if above tolerance
			  printf("\nMax dvdt: %e (NS solver step: %i). Continuing...", error_v, ft_step);
		      //else if (NaN(tmp1)) // too slow, comment unless necessary 
		      //abort(); // abort if NaN encountered (algorithm diverges)
			else // break if within tolerance
			  break;

//  		      if (NaN(tmp3)) // too slow, comment out
//  			{cout << "\nAborting resolver due to divergence."; abort();} // abort if NaN encountered (algorithm diverges)
//   		      else printf("\nMax dvdt: %e (NS re-solver step: %i)", max(fabs(tmp3)), ft_step);
		      printf("\nMax dvdt: %e (NS re-solver step: %i)", max(fabs(tmp3)), ft_step);
// 		      //-----------------
		    }
		  
 		} // end if(!False_transients)
	      // *********************************** END: Navier-Stokes re-solver by FALSE TRANSIENTS ***********************
	      int interp0_vel=interpolatePoints(positionToInterpolate,u_fluid,u_fluid_interpolated, Dim);  // update u_fluid_interpolated
	  if (Debug & 1)
	    {
	      cout << "\n===== DEBUG INFO: STOKESLET =====";
	      u_fluid_interpolated.display("\nU_fluid_interpolated:");
	      printf("\nG = %e", u_fluid_interpolated(0,0)*u_0/(-1.*f_0)*eta*cg[0].gridSpacing(axis2)*L);
	    }
      	      time_NSUPDATE+=(getCPU()-time0);
	    } // end if( step % MTFreq == 0)
	} // end if(Momentum_transfer)
      t+=dt_nondim; // advance in time

      time0=getCPU();

#ifdef USE_MPI
if(rank==0) // only root does I/O...
{
#endif
      if( Export_vmd && sim==1 && step % ExportFreq == 0 ) // export VMD files for the first realization, if time to export
	export_history(history, position); 
      time_HISTEXPORT+=(getCPU()-time0);
#ifdef USE_MPI
}
#endif 

//  // STOP SIMULATION IF COM OF THE CHAIN EXITS GEOMETRY
//  if (1 && TransStatus==2)
//    {
//      printf("\nChain's COM (%f) exited domain at step=%i. Exiting time loop prematurely...",com(Direction),step);
//      break;
//    }
 
 if(Debug & 4) 
   checkIfPenetration(step, position, domain_min, domain_max);
 
    time_REALFIN+=(getCPU()-time0);
    step++; // *** TRANSLOCATION
    } // END OF INNER LOOP (over time steps)

    time0=getCPU();
    ppAverage(PPD,PPS,dt_nondim,norm); // average dynamic PP & evaluate static PP
    PPDsum+=PPD; // keep running average
    PPSsum+=PPS;

    DiffSims(sim-1)=DIFF_MSD; // save diffusivities for std's

    time_PPEVAL+=(getCPU()-time0);


  } // END OF OUTER LOOP (over realizations)
  time_PD=getCPU()-time_PD;

  time0=getCPU();

#ifdef USE_MPI
  double * PPDsum_p = PPDsum.getDataPointer(); // define pointers (doesn't work with references)
  double * PPSsum_p = PPSsum.getDataPointer();
  double * PPD_p = PPD.getDataPointer();
  double * PPS_p = PPS.getDataPointer();

  double * PPSIM_p = PPSIM.getDataPointer();
  double * PPSIM2_p = PPSIM2.getDataPointer();

  double * DiffSims_p = DiffSims.getDataPointer();
  double * DiffSims2_p = DiffSims2.getDataPointer();

  MPI::COMM_WORLD.Reduce(PPDsum_p, PPD_p, NPPD*NoOfProdSteps, MPI::DOUBLE, MPI::SUM,0); // collect results from all processors
  MPI::COMM_WORLD.Reduce(PPSsum_p, PPS_p, NPPS, MPI::DOUBLE, MPI::SUM,0);

  MPI::COMM_WORLD.Reduce(PPSIM_p, PPSIM2_p, NPPSIM*NoOfSims, MPI::DOUBLE, MPI::SUM,0);
  PPSIM=PPSIM2;

  MPI::COMM_WORLD.Reduce(DiffSims_p, DiffSims2_p, NoOfSims, MPI::DOUBLE, MPI::SUM,0);
  DiffSims=DiffSims2;
#else
  PPD=PPDsum;
  PPS=PPSsum;
#endif 
  PPD/=NoOfSims; // average over realizations
  PPS/=NoOfSims;
  time_PPEVAL+=(getCPU()-time0);

  time_PPEXPORT=getCPU();
#ifdef USE_MPI
if (rank==0)
{
#endif
  double std_diff=sqrt(variance( DiffSims, NoOfSims )); // standard deviation (std) of diffusivity
  ppExport(PPD,PPS,PPSIM,time,dt_nondim,"Output/DATA_" + outputSuffix + ".HDF", std_diff); // export polymer properties
#ifdef USE_MPI
}
#endif

  time_PPEXPORT=getCPU()-time_PPEXPORT;

#ifdef USE_MPI
if(rank==0) // only root does I/O...
{
#endif
  if(Export_vmd)
    history.close();
#ifdef USE_MPI
}
#endif 

// // *ph092310* Causes *significant* delay in MPI simulations (tested with NoOfSims=10)
// #ifdef USE_DLPOLY
//   if(Molecular_dynamics)
//     {
//       dlpoly_terminate_(); // DLPOLY: Terminate (phase 3 of 3)
//     }
// #endif 
  
  if (NoOfTimeSteps>1) // report force magnitudes
    {
      Zeta_at_Tf=Fep_at_Tf=Fdep_at_Tf=Frnd_at_Tf=Fmd_at_Tf=Fw_at_Tf=0; // initialize to zero
	  
      ForDims(i)
      {
	Zeta_at_Tf += SQR(velocity(all,i)-u_fluid_interpolated(all,i));
	Fep_at_Tf += SQR(E_interpolated(all,i));
	Fdep_at_Tf += SQR(grad_E_sq_interpolated(all,i));
	Frnd_at_Tf += SQR(Random(all,i));
	Fmd_at_Tf += SQR(md_force(all,i));
	Fw_at_Tf += SQR(Fwall(all,i));
      }
	  
      Zeta_max = max(sqrt(Zeta_at_Tf));
      Fep_max = max(sqrt(Fep_at_Tf));
      Fdep_max = max(sqrt(Fdep_at_Tf));
      Frnd_max = max(sqrt(Frnd_at_Tf));
      Fmd_max = max(sqrt(Fmd_at_Tf));
      Fw_max = max(sqrt(Fw_at_Tf));
	  
      cout.precision(4); cout.setf(ios_base::scientific);
#ifdef USE_MPI
if(rank==0) // only root does I/O...
{
#endif 
      cout << "\n ****************** MAGNITUDES OF FORCES *************************";
      cout << "\n Printing out max magnitudes over all particles at final time...";
      cout << "\n FORCE " << setw(10) << "COEFF" << setw(20) << "SCALED_VAR"<< setw(21) << "COEFF*SCALED_VAR";
      cout << "\n F_st: " << setw(15) << Zeta << setw(15) <<  Zeta_max << setw(15) <<  Zeta*Zeta_max;
      cout << "\n F_rnd:" << setw(15) << Sigma_reduced  << setw(15) <<  Frnd_max << setw(15) <<  1*Frnd_max;
      cout << "\n F_md: " << setw(15) << Fmd  << setw(15) <<  Fmd_max  << setw(15) <<  Fmd*Fmd_max;
      cout << "\n F_ep: " << setw(15) << Fep  << setw(15) <<  Fep_max  << setw(15) <<  Fep*Fep_max;
      cout << "\n F_dep:" << setw(15) << Fdep << setw(15) <<  Fdep_max << setw(15) <<  Fdep*Fdep_max;
      cout << "\n F_w:  " << setw(15) << double(Wall) << setw(15) <<  Fw_max   << setw(15) <<  Wall*Fw_max << endl;
#ifdef USE_MPI
}
#endif 
    }

  //=========================================================================================
  //    Summary of timers
  //=========================================================================================
   
  real t_tot=(getCPU()-time_START)/100;
int Tsec=int(t_tot*100);
  cout.unsetf(ios_base::scientific); cout.precision(2); // format scientific -> fixed
  cout.setf(ios_base:: fixed); cout.setf(ios_base::showpoint);
 
#ifdef USE_MPI
if(rank==0) // only root does I/O...
{
#endif 
  // *** PRINT-OUT A SUMMARY ***
  cout << "\n ****************** TIMERS - SUMMARY *****************************";
  cout << "\n Total runtime:            " << Tsec/3600 << " hrs " << (Tsec%3600)/60 << " mins " << Tsec%60 << " s ";
  cout << "\n ---PHASE:                 ";
  cout << "\n                           " << setw(10) << "[% total runtime]";
  cout << "\n Initialization:           " << setw(10) << time_INI/t_tot;
  cout << "\n Eikonal:                  " << setw(10) << time_EIK/t_tot;
  cout << "\n Poisson-Boltzmann:        " << setw(10) << time_PB/t_tot;
  cout << "\n Navier-Stokes:            " << setw(10) << time_NS/t_tot;
  cout << "\n Tracer dynamics:          " << setw(10) << time_TD/t_tot;
  cout << "\n Polymer dynamics:         " << setw(10) << time_PD/t_tot;
  cout << "\n Export to HDF:            " << setw(10) << time_PPEXPORT/t_tot << endl;

  cout << "\n ---DETAILS:               ";
  cout << "\n                           " << setw(10) << "[% polymer dynamics]";
  cout << "\n Initialization:           " << setw(10) << time_PDINI/time_PD*100 + time_REALINI/time_PD*100;
  cout << "\n Particle integrator:      " << setw(10) << time_INTEG/time_PD*100;
  cout << "\n Momentum transfer:        " << setw(10) << time_MTRANSFER/time_PD*100;
  cout << "\n Navier-Stokes rhs update: " << setw(10) << time_RHSUPDATE/time_PD*100;
  cout << "\n Navier-Stokes resolve:    " << setw(10) << time_NSUPDATE/time_PD*100;
  cout << "\n Polymer properties:       " << setw(10) << time_PPEVAL/time_PD*100;
  cout << "\n Finalization:             " << setw(10) << time_REALFIN/time_PD*100;
  cout << "\n                           " << setw(10) << "[% particle integrator]";
  cout << "\n Force evaluations:        " << setw(10) << time_DLPOLY/time_INTEG*100;
  cout << "\n                           " << setw(10) << "[% finalization]";
  cout << "\n HISTORY export:           " << setw(10) << time_HISTEXPORT/time_REALFIN*100;
  cout << "\n                           " << setw(10) << "[% polymer properties]";
  cout << "\n Center of mass:           " << setw(10) << time_COM/time_PPEVAL*100;
  cout << "\n Radius of gyration:       " << setw(10) << time_MSRGYR/time_PPEVAL*100;
  cout << "\n End to end distance:      " << setw(10) << time_MSEED/time_PPEVAL*100;
  cout << "\n Directional stretch:      " << setw(10) << time_STRETCH/time_PPEVAL*100;
  cout << "\n MSD & VAF:                " << setw(10) << time_MSD/time_PPEVAL*100;
  cout << endl << endl;
#ifdef USE_MPI
}
#endif
  Overture::printMemoryUsage("memory usage");

  //=========================================================================================
  //    Plotting
  //=========================================================================================

  if (Plot_final)
    {
      // Plot results using PlotIt class

      cout << "\n\n *****************************************************************\n";
      cout << " Plotting results...\n";
      cout << " *****************************************************************\n";

      GenericGraphicsInterface * psPointer = Overture::getGraphicsInterface("Results Visualization", TRUE); 
      GenericGraphicsInterface & ps = *psPointer;

      int win0 = 0; // the default graphics window has number 0
      char buf[100];
      GraphicsParameters gp;               // create an object that is used to pass parameters
      GUIState interface;                  // GUI object
      SelectionInfo select;                // the results from picking will be recorded here
  
      aString answer;

      // define push buttons
      aString pbCommands[] = {"plot grid", "plot velocity field base","plot velocity field pert","plot pressure field base","plot point force","plot E field", "plot intrinsic field", "plot applied field","plot velocity divergence","plot tracer particles","plot colloidal particles","plot RDF","plot stokeslet","plot stretch","plot msrgyr","plot mseed","plot msd_com","plot D",""};
      aString pbLabels[] = {"Composite Grid","Velocity field (base)","Velocity field (perturbed)","Pressure field (base)","Point forces","E field", "Intrinsic field ", "Applied field","Velocity Divergence","Tracer Particles","Colloidal Particles","RDF","Stokeslet","Stretch","Rgyr","E-E","MSD of COM","WallDist",""};

      interface.setPushButtons( pbCommands, pbLabels, 6 ); // 6 rows
      interface.setWindowTitle("Particle Tracking Model Visualization");
      interface.setExitCommand("exit", "Exit");

      ps.pushGUI(interface);
  
      for(;;)
	{
	  ps.getAnswer(answer,"Ready>", select);
  
	  if (select.active == 1)
	    {
	      ps.outputString(sPrintF(buf,"Window coordinates: %e, %e", select.r[0], select.r[1]));
	    }
	  if( select.nSelect )
	    {
	      ps.outputString(sPrintF(buf,"World coordinates: %e, %e, %e", select.x[0], select.x[1], select.x[2]));
	      ps.outputString(sPrintF(buf,"globalID=%i", select.globalID));
	      ps.outputString(sPrintF(buf,"Selection:"));
	      int i;
	      for (i=0; i<select.nSelect; i++)
		{
		  ps.outputString(sPrintF(buf,"ID[%i]=%i, back-z=%i, front-z=%i",i,
					  select.selection(i,0),select.selection(i,1),select.selection(i,2)));
		}
	    }
	  if( answer == "plot grid" )
	    {
	      gp.set(GI_TOP_LABEL,"Composite grid");  // set title
	      gp.set(GI_PLOT_SHADED_SURFACE_GRIDS, TRUE);
	      gp.set(GI_LABEL_GRIDS_AND_BOUNDARIES, TRUE);
	      gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, FALSE);
	      PlotIt::plot(ps, cg, gp);   // plot the composite grid
	    }
	  else if( answer == "plot velocity field base" )
	    {
	      gp.set(GI_TOP_LABEL,"Velocity field"); gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, FALSE);
	      PlotIt::streamLines(ps, u_fluid_base,gp);  // streamLines / contour plot (base) ( (x) component )
	      //u_fluid_base.display();
	    }
	  else if( answer == "plot velocity field pert" )  // streamLines / contour plot (perturbed=base+perturbation)
	    {
	      gp.set(GI_TOP_LABEL,"Velocity field"); gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, FALSE);

	      //gp.set(GI_MINIMUM_CONTOUR_SPACING,0.1); gp.set(GI_NUMBER_OF_CONTOUR_LEVELS,300); // for dipole Stokeslet
	      realCompositeGridFunction perturb(cg);
	      ForGrids(grid) // calculate the magnitude of u_fluid
	      {
		ForDims(i)
		  perturb[grid] += SQR(u_fluid[grid](all,all,all,i));
		perturb[grid]=sqrt(perturb[grid]);
	      }
	      PlotIt::contour(ps, perturb,gp);  // plot magnitude of u_fluid 
	      //PlotIt::contour(ps, u_fluid, gp);  // Overture will implicitly plot only the first (x) component of the vector field

// 	      // *** DEP QUADRUPOLE *** (START)
// 	      // Quadrupole correction: large at boundaries, why? (mesh quality?)
// 	      realCompositeGridFunction dep(cg), depq(cg), Q_c(cg);
// 	      ForGrids(grid) // calculate the magnitude 
// 	      {
// 		ForDims(i)
// 		  dep[grid] += SQR(grad_E_sq[grid](all,all,all,i));
// 		dep[grid]=sqrt(dep[grid]);
// 	      }
// 	      ForGrids(grid) // calculate the magnitude 
// 	      {
// 		ForDims(i)
// 		  depq[grid] += SQR(Q[grid](all,all,all,i));
// 		depq[grid]=sqrt(depq[grid]);
// 	      }
// 	      Q_c=Fdep_q*depq / (Fdep*dep);
// 	      PlotIt::contour(ps, Q_c,gp);
// 	      // *** DEP QUADRUPOLE *** (END)

	    }
	  else if( answer == "plot pressure field base" )
	    {
	      gp.set(GI_TOP_LABEL,"Pressure field"); gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, FALSE);
	      PlotIt::contour(ps, p_base, gp);  // streamLines / contour plot (base)
	    }
	  else if( answer == "plot point force" )
	    {
	      gp.set(GI_TOP_LABEL,"Point forces"); gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, FALSE);
	      PlotIt::contour(ps, force_Transfered, gp);

	    }
	  else if( answer == "plot E field" )
	    {
	      gp.set(GI_TOP_LABEL,"E field"); gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, FALSE);
	      PlotIt::streamLines(ps, E, gp);
	    }
	  else if( answer == "plot intrinsic field" )
	    {
	      gp.set(GI_TOP_LABEL,"Intrinsic Electrostatic Field"); gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, FALSE);
	      PlotIt::contour(ps, u(R0), gp);
	    }
	  else if( answer == "plot applied field" )
	    {
	      gp.set(GI_TOP_LABEL,"Applied Electrostatic Field"); gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, FALSE);
	      PlotIt::contour(ps, u(R1), gp);
	    }
	  else if( answer == "plot velocity divergence" )
	    {
	      gp.set(GI_TOP_LABEL,"Velocity Divergence");
	      gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, FALSE);
	      u_fluid.setOperators(op); RealCompositeGridFunction u_fluid_div(cg); 
	      u_fluid_div=u_fluid.div(); u_fluid_div.interpolate();
	      const aString & title="Velocity divergence";
	      PlotIt::contour(ps, u_fluid_div, gp);
	    }
	  else if( answer == "plot tracer particles" )
	    {
	      gp.set(GI_TOP_LABEL,"Tracer Particles");
	      gp.set(GI_POINT_COLOUR,"blue"); gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, TRUE);
	      Range R_steps(0,NoOfTimeStepsT-1), R_dim(0,Dim-1), R_singleton(0,Singleton-1); 
	      realArray position_redim(1, Dim, NoOfTimeStepsT*NoOfParticles), position_reshape(NoOfTimeStepsT*NoOfParticles, Dim, 1);

	      for (int k=0; k<NoOfParticles; k++)  // redimension coordinate array
		{
		  position_redim(R_singleton, R_dim, Range(k*NoOfTimeStepsT,(k+1)*NoOfTimeStepsT-1))=position_tracer(Range(k,k),R_dim,R_steps);
		}
	      for (int k=0; k<NoOfParticles*NoOfTimeStepsT; k++)  // reshape coordinate array
		{
		  position_reshape(k, R_dim, R_singleton)=position_redim(R_singleton, R_dim, k);
		}
	      ps.plotPoints(position_reshape,gp); // particle positions plot
	    }
	  else if( answer == "plot colloidal particles" )
	    {
	      gp.set(GI_TOP_LABEL,"Colloidal Particles");
	      gp.set(GI_POINT_COLOUR,"red"); gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, TRUE);
	      int Singleton=1;
	      Range R_steps(0,NoOfTimeSteps-1), R_dim(0,Dim-1), R_singleton(0,Singleton-1); 
	      realArray position_redim(1, Dim, NoOfTimeSteps*NoOfParticles), position_reshape(NoOfTimeSteps*NoOfParticles, Dim, 1);
	      for (int k=0; k<NoOfParticles; k++)  // redimension coordinate array
		{
		  position_redim(R_singleton, R_dim, Range(k*NoOfTimeSteps,(k+1)*NoOfTimeSteps-1))=position(Range(k,k),R_dim,R_steps);
		}
	      for (int k=0; k<NoOfParticles*NoOfTimeSteps; k++)  // reshape coordinate array
		{
		  position_reshape(k, R_dim, R_singleton)=position_redim(R_singleton, R_dim, k);
		}
	      ps.plotPoints(position_reshape,gp); // particle positions plot
	    }
	  else if( answer == "plot stokeslet" )
	    {
	      // Stokeslet for 1 particle (preferably in the center) at T_final
	      gp.set(GI_TOP_LABEL,"Stokeslet");
	      gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, TRUE);
	      const aString & title="Stokeslet"; const aString & xlabel ="distance"; const aString * ylabel=NULL;//& ylabel="U/F";
	      int PARTICLE=0, NMAX=100, dimension=0;
	      Range dist_(0,NMAX-1), dist_plus_1(1,NMAX-1);
	      doubleArray distance(dist_,Dim), U_over_F(dist_,Dim), Oseen(dist_,Dim), displacement(R0,Dim); 
	      displacement=0; u_fluid_interpolated=0; distance=0; 
	      displacement(0,dimension)=0.01; // need 2D array (1st dim dummy) for the addition below to be conformable
	      for(i=0;i<NMAX;i++)
		{
		  distance(i,all)=i*displacement(0,all); // march from the point source
		  int interp_vel=interpolatePoints(position(PARTICLE,all,NoOfTimeSteps-1)+distance(i,all),u_fluid,u_fluid_interpolated, Dim); // assumes base fluid field = 0 => u_fluid is pure velocity perturbation
		  U_over_F(i,all)=u_fluid_interpolated(PARTICLE,all)/(forceToExtrapolate(PARTICLE,all)); // O ~ U/F
		  if(Debug & 2) 
		    u_fluid_interpolated.display("\nU_fluid_interpolated:");
		}
	      Oseen(all,dimension)=1./(4*PI*abs(distance(all,dimension))); // decay of Oseen tensor along x-axis

	      distance(dist_plus_1,dimension).display("\nDistance from point force: ");
	      U_over_F(dist_plus_1,dimension).display("\nU_over_F: ");
	      Oseen(dist_plus_1,dimension).display("\nOseen: ");

	      // Look at the decay along x-axis
	      PlotIt::plot(ps,distance(dist_plus_1,dimension),U_over_F(dist_plus_1,dimension),title,xlabel,ylabel,gp);
	      PlotIt::plot(ps,distance(dist_plus_1,dimension),Oseen(dist_plus_1,dimension),title,xlabel,ylabel,gp); // skip the first value (point force location) to avoid singularity
	      if(NoOfParticles>1)
		cout << "\a\nNote that this plot is meaningful only for 1 particle (no HI) and only for vanishing base velocity field (F_stokes + F_brownian)!\n";
	    }
	  // else if( answer == "plot RDF" )
	  //     {
	  //       gp.set(GI_TOP_LABEL,"RDF");
	  //       gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, TRUE);
	  //       const aString & title="Radial distribution function"; const aString & xlabel ="r"; const aString * ylabel=NULL;
	  //       PlotIt::plot(ps,shell_r,RDF,title,xlabel,ylabel,gp);
	  //     }
	  else if( answer == "plot stretch" )
	    {
	      gp.set(GI_TOP_LABEL,"Directional stretch");
	      gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, TRUE);
	      const aString & title="Directional Stretch"; const aString & xlabel ="Time"; const aString * ylabel=NULL;
	      PlotIt::plot(ps,time,STRETCH(all),title,xlabel,ylabel,gp);
	    }
	  else if( answer == "plot msrgyr" )
	    {
	      gp.set(GI_TOP_LABEL,"Mean square Radius of gyration");
	      gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, TRUE);
	      const aString & title="MSRGYR"; const aString & xlabel ="Time"; const aString * ylabel=NULL;
	      PlotIt::plot(ps,time,MSRGYR(all),title,xlabel,ylabel,gp);
	    }
	  else if( answer == "plot mseed" )
	    {
	      gp.set(GI_TOP_LABEL,"Mean square end-to-end distance");
	      gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, TRUE);
	      const aString & title="MSEED"; const aString & xlabel ="Time"; const aString * ylabel=NULL;
	      PlotIt::plot(ps,time,MSEED(all),title,xlabel,ylabel,gp);
	    }
	  else if( answer == "plot msd_com" )
	    {
	      gp.set(GI_TOP_LABEL,"Mean square displacement of center-of-mass");
	      gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, TRUE);
	      const aString & title="MSD_COM"; const aString & xlabel ="Time"; const aString * ylabel=NULL;
	      PlotIt::plot(ps,time,MSD(all),title,xlabel,ylabel,gp);
	    }
	  else if( answer == "plot D" )
	    {
	      gp.set(GI_TOP_LABEL,"Wall distance"); gp.set(GI_PLOT_THE_OBJECT_AND_EXIT, FALSE);
	      cout << "\nDmax=" << max(fabs(D)) << ", Dmin=" << min(fabs(D));
	      PlotIt::contour(ps, D,gp);  // streamLines / contour plot (base)
	    }
	  else if( answer=="exit" )
	    {
	      break;
	    }
	}
      ps.popGUI();

    } // end if(Plot_final)

  // Export P-B and N-S solutions as show files
  if(!FileExists(pbShow)) // check if file already exists
    {   
      cout << " \nSaving P-B solution in " << pbShow << " file...\n";   
      Ogshow pbshow( pbShow );  // create a show file
      u.setName("Electric potential");   
      u.setName("intrinsic", 0);   
      u.setName("applied", 1);
      pbshow.startFrame();   
      pbshow.saveComment(0,"Electric potentials (intrinsic & applied)");   
      pbshow.saveSolution(u);  // save the component solutions   
      pbshow.endFrame();   
      pbshow.close();
    }

  if(!FileExists(nsShow)) // check if file already exists
    {   
      cout << " \nSaving N-S solution in " << nsShow << " file...\n"; 
      Ogshow nsshow( nsShow );            // create a show file
      u_fluid.setName("Base velocity field");   
      nsshow.startFrame();   
      nsshow.saveComment(0,"Base velocity field");   
      nsshow.saveSolution(u_fluid_base);  // save the composite solution   
      nsshow.endFrame();

      realCompositeGridFunction u_fluid_mag(cg);
      ForGrids(grid) // calculate the magnitude of u_fluid
      {
	ForDims(i)
	  u_fluid_mag[grid] += SQR(u_fluid[grid](all,all,all,i));
	u_fluid_mag[grid]=sqrt(u_fluid_mag[grid]);
      }
      u_fluid.setName("Base velocity field - magnitude");   
      nsshow.startFrame();   
      nsshow.saveComment(0,"Base velocity field - magnitude");
      nsshow.saveSolution(u_fluid_mag);  // save the composite solution   
      nsshow.endFrame();

      nsshow.close();
    }
  

  // Finish ANN
  if (Wall)  delete kdTree;
  annClose(); // done with ANN
  // Finish Overture
  Overture::finish();  
#ifdef USE_MPI
  // Finish MPI
  MPI::Finalize();
#endif
        
  cout << "\n *****************************************************************\n";   
  cout << " RUN SUCCESSFUL.\n";   
  cout << " *****************************************************************\n\n";
  return(0);
}

// // ************************************** ADDITIONAL SUB-ROUTINES and FUNCTIONS **************************************//
double find_minmax(double voltages[], int NoOfBCs, int minmax) // find minimum maximum of an array
// voltages = array
// NoOfBCs = # elements in array
// minmax = [0 / 1] = minimum / maximum
{
  vector<double> v_voltages(voltages, voltages+NoOfBCs); // vector template
  sort(v_voltages.begin(),v_voltages.end()); // sort in ascending order
  return (minmax == 1 ? v_voltages[NoOfBCs-1] : v_voltages[0]); // last element = max, first element = min
}

void export_FIELD_CONTROL(const char * field_file, const char* control_file, int NoOfWallAtoms)
{
  using namespace Data_ini;
  using namespace Main_misc;
    
  int MOLECULES, VDW;
  // If Wall=4, add info on wall atoms
  if (Wall==4)
    {MOLECULES=2; VDW=3;}
  else
    {MOLECULES=VDW=1;}

  // 1) Export FIELD
  cout << "\nExporting " << field_file << " file...\n";
  ofstream field(field_file); // open file for output
  if (!field.is_open())   // open attempt failed
    {
      cerr << "\nCan't open " << field_file << " file for output!\n"; field.clear(); field.close(); // disconnect file
      exit(EXIT_FAILURE);
    }
  field.precision(3); field.setf(ios_base::scientific);  // set precision and scientific notation

  // POLYMER BEADS, ATOMS
  // Units conversion: 1 [kg] = 1e3*Na [Da], 1 [C] = 1/e [e]
  field << "CHAIN (" << NoOfParticles << " beads)\nUNITS internal\nMOLECULES " << MOLECULES << "\nPolymer\nNUMMOLS 1\nATOMS " << NoOfParticles
	<< "\nbead " << setw(14) << 1e3*AVOG*mass_p << setw(14) << charge_p << setw(14) << NoOfParticles; // mass, charge, repeat counter
  
  // POLYMER BEADS, BONDS
  field	<< "\nBONDS " << NoOfParticles-1;
  // Units conversion: 1 [J/m2] = 1/(10*1e20*Na) [10J/mol/A2], 1 [m] = 1e10 [A]
  switch(SpringLaw)
    {
    case 1: cout << "---Spring force: FENE (ks, q0, delta=0)\n";
#ifdef USE_DLPOLY
      for(int i=1; i<NoOfParticles; i++) field << "\n-fen " <<  setw(6) << i << setw(6) << i+1 << setw(12) << ks/(1e21*AVOG) << setw(12) << q0*1e10 << setw(12) << 0; 
#else
      cout << "FENE requires DLPOLY support! Use -DUSE_DLPOLY in your Makefile. Aborting now.\n";
      abort();
#endif
    break;
    case 2: cout << "---Spring force: Marko-Siggia (bk, q0)\n";
      for(int i=1; i<NoOfParticles; i++) field << "\n-mar " <<  setw(6) << i << setw(6) << i+1 << setw(12) << bk*1e10 << setw(12) << q0*1e10; break;
    }
  field << "\nFINISH";

  // WALL ATOMS
  double // mass and charge of wall atoms
    mass_w=1, 
    charge_w=1;

  if (Wall==4)
    {
      field << "\nWalls\nNUMMOLS 1\nATOMS " << NoOfWallAtoms
	    << "\nwall " << setw(14) << 1e3*AVOG*mass_w << setw(14) << charge_w << setw(14) << NoOfWallAtoms << setw(14) << "1"; // mass, charge, repeat counter, freeze (>0 if frozen)
      field << "\nFINISH";
    }

  // NON-BONDED FORCES (VDW)
  field << "\nVDW " << VDW;
  // Params for Jendrejack's potential
  double Nks=q0/bk, Rs2=Nks*bk*bk; // # Kuhn segments per spring, mean square bond length [SI units]
  double // potential prefactor, force prefactor and exponential prefactor, respectively
    exp_EV=-9./(2*Rs2) *1e-20, // [m^-2] -> [A^-2] (dlpoly)
    u_EV=.5*KB*temp*v_ev*Nks*Nks*pow(9./(2*PI*Rs2),3./2.) *(0.1*AVOG), // [J] -> [10J/mol]
    f_EV=-2.*exp_EV * u_EV;

  switch(Vdw)
    {
    case 1: cout << "---VDW force (beads): Jendrejack\n"; 
      field << "\nbead   bead   jen " << setw(12) << u_EV << setw(12) << f_EV << setw(12) << exp_EV; // Use Jendrejack for beads
      break;
    case 2: cout << "---VDW force (beads): Lennard-Jones\n";
      field << "\nbead   bead   wca  " << setw(12) << vdw_epsilon << setw(12) << vdw_sigma; // Use L-J for beads
    }

  if (Wall==4)
    {
      switch(Vdw)
	{
	case 1: cout << "---VDW force (beads-walls): Jendrejack\n"; 
	  // Use Jendrejack for discrete walls
	  field << "\nbead   wall   jen " << setw(12) << u_EV << setw(12) << f_EV << setw(12) << exp_EV;
	  break;
	case 2: cout << "---VDW force (beads-walls): Lennard-Jones\n";
	  // Use Lennard-Jones for discrete walls (params already in dlpoly's internal units)
	  field << "\nbead   wall   wca  " << setw(12) << vdw_epsilon << setw(12) << vdw_sigma;
	}
      cout << "---VDW force (walls-walls): Lennard-Jones\n"; 
      field << "\nwall   wall   wca  " << setw(12) << vdw_epsilon << setw(12) << vdw_sigma;
    }

  field <<"\nCLOSE\n";
  field.close();

  // 2) Export CONTROL
  cout << "\nExporting " << control_file << " file...\n";
  ofstream control(control_file); // open file for output
  if (!control.is_open())   // open attempt failed
    {
      cerr << "\nCan't open " << control_file << " file for output!\n"; control.clear(); control.close(); // disconnect file
      exit(EXIT_FAILURE);
    }
  control.setf(ios_base::left);  // left-justify the columns

  control << "CHAIN\n"
	  << setw(30) << "\ntemperature" << temp
	  << setw(30) << "\nequilibration steps" << 0 // no equilibration
	  << setw(30) << "\nsteps" << 0 // dry run
	  << setw(30) << "\ntimestep" << 0
//  	  << setw(30) << "\ncutoff" << 2.5*vdw_sigma // standard cut-off
  	  << setw(30) << "\ncutoff" << 3.0e3 // large cut-off
      	  << setw(30) << "\nrvdw" << 2.5*vdw_sigma // cut-off for VDW forces (optional)
//    	  << setw(30) << "\ndelr" << (2.5*vdw_sigma)+0.3*vdw_sigma; // shell width for verlet neighbor list
          << setw(30) << "\ndelr" << 0; // no verlet neighbor list to prevent 'failed configuration arrays' memalloc error
 
//   if(Electrostatics)
//     {
//       control << "\nreaction"; 
//       control << setw(30) << "\neps" << 80;
//     } 
//   else
  control << "\nno electrostatics"; // electrostatics: ON (reaction field) / OFF
  
  if(!Vdw) 
    control << "\nno vdw" ; // vdw forces ON / OFF
  
  control << setw(30) << "\nprint every" << "1 stps"
	  << setw(30) << "\nstats every" << "1 stps"
	  << setw(30) << "\ntrajectory" << "0 1 2" // output coords, vels & forces
	  << setw(30) << "\njob time" << "10 seconds"
	  << setw(30) << "\nclose time" << "2 seconds"
	  << setw(30) << "\nno link" // do not use linked cells for vdw forces

	  << "\nfinish\n";
  
  control.close();
}

void init_history(const char * history_file, ofstream & history)
{
  using namespace Data_ini;
  using namespace Main_misc;

  // Init HISTORY
  history.open(history_file,ios::out); // open file for output
  if (!history.is_open())   // open attempt failed
    {
      cerr << "\nCan't open " << history_file << " file for output!\n"; history.clear(); history.close(); // disconnect file
      exit(EXIT_FAILURE);
    }
  history.setf(ios_base::scientific);  // set scientific notation
  history.precision(4); // set precision

}

void export_history(ofstream & history, const doubleArray & position)
{
  using namespace Data_ini;
  using namespace Main_misc;

  // Export HISTORY

  int keytrj=0, imcon=0; 
  double tstep=0.;
 
  history << setw(8) << "timestep" << setw(10) << step+1 << setw(10) << NoOfParticles << setw(10) << keytrj << setw(10) << imcon << setw(12) << tstep << endl;
  ForParticles(particle)
  {
    history	<< setw(8) << "bead" << setw(10) << particle+1 << setw(12) << 1e3*AVOG*mass_p << setw(12) << charge_p << endl;
    ForDims(i) 
    {
      history << setw(12) << position(particle, i)*L*1e6; // [microns]
    }
    if(Dim < 3) history << setw(12) << 0.; // add a dummy 3rd coord if in 2-D (slow but simple)
    history << endl;
  }
}

void export_psf(const char* psf_file)
{
  using namespace Data_ini;
  using namespace Main_misc;

  // Export .PSF
  /*  
      Carbons used as dummy atoms (i.e. their parameters, such as charge and mass, are irrelevant) 
      representing beads CREATED BY USER: petr.hotmar                
      The fields in the atom section are:
      atom ID, segment name, residue ID, residue name, atom name, atom type, charge, mass, and an unused 0.
  */

  cout << "\nExporting " << psf_file << " file...\n";
  ofstream psf(psf_file); // open file for output
  if (!psf.is_open())   // open attempt failed
    {
      cerr << "\nCan't open " << psf_file << " file for output!\n"; psf.clear(); psf.close(); // disconnect file
      exit(EXIT_FAILURE);
    }
  double C_charge=0.51, C_mass=12.011; 
  int W=8, NoOfComments=1;

  psf << "PSF\n\n" << setw(W) << NoOfComments << " !NTITLE" << endl 
      << "* Carbons used as dummy atoms (i.e. their parameters, such as charge and mass, are irrelevant) representing beads CREATED BY USER: petr.hotmar. The fields in the atom section are: atom ID, segment name, residue ID, residue name, atom name, atom type, charge, mass, and an unused 0.\n\n"
      << setw(W) << NoOfParticles << " !NATOM\n";
  ForParticles(particle)
  {
    psf << setw(W) << particle+1 << " CHAIN 1 " << setw(W) << "bead" << setw(W) << "C" <<  setw(W) << "C" << setw(W)
	<<  C_charge << setw(W) << C_mass << setw(W) << "0\n";
  }
  int NoOfBonds=NoOfParticles-1;
  psf << endl << setw(W) << NoOfBonds << " !NBOND: bonds\n";
  for (int bond=1; bond<=NoOfBonds; bond++) // loop over bonds
    {
      psf << setw(W) << bond << setw(W) << bond+1;
      if(bond % 4 == 0) psf << endl;
    }
  psf.close();
}

void import_grid(aString Ogfile)
{
  if (Ogfile=="Grids/separator.hdf")
    {
      // SEPARATOR GEOMETRY (separator.hdf)
      // background grid 1 (assignments determined by BC's and share flags)
      cg[0].boundaryCondition()(Start,axis1)=noSlipWall2;
      cg[0].boundaryCondition()(Start,axis2)=outflow1;
      cg[0].boundaryCondition()(End  ,axis2)=outflow2;
 
      // background grid 2
      cg[1].boundaryCondition()(End  ,axis1)=inflow2;
      cg[1].boundaryCondition()(Start,axis2)=inflow1;
      cg[1].boundaryCondition()(End  ,axis2)=noSlipWall;

      // left polygon
      cg[2].boundaryCondition()(Start,axis1)=outflow1;
      cg[2].boundaryCondition()(End  ,axis1)=outflow2;
      cg[2].boundaryCondition()(Start,axis2)=noSlipWall;

      // upper polygon
      cg[3].boundaryCondition()(Start,axis1)=outflow2;
      cg[3].boundaryCondition()(End  ,axis1)=inflow2;
      cg[3].boundaryCondition()(Start,axis2)=noSlipWall;

      // lower polygon
      cg[4].boundaryCondition()(Start,axis1)=outflow1;
      cg[4].boundaryCondition()(End  ,axis1)=inflow1;
      cg[4].boundaryCondition()(Start,axis2)=noSlipWall;

      // corner polygon
      cg[5].boundaryCondition()(Start,axis1)=inflow1;
      cg[5].boundaryCondition()(End  ,axis1)=inflow2;
      cg[5].boundaryCondition()(Start,axis2)=noSlipWall;
    }
  else if (Ogfile=="Grids/separator3D.hdf")
    {
      // SEPARATOR GEOMETRY (separator.hdf)
      // background grid 1 (assignments determined by BC's and share flags)
      cg[0].boundaryCondition()(Start,axis1)=noSlipWall2;
      cg[0].boundaryCondition()(Start,axis2)=outflow1;
      cg[0].boundaryCondition()(End  ,axis2)=outflow2;
      cg[0].boundaryCondition()(Start,axis3)=noSlipWall;
      cg[0].boundaryCondition()(End  ,axis3)=noSlipWall;

      // background grid 2
      cg[1].boundaryCondition()(End  ,axis1)=inflow2;
      cg[1].boundaryCondition()(Start,axis2)=inflow1;
      cg[1].boundaryCondition()(End  ,axis2)=noSlipWall;
      cg[1].boundaryCondition()(Start,axis3)=noSlipWall;
      cg[1].boundaryCondition()(End  ,axis3)=noSlipWall;

      // left polygon
      cg[2].boundaryCondition()(Start,axis1)=outflow1;
      cg[2].boundaryCondition()(End  ,axis1)=outflow2;
      cg[2].boundaryCondition()(Start,axis2)=noSlipWall;
      cg[2].boundaryCondition()(Start,axis3)=noSlipWall;
      cg[2].boundaryCondition()(End  ,axis3)=noSlipWall;

      // upper polygon
      cg[3].boundaryCondition()(Start,axis1)=outflow2;
      cg[3].boundaryCondition()(End  ,axis1)=inflow2;
      cg[3].boundaryCondition()(Start,axis2)=noSlipWall;
      cg[3].boundaryCondition()(Start,axis3)=noSlipWall;
      cg[3].boundaryCondition()(End  ,axis3)=noSlipWall;

      // lower polygon
      cg[4].boundaryCondition()(Start,axis1)=outflow1;
      cg[4].boundaryCondition()(End  ,axis1)=inflow1;
      cg[4].boundaryCondition()(Start,axis2)=noSlipWall;
      cg[4].boundaryCondition()(Start,axis3)=noSlipWall;
      cg[4].boundaryCondition()(End  ,axis3)=noSlipWall;

      // corner polygon
      cg[5].boundaryCondition()(Start,axis1)=inflow1;
      cg[5].boundaryCondition()(End  ,axis1)=inflow2;
      cg[5].boundaryCondition()(Start,axis2)=noSlipWall;
      cg[5].boundaryCondition()(Start,axis3)=noSlipWall;
      cg[5].boundaryCondition()(End  ,axis3)=noSlipWall;
    }
  else if (Ogfile=="Grids/cd.hdf")
    {  
      // CONVERGING-DIVERGING GEOMETRY (cd.hdf) => only inflow1 and outflow1 used
      // background grid
      cg[0].boundaryCondition()(Start,axis1)=outflow1;
      cg[0].boundaryCondition()(End  ,axis1)=inflow1;

      // upper polygon
      cg[1].boundaryCondition()(Start,axis1)=outflow1;
      cg[1].boundaryCondition()(End  ,axis1)=inflow1;
      cg[1].boundaryCondition()(Start,axis2)=noSlipWall;

      // lower polygon
      cg[2].boundaryCondition()(Start,axis1)=outflow1;
      cg[2].boundaryCondition()(End  ,axis1)=inflow1;
      cg[2].boundaryCondition()(Start,axis2)=noSlipWall2;
    }
  else if (Ogfile=="Grids/trap3D.hdf")
    {
      // ENTROPIC TRAP 3D GEOMETRY  => only inflow1 and outflow1 used
       // background grid
      cg[0].boundaryCondition()(Start,axis1)=outflow1;
      cg[0].boundaryCondition()(End  ,axis1)=inflow1;
      cg[0].boundaryCondition()(Start,axis3)=noSlipWall;
      cg[0].boundaryCondition()(End  ,axis3)=noSlipWall;
      // upper polygon
      cg[1].boundaryCondition()(Start,axis1)=outflow1;
      cg[1].boundaryCondition()(End  ,axis1)=inflow1;
      cg[1].boundaryCondition()(Start,axis2)=noSlipWall2; // upper wall
      cg[1].boundaryCondition()(Start,axis3)=noSlipWall;
      cg[1].boundaryCondition()(End,axis3)=noSlipWall;
      // lower polygon
      cg[2].boundaryCondition()(Start,axis1)=outflow1;
      cg[2].boundaryCondition()(End  ,axis1)=inflow1;
      cg[2].boundaryCondition()(Start,axis2)=noSlipWall; // lower wall
      cg[2].boundaryCondition()(Start,axis3)=noSlipWall;
      cg[2].boundaryCondition()(End,axis3)=noSlipWall;
    }
  else if (Ogfile=="Grids/pipe2D.hdf")
    {
      // PARALLEL-WALLED 2D GEOMETRY (pipe2D.hdf)  => only inflow1 and outflow1 used
      // background grid
      cg[0].boundaryCondition()(Start,axis1)=outflow1;
      cg[0].boundaryCondition()(End  ,axis1)=inflow1;
      cg[0].boundaryCondition()(Start,axis2)=noSlipWall; // wall fixed / moving
      cg[0].boundaryCondition()(End  ,axis2)=noSlipWall2; // wall fixed / moving
    }
  else if (Ogfile=="Grids/pipe3D.hdf")
    {
      // PARALLEL-WALLED 3D GEOMETRY (pipe3D.hdf)  => only inflow1 and outflow1 used
      // background grid
      cg[0].boundaryCondition()(Start,axis1)=outflow1;
      cg[0].boundaryCondition()(End  ,axis1)=inflow1;
      cg[0].boundaryCondition()(Start,axis2)=noSlipWall; // wall fixed / moving
      cg[0].boundaryCondition()(End  ,axis2)=noSlipWall2; // wall fixed / moving
      cg[0].boundaryCondition()(Start,axis3)=noSlipWall;
      cg[0].boundaryCondition()(End  ,axis3)=noSlipWall2;
    }
  else if (Ogfile=="Grids/cylArray.hdf")
    {
      int NoOfCyls=4;
      //      Array of cylinders   => only inflow1 and outflow1 used
      // background grid
      cg[0].boundaryCondition()(Start,axis1)=outflow1;
      cg[0].boundaryCondition()(End  ,axis1)=inflow1;
      cg[0].boundaryCondition()(Start,axis2)=noSlipWall; // wall fixed / moving
      cg[0].boundaryCondition()(End  ,axis2)=noSlipWall; // wall fixed / moving
      cg[0].boundaryCondition()(Start,axis3)=noSlipWall;
      cg[0].boundaryCondition()(End  ,axis3)=noSlipWall;
      // cylinders (4x)
      for(int cyl=1; cyl<=NoOfCyls; cyl++)
	{
	  cg[cyl].boundaryCondition()(Start,axis2)=noSlipWall;
	  cg[cyl].boundaryCondition()(End  ,axis2)=noSlipWall; 
	  cg[cyl].boundaryCondition()(Start,axis3)=noSlipWall3; // inner cylinder wall
	}
    }
  else
    {
      cerr << "\nGrid not recognized. Update import_grid() routine! Aborting now...\n";
      abort();
    }
}

void import_CONFIG(doubleArray & position, doubleArray & velocity, int sim)
{
  using namespace Main_misc;
  int index;
  // Note!: This routine needs to be redone with string class (char array too cumbersome!)
  position=0; velocity=0; // initialize to zero

  char file_name[128]; char *prefix="CONFIG";
  if(sim==0)
    strcpy(file_name,prefix); // file_name <- CONFIG
  else
    snprintf(file_name, sizeof(file_name), "%s%d", prefix, sim); // file <- CONFIG<sim>

  printf("\nReading %s for initial positions and/or velocities...\n", file_name);
  ifstream config(file_name); // open CONFIG file
  if (!config.is_open())   // open attempt failed
    {
      cout << "\nFile not found; reading CFGMIN instead...\n";
      config.clear(); config.close(); // disconnect file
      config.open("CFGMIN"); // open CONFIG file instead
    }
  config.get(buffer,80); // skip 1st line (label)...
  config >> buffer;   // read next word

  while (!isalpha(buffer[0]) || buffer[0] == 'N' ) // read until you get to the 1st atom name
    {
      config >> buffer;   // read over all cell vectors
    }
  while (config.good()) // while open attempt success
    {
      config >> index;   // read atom index
      config >> buffer; // read next word

      if(!isalpha(buffer[1]) && !config.eof()) // while reading digits and not at eof...
	{                                      // convert Cstring to double and save to array
	  position(index-1,0) = atof(buffer); // xxx
	  config >> buffer; 
	  if(!isalpha(buffer[1]) && !config.eof())
	    {
	      position(index-1,1) = atof(buffer); // yyy
	      config >> buffer; 
	      if(!isalpha(buffer[1]) && !config.eof())
		{
		  if (Dim == 3) position(index-1,2) = atof(buffer); // zzz
		  config >> buffer; 
		  if(!isalpha(buffer[1]) && !config.eof())
		    {
		      velocity(index-1,0) = atof(buffer); // vxx
		      config >> buffer; 
		      if(!isalpha(buffer[1]) && !config.eof())
			{
			  velocity(index-1,1) = atof(buffer); //vyy
			  config >> buffer;  			      
			  if(!isalpha(buffer[1]) && !config.eof())
			    {
			      if (Dim == 3) velocity(index-1,2) = atof(buffer); //vzz
			      config >> buffer; 
			    }
			}
		    }
		}
	    }
	}
    }

  if(sim>0)  // delete CONFIG<sim> files, keep only the first one (CONFIG) for DLPOLY
    if( remove( file_name ) != 0 )
      perror( "Error deleting file.\n" );
}

// Generate a uniform random number
extern float ranf()         /* ranf() is uniform in range -1..1 */
{
  return(2*(rand()/(1.0+RAND_MAX))-1. );
}

// Transform uniform rnd to Gaussian using Box Muller transform (fast version, algorithm by Dr. Everett (Skip) Carter, Jr.)
float box_muller(float m, float s)	/* normal random variate generator */
{				        /* mean m, standard deviation s */
  float x1, x2, w, y1;
  static float y2;
  static int use_last;
  
  do{
    use_last = 0;
    if (use_last)		        /* use value from previous call */
      {
	y1 = y2;
	use_last = 0;
      }
    else
      {
	do {
	  x1 = 2.0 * ranf() - 1.0;
	  x2 = 2.0 * ranf() - 1.0;
	  w = x1 * x1 + x2 * x2;
	} while ( w >= 1.0 );
	
	w = sqrt( (-2.0 * log( w ) ) / w );
	y1 = x1 * w;
	y2 = x2 * w;
	use_last = 1;
      }
  } while ( abs(m + y1 * s) > 3*s); // cap to 3*sigma (99.7%)
  
  return( m + y1 * s );
}

// Quicksort routines (Partition and Quick_sort)
int Partition(int low,int high,double arr[])
{ int i; double high_vac,low_vac, pivot;
  pivot=arr[low];
  while(high>low)
    { high_vac=arr[high];

      while(pivot<high_vac)
	{
	  if(high<=low) break;
	  high--;
	  high_vac=arr[high];
	}

      arr[low]=high_vac;
      low_vac=arr[low];
      while(pivot>low_vac)
	{
	  if(high<=low) break;
	  low++;
	  low_vac=arr[low];
	}
      arr[high]=low_vac;
    }
  arr[low]=pivot;
  return low;
}

void Quick_sort(int low,int high,double arr[])
{
  int Piv_index,i;
  if(low<high)
    {
      Piv_index=Partition(low,high,arr);
      Quick_sort(low,Piv_index-1,arr);
      Quick_sort(Piv_index+1,high,arr);
    }
}

string StringToLower(string strToConvert)
{//change each element of the string to lower case
  for(unsigned int i=0;i<strToConvert.length();i++)
    {
      strToConvert[i] = tolower(strToConvert[i]);
    }
  return strToConvert;//return the converted string
}

void import_INI(aString nameOfINIFile, int rank)
{
  using namespace Data_ini;
  dictionary * ini ; // INI dictionary/map (sec_name:var_name = var_value)
  const int undef=123456; // unique error code
  const char * nondef="not defined"; // unique error string
  aString warning="format unknown or missing. Aborting now.\n";

  ini = iniparser_load(nameOfINIFile); // load INI file
  if (ini==NULL) {
    cout << "Cannot parse file " <<  nameOfINIFile << ". Aborting now.\n";
    abort();
  }
  cout << "\nParsing " << nameOfINIFile << "...";

  // Import BOOLEANs
  if( (Momentum_transfer=iniparser_getboolean(ini, "code:Momentum_transfer", undef)) == undef ) {cerr << "Momentum_transfer: " << warning; abort();}
  if( (False_transients=iniparser_getboolean(ini, "code:False_transients", undef)) == undef ) {cerr << "False_transients: " << warning; abort();}
  if( (BD=iniparser_getboolean(ini, "code:BD", undef)) == undef ) {cerr << "BD: " << warning; abort();}
  if( (P_Oseen=iniparser_getboolean(ini, "code:P_Oseen", undef)) == undef ) {cerr << "P_Oseen: " << warning; abort();}
  if( (Tracers=iniparser_getboolean(ini, "code:Tracers", undef)) == undef ) {cerr << "Tracers: " << warning; abort();}
  if( (Electrophoretic=iniparser_getboolean(ini, "code:Electrophoretic", undef)) == undef ) {cerr << "Electrophoretic: " << warning; abort();}
  if( (Dielectrophoretic=iniparser_getboolean(ini, "code:Dielectrophoretic", undef)) == undef ) {cerr << "Dielectrophoretic: " << warning; abort();}
  if( (Molecular_dynamics=iniparser_getboolean(ini, "code:Molecular_dynamics", undef)) == undef ) {cerr << "Molecular_dynamics: " << warning; abort();}
  if( (Electroosmotic=iniparser_getboolean(ini, "code:Electroosmotic", undef)) == undef ) {cerr << "Electroosmotic: " << warning; abort();}
  if( (Shear_driven=iniparser_getboolean(ini, "code:Shear_driven", undef)) == undef ) {cerr << "Shear_driven: " << warning; abort();}
  if( (Pressure_driven=iniparser_getboolean(ini, "code:Pressure_driven", undef)) == undef ) {cerr << "Pressure_driven: " << warning; abort();}
  if( (Electric_field=iniparser_getboolean(ini, "code:Electric_field", undef)) == undef ) {cerr << "Electric_field: " << warning; abort();}
  if( (Plot_intermediate=iniparser_getboolean(ini, "code:Plot_intermediate", undef)) == undef ) {cerr << "Plot_intermediate: " << warning; abort();}
  if( (Plot_final=iniparser_getboolean(ini, "code:Plot_final", undef)) == undef ) {cerr << "Plot_final: " << warning; abort();}
  if( (Electrostatics=iniparser_getboolean(ini, "code:Electrostatics", undef)) == undef ) {cerr << "Electrostatics: " << warning; abort();}
  if( (Export_vmd=iniparser_getboolean(ini, "code:Export_vmd", undef)) == undef ) {cerr << "Export_vmd: " << warning; abort();}
  if( (Com=iniparser_getboolean(ini, "code:Com", undef)) == undef ) {cerr << "Com: " << warning; abort();}
  if( (Msrgyr=iniparser_getboolean(ini, "code:Msrgyr", undef)) == undef ) {cerr << "Msrgyr: " << warning; abort();}
  if( (Mseed=iniparser_getboolean(ini, "code:Mseed", undef)) == undef ) {cerr << "Mseed: " << warning; abort();}
  if( (Stretch=iniparser_getboolean(ini, "code:Stretch", undef)) == undef ) {cerr << "Stretch: " << warning; abort();}
  if( (MsdVaf=iniparser_getboolean(ini, "code:MsdVaf", undef)) == undef ) {cerr << "MsdVaf: " << warning; abort();}
  if( (Rdf=iniparser_getboolean(ini, "code:Rdf", undef)) == undef ) {cerr << "Rdf: " << warning; abort();}

  // Import INTs
  if( (Vdw=iniparser_getint(ini, "code:Vdw", undef)) == undef ) {cerr << "Vdw: " << warning; abort();}
  if( (Wall=iniparser_getint(ini, "code:Wall", undef)) == undef ) {cerr << "Wall: " << warning; abort();}
  if( (Debug=iniparser_getint(ini, "code:Debug", undef)) == undef ) {cerr << "Debug: " << warning; abort();}
  if( (SpringLaw=iniparser_getint(ini, "physics:SpringLaw", undef)) == undef ) {cerr << "SpringLaw: " << warning; abort();}
  if( (LengthScale=iniparser_getint(ini, "physics:LengthScale", undef)) == undef ) {cerr << "LengthScale: " << warning; abort();}
  if( (TimeScale=iniparser_getint(ini, "physics:TimeScale", undef)) == undef ) {cerr << "TimeScale: " << warning; abort();}
  if( (ForceScale=iniparser_getint(ini, "physics:ForceScale", undef)) == undef ) {cerr << "ForceScale: " << warning; abort();}
  if( (OrderOfAccuracy=iniparser_getint(ini, "code:OrderOfAccuracy", undef)) == undef ) {cerr << "OrderOfAccuracy: " << warning; abort();}
  if( (NoOfParticles=iniparser_getint(ini, "physics:NoOfParticles", undef)) == undef ) {cerr << "NoOfParticles: " << warning; abort();}
  if( (NoOfTimeStepsT=iniparser_getint(ini, "code:NoOfTimeStepsT", undef)) == undef ) {cerr << "NoOfTimeStepsT: " << warning; abort();}
  if( (NoOfTimeSteps=iniparser_getint(ini, "code:NoOfTimeSteps", undef)) == undef ) {cerr << "NoOfTimeSteps: " << warning; abort();}
  if( (NoOfSims=iniparser_getint(ini, "code:NoOfSims", undef)) == undef ) {cerr << "NoOfSims: " << warning; abort();}
  if( (N1=iniparser_getint(ini, "code:N1", undef)) == undef ) {cerr << "N1: " << warning; abort();}
  if( (N2=iniparser_getint(ini, "code:N2", undef)) == undef ) {cerr << "N2: " << warning; abort();}
  if( (N3=iniparser_getint(ini, "code:N3", undef)) == undef ) {cerr << "N3: " << warning; abort();}
  if( (Mode=iniparser_getint(ini, "physics:Mode", undef)) == undef ) {cerr << "Mode: " << warning; abort();}
  if( (StepMax=iniparser_getint(ini, "code:StepMax", undef)) == undef ) {cerr << "StepMax: " << warning; abort();}
  if( (ErrorFreq=iniparser_getint(ini, "code:ErrorFreq", undef)) == undef ) {cerr << "ErrorFreq: " << warning; abort();}
  if( (MTFreq=iniparser_getint(ini, "code:MTFreq", undef)) == undef ) {cerr << "MTFreq: " << warning; abort();}
  if( (Seed=iniparser_getint(ini, "physics:Seed", undef)) == undef ) {cerr << "Seed: " << warning; abort();}
  if( (Direction=iniparser_getint(ini, "code:Direction", undef)) == undef ) {cerr << "Direction: " << warning; abort();}
  if( (NoOfShells=iniparser_getint(ini, "code:NoOfShells", undef)) == undef ) {cerr << "NoOfShells: " << warning; abort();}
  if( (ExportFreq=iniparser_getint(ini, "code:ExportFreq", undef)) == undef ) {cerr << "ExportFreq: " << warning; abort();}

  // Import FLOATING POINT NUMBERS
  if( (BOND=iniparser_getdouble(ini, "physics:BOND", undef)) == undef ) {cerr << "BOND: " << warning; abort();}
  if( (ALPHA=iniparser_getdouble(ini, "physics:ALPHA", undef)) == undef ) {cerr << "ALPHA: " << warning; abort();}
  if( (THETA=iniparser_getdouble(ini, "physics:THETA", undef)) == undef ) {cerr << "THETA: " << warning; abort();}
  if( (ks=iniparser_getdouble(ini, "physics:ks", undef)) == undef ) {cerr << "ks: " << warning; abort();}
  if( (bk=iniparser_getdouble(ini, "physics:bk", undef)) == undef ) {cerr << "bk: " << warning; abort();}
  if( (q0=iniparser_getdouble(ini, "physics:q0", undef)) == undef ) {cerr << "q0: " << warning; abort();}
  if( (vdw_epsilon=iniparser_getdouble(ini, "physics:vdw_epsilon", undef)) == undef ) {cerr << "vdw_epsilon: " << warning; abort();}
  if( (vdw_sigma=iniparser_getdouble(ini, "physics:vdw_sigma", undef)) == undef ) {cerr << "vdw_sigma: " << warning; abort();}
  if( (dtT=iniparser_getdouble(ini, "code:dtT", undef)) == undef ) {cerr << "dtT: " << warning; abort();}
  if( (dt=iniparser_getdouble(ini, "code:dt", undef)) == undef ) {cerr << "dt: " << warning; abort();}
  if( (dt_d=iniparser_getdouble(ini, "code:dt_d", undef)) == undef ) {cerr << "dt_d: " << warning; abort();}
  if( (dt_u=iniparser_getdouble(ini, "code:dt_u", undef)) == undef ) {cerr << "dt_u: " << warning; abort();}
  if( (dt_v=iniparser_getdouble(ini, "code:dt_v", undef)) == undef ) {cerr << "dt_v: " << warning; abort();}
  if( (eps_d=iniparser_getdouble(ini, "code:eps_d", undef)) == undef ) {cerr << "eps_d: " << warning; abort();}
  if( (eps_u=iniparser_getdouble(ini, "code:eps_u", undef)) == undef ) {cerr << "eps_u: " << warning; abort();}
  if( (eps_v=iniparser_getdouble(ini, "code:eps_v", undef)) == undef ) {cerr << "eps_v: " << warning; abort();}
  if( (debye=iniparser_getdouble(ini, "physics:debye", undef)) == undef ) {cerr << "debye: " << warning; abort();}
  if( (epsilon=iniparser_getdouble(ini, "physics:epsilon", undef)) == undef ) {cerr << "epsilon: " << warning; abort();}
  if( (eta=iniparser_getdouble(ini, "physics:eta", undef)) == undef ) {cerr << "eta: " << warning; abort();}
  if( (rho=iniparser_getdouble(ini, "physics:rho", undef)) == undef ) {cerr << "rho: " << warning; abort();}
  if( (temp=iniparser_getdouble(ini, "physics:temp", undef)) == undef ) {cerr << "temp: " << warning; abort();}
  if( (dep_corr=iniparser_getdouble(ini, "physics:dep_corr", undef)) == undef ) {cerr << "dep_corr: " << warning; abort();}
  if( (height=iniparser_getdouble(ini, "physics:height", undef)) == undef ) {cerr << "height: " << warning; abort();}
  if( (i_wall=iniparser_getdouble(ini, "physics:i_wall", undef)) == undef ) {cerr << "i_wall: " << warning; abort();}
  if( (i_wall2=iniparser_getdouble(ini, "physics:i_wall2", undef)) == undef ) {cerr << "i_wall2: " << warning; abort();}
  if( (a_inflow1=iniparser_getdouble(ini, "physics:a_inflow1", undef)) == undef ) {cerr << "a_inflow1: " << warning; abort();}
  if( (a_inflow2=iniparser_getdouble(ini, "physics:a_inflow2", undef)) == undef ) {cerr << "a_inflow2: " << warning; abort();}
  if( (a_outflow1=iniparser_getdouble(ini, "physics:a_outflow1", undef)) == undef ) {cerr << "a_outflow1: " << warning; abort();}
  if( (a_outflow2=iniparser_getdouble(ini, "physics:a_outflow2", undef)) == undef ) {cerr << "a_outflow2: " << warning; abort();}
  if( (charge_p=iniparser_getdouble(ini, "physics:charge_p", undef)) == undef ) {cerr << "charge_p: " << warning; abort();}
  if( (rad_p=iniparser_getdouble(ini, "physics:rad_p", undef)) == undef ) {cerr << "rad_p: " << warning; abort();}
  if( (mass_p=iniparser_getdouble(ini, "physics:mass_p", undef)) == undef ) {cerr << "mass_p: " << warning; abort();}
  if( (lambda=iniparser_getdouble(ini, "physics:lambda", undef)) == undef ) {cerr << "lambda: " << warning; abort();}
  if( (shear_rate=iniparser_getdouble(ini, "physics:shear_rate", undef)) == undef ) {cerr << "shear_rate: " << warning; abort();}
  if( (pressure=iniparser_getdouble(ini, "physics:pressure", undef)) == undef ) {cerr << "pressure: " << warning; abort();}
  if( (Aw=iniparser_getdouble(ini, "physics:Aw", undef)) == undef ) {cerr << "Aw: " << warning; abort();}
  if( (com_ref_x=iniparser_getdouble(ini, "physics:com_ref_x", undef)) == undef ) {cerr << "com_ref_x: " << warning; abort();}
  if( (com_ref_y=iniparser_getdouble(ini, "physics:com_ref_y", undef)) == undef ) {cerr << "com_ref_y: " << warning; abort();}
  if( (com_ref_z=iniparser_getdouble(ini, "physics:com_ref_z", undef)) == undef ) {cerr << "com_ref_z: " << warning; abort();}
  if( (equil_frac=iniparser_getdouble(ini, "code:equil_frac", undef)) == undef ) {cerr << "equil_frac: " << warning; abort();}
  if( (nof=iniparser_getdouble(ini, "code:nof", undef)) == undef ) {cerr << "nof: " << warning; abort();}
  if( (friction_corr=iniparser_getdouble(ini, "code:friction_corr", undef)) == undef ) {cerr << "friction_corr: " << warning; abort();}
  if( (v_ev=iniparser_getdouble(ini, "physics:v_ev", undef)) == undef ) {cerr << "v_ev: " << warning; abort();}

  // Import STRINGS
  if( (nameOfOGFile=iniparser_getstring(ini, "code:nameOfOGFile", NULL)) == NULL ) {cerr << "nameOfOGFile: " << warning; abort();}
  if( (outputSuffix=iniparser_getstring(ini, "code:outputSuffix", NULL)) == NULL ) {cerr << "outputSuffix: " << warning; abort();}

  cout << "done.\n\n *** IMPORTED VARIABLES *** \n";
#ifdef USE_MPI
if(rank==0) // only root does I/O...
{
#endif 
  iniparser_dump(ini, stdout); // print out the dictionary
#ifdef USE_MPI
}
#endif
  iniparser_freedict(ini); // free memory

}

void calculate_RDF(const doubleArray & position, doubleArray & RDF, int NoOfShells)
{
  using namespace Data_ini;  // use data.ini vars
  using namespace Main_misc; // use misc vars

  //doubleArray RDF(NoOfShells); RDF=0. // init -> use outside

  //////////////////////////////// RADIAL DISTRIBUTION FUNCION - START ////////////////////////////////
  // Calculate radial distribution function (RDF)
  // 1) Fill up the distance matrix (condensed into a vector)
  cout << " \nCalculating RDF...\n";
  int index, N=NoOfParticles*(NoOfParticles-1)/2; // # of distinct pairs of atoms
  Range particle_pair_Range(N), time_Range(0,NoOfTimeSteps-1), all;
  doubleArray distance(particle_pair_Range,time_Range), // distance matrix (condensed to vector)
    distance_sorted(particle_pair_Range,time_Range); // sorted distance matrix

  cout << " \nRDF: Setting up distance matrix...";
  for (step=0; NoOfParticles>1 && step<NoOfTimeSteps; step++)
    {
      index=0;
      for (i=0; i<NoOfParticles-1; i++) // loop over all pairs of atoms (i,j)
	{
	  for (j=i+1; j<NoOfParticles; j++,index++) // include only distinct pairs
	    {
	      distance(index,step)=sqrt(SQR(position(i,0,step)-position(j,0,step))+SQR(position(i,1,step)-position(j,1,step))); // distance (Rij)
	    }
	}
    }

  // 2) Quicksort to speed up subsequent histogramming
  double *array; int low,high;
  array=new double[N]; // allocate dynamic array

  cout << " \nRDF: Quicksort...";
  for(j=1;j<NoOfTimeSteps;j++) // loop over time steps
    {
      for(i=0;i<N;i++) // loop over particle pairs
	array[i]=distance(i,j); // extract current time-level data **ph080207: why doesn't work for j=0?!

      high=N-1; low=0; Quick_sort(low,high,array); // call Quicksort

      for(i=0;i<N;i++) // loop over particle pairs
	distance_sorted(i,j)=array[i]; // save sorted array
    }
  
  // 3) Create spherical shells with thickness delta_r
  double delta_r=max(distance)/NoOfShells; // largest shell determined by maximum inter-particle distance
  cout << " \nRDF: Creating " << NoOfShells << " shells...";
  Range shell_Range(NoOfShells);
  doubleArray 
    shell_r(shell_Range), // shell thicknesses (r)
    shell_V(shell_Range), // shell volumes (V)
    histogram(shell_Range, time_Range); // local density (# of atoms in a spherical shell), 
  RDF.display("\nRDF before:");

  for (k=0; k<NoOfShells; k++) // fill in shell arrays (r,V)
    {
      shell_r(k)=k*delta_r;
      shell_V(k)=4./3*Pi*pow(shell_r(k)+delta_r,3)-4./3*Pi*pow(shell_r(k),3);
    }

  // 4) Create the histogram
  cout << " \nRDF: Sorting into bins...";
  for(j=1;j<NoOfTimeSteps;j++) // loop over time steps
    {
      i=0; // reset counter
      for(k=0;k<NoOfShells;k++) // loop over spherical shells
	{
	  while(distance_sorted(i,j) < shell_r(k) && i < N) // sort into bins, fast linear search without repetitive scanning thanks to pre-sorting
	    {
	      histogram(k,j)++; i++; // number of atoms found in the sperical shell 'k' at time 'j'
	    }
	}
    }
  for(k=0;k<NoOfShells;k++) // collect RDF terms
    RDF(k)=2*sum(histogram(k,all))/NoOfTimeSteps/NoOfParticles/shell_V(k); // RDF (J.M.Haile, MD Simulation)
  //////////////////////////////// RADIAL DISTRIBUTION FUNCION - END ////////////////////////////////
}

void ann(ANNkd_tree* kdTree, const doubleArray & position, const doubleArray & Dwall, doubleArray & Fwall, realArray & domain_min, realArray & domain_max, bool ProductionPhase)
{
  using namespace Data_ini;  // use data.ini vars
  using namespace Main_misc; // use misc vars

  // parameters
  int			k			= 1;		// number of nearest neighbors
  double		eps			= 0;		// error bound
  ANNpoint		queryPt;				// query point
  ANNidxArray		nnIdx;					// near neighbor indices
  ANNdistArray		dists;					// near neighbor distances
  int idx;
  double diff;

  // allocate space
  queryPt = annAllocPt(Dim);					// allocate query point
  nnIdx = new ANNidx[k];					// allocate near neigh indices
  dists = new ANNdist[k];					// allocate near neighbor dists

  ForParticles(particle)
  {
//     // if array periodic in x-dir, shift accordingly
//     float p=1.; // period
//     position(particle,0)-=int(position(particle,0)/p);

    // read query points
    ForDims(dim) // input positions
      queryPt[dim]=position(particle,dim); 
    
    kdTree->annkSearch(						// search
		       queryPt,					// query point
		       k,					// number of near neighbors
		       nnIdx,					// nearest neighbors (returned)
		       dists,					// distance (returned)
		       eps);					// error bound

    for (i = 0; i < k; i++) // loop over all nearest neigbors and find their field values
      {		
	idx = nnIdx[i]; // index of the record

	// *** DIRECT DISTANCE CALCULATION FOR A PIPE *** (START)
	NDist(position(particle,0), position(particle,1), position(particle,2), domain_max(0), domain_max(1), domain_max(2), Dwall(idx,0), Dwall(idx,1), Dwall(idx,2), Dwall(idx,3), ProductionPhase);
	// *** DIRECT DISTANCE CALCULATION FOR A PIPE *** (END)

	if (Wall == 1)
	  {
	    // 1) Jednrejack potential
	    diff = Dwall(idx,0)-delta_w; // y-delta_w
	    if (diff<0) // if close to the wall...
	      {
		ForDims(dim)
		{
		  Fwall(particle,dim)=C_wall*diff*diff*Dwall(idx,dim+1);
		}
	      }
	    else 
	      Fwall(particle,all)=0.;
	  }

	else if (Wall == 2)
	  {
	    // 2) 9-3 potential
	    diff = Dwall(idx,0);
	    if (diff<delta_w)
	      {
		ForDims(dim)
		{
		  //Fwall(particle,dim)=A_wall/B_wall*(9.*pow(diff/B_wall,-10) - 3.*pow(diff/B_wall,-4))*Dwall(idx,dim+1); // full (requires time step ~ 100 fs)
		  Fwall(particle,dim)=A_wall/B_wall*(9.*pow(diff/B_wall,-10) )*Dwall(idx,dim+1); // repulsive part only
		}
	      }
	    else 
		Fwall(particle,all)=0.;
	  }
	
	else if (Wall == 3)
	  {
	    // 3) 10-4 potential
	    diff = Dwall(idx,0);
	    if (diff<delta_w)
	      {
		ForDims(dim)
		  //Fwall(particle,dim)=A_wall/B_wall*(10.*pow(diff/B_wall,-11) - 4.*pow(diff/B_wall,-5))*Dwall(idx,dim+1);
		  Fwall(particle,dim)=A_wall/B_wall*(10.*pow(diff/B_wall,-11) )*Dwall(idx,dim+1); // repulsive part only
	      }
	    else 
	      Fwall(particle,all)=0.;
	  }

	// (Wall == 4) is a discrete wall

	else if (Wall == 5)
	  {
	    // 4) L-J potential
	    diff = Dwall(idx,0);

	    ForDims(dim)
	      Fwall(particle,dim)=A_wall/B_wall*(12.*pow(diff/B_wall,-13) - 6.*pow(diff/B_wall,-7))*Dwall(idx,dim+1);
	  }

	if (Debug & 2)
	  {
// 	    cout << "\nParticle="<<particle<<", NN="<<i<<", index="<<nnIdx[i]<<", distance="<<sqrt(dists[i]);
// 	    ForDims(dim) // input positions
// 	    {
// 	      cout <<"\nQuery ("<<dim<<"): "<<queryPt[dim];
// 	      cout <<"\nFwall ("<<dim<<"): "<<Fwall(particle,dim);
// 	    }
	    printf("\nWALL: step=%i, particle=%i, pos=(%e,%e,%e), diff<delta_w=%i, walldist=%e, derivatives=(%e,%e,%e), force=(%e,%e,%e)\n", step, particle, position(particle,0), position(particle,1), position(particle,2), Dwall(idx,0)<delta_w, Dwall(idx,0), Dwall(idx,1), Dwall(idx,2), Dwall(idx,3), Fwall(particle,0), Fwall(particle,1), Fwall(particle,2) );
	  }

      }
  }
  delete [] nnIdx;						// clean things up
  delete [] dists;
}

//******************* POLYMER PROCESSING  *******************

void ppInit (doubleArray & PPD, doubleArray & PPS, doubleArray & PPSIM, doubleArray & x0, doubleArray& v0, doubleArray & norm)
{
  using namespace PP;

  // do not set PPSIM to zero!
  PPD=0.; PPS=0.; 
  x0=0.; v0=0.; norm=0.; Norg=0;
}

void ppUpdate(doubleArray & PPD, doubleArray & PPS, doubleArray & PPSIM, const int prodstep, const doubleArray & position, const doubleArray & velocity, doubleArray & x0, doubleArray & v0, doubleArray & N0, doubleArray & norm, int & TransStatus, realArray & domain_min, realArray & domain_max, doubleArray & com, doubleArray & vcom, doubleArray & eev)
{
  using namespace Data_ini;
  using namespace Main_misc;
  using namespace PP;

  // Calculate Center Of Mass - position (com) and velocity (vcom)
  // Note: COM is needed for other quantities below, so it has to be calculated for each realization, unlike the other quantities, which are not evaluated until the end of the major loop. However, we also need an average COM for pdf plot.
  time00=getCPU();
  com=vcom=0.;
  ForParticlesAndDims(particle,dim)
    {
      com(dim)+=position(particle,dim);
      vcom(dim)+=velocity(particle,dim);
    }
  com/=NoOfParticles; // average
  vcom/=NoOfParticles;

  if(Com)
    {
      ForDims(dim)
      {
	COM(dim,prodstep)+=com(dim);
	VCOM_AVG(dim)+=vcom(dim);
//       //*** MSV *** (START)
// 	 VCOM_AVG(dim)+=vcom(dim)*vcom(dim); // if mean-square velocity (MSV) needed (for VAF decay test)
//       //*** MSV *** (END)
      }
    }
  time_COM+=(getCPU()-time00);

    if(Msrgyr)
      {
	// Calculate Mean-Square Radius of Gyration (MSRGYR)
	time00=getCPU();
	ForParticlesAndDims(particle,dim)
	  MSRGYR(prodstep)+=SQR( position(particle,dim)-com(dim) );
	time_MSRGYR+=(getCPU()-time00);
      }

    if(Mseed)
      {
	// Calculate Mean-Square End-to-End Distance (MSEED)
	time00=getCPU();
	eev=0;
	ForDims(dim)
	{
	  eev(dim)=position(NoOfParticles-1,dim)-position(0,dim); // end-to-end vector
	  MSEED(prodstep)+=eev(dim)*eev(dim);
	}
	time_MSEED+=(getCPU()-time00);
      }

    if(Stretch)
      {
	// Calculate Directional Stretch (STRETCH)
	time00=getCPU();
	STRETCH(prodstep)+=( max(position(all,Direction))-min(position(all,Direction)) );
	time_STRETCH+=(getCPU()-time00);
      }

    if(MsdVaf)
      {
	// Calculate MSD & VAF of center of mass
	time00=getCPU();
	if( prodstep % NT0 == 0 )
	  {
	    N0(Norg)=prodstep;
	    ForDims(dim) 
	    {
	      x0(dim,Norg)=com(dim);
	      v0(dim,Norg)=vcom(dim);
	    }
	    Norg+=1; 
	  }
	for(iorg=0; iorg<Norg; iorg++)
	  {
	    dorg=prodstep-N0(iorg);
	    if(dorg<NoOfProdSteps) 
	      {
		norm(dorg)+=1;
 		ForDims(dim)
 		{
    		  MSD(dorg)+=(com(dim)-x0(dim,iorg))*(com(dim)-x0(dim,iorg)); // *** 3D MSD
 		  VAF(dorg)+=vcom(dim)*v0(dim,iorg);
 		}

		// In-plane MSD (also, MSD=4*D_{yz}*dt)
//  		MSD(dorg)+=(com(1)-x0(1,iorg))*(com(1)-x0(1,iorg)); // *** 2D MSD 12
//    		MSD(dorg)+=(com(2)-x0(2,iorg))*(com(2)-x0(2,iorg)); // *** 2D MSD 12
// 		MSD(dorg)+=(com(0)-x0(0,iorg))*(com(0)-x0(0,iorg)); // *** 2D MSD 01
//   		MSD(dorg)+=(com(1)-x0(1,iorg))*(com(1)-x0(1,iorg)); // *** 2D MSD 01
// 		MSD(dorg)+=(com(0)-x0(0,iorg))*(com(0)-x0(0,iorg)); // *** 2D MSD 02
//   		MSD(dorg)+=(com(2)-x0(2,iorg))*(com(2)-x0(2,iorg)); // *** 2D MSD 02

// 		// Axial MSD (also, MSD=2*D_{x}*dt)
//		MSD(dorg)+=(com(1)-x0(1,iorg))*(com(1)-x0(1,iorg)); // *** 1D MSD 1
	      }
	  }
	time_MSD+=(getCPU()-time00);
      }

    if(1)
// ================================================================================
// INPUTS:
//    Dir ... direction of translocation (assume translocation proceeds in the positive direction)
// OUTPUTS:
//    Translocation status (based on position (pos) of chain's COM or a bead):
//    0 = not occuring (pos < min)
//    1 = in progress (min <= pos <= max)
//    2 = finished (pos > max)
//
// ================================================================================
      {
	// Translocation = at least one bead is inside the pore
	int cis=0; int trans=0;

	// cout << "\n[Status, TransTime BEFORE, cis, trans] = [" <<  TransStatus << ", " << TRANSLOC(0) << ", "<< cis << ", "<< trans << "]";

	ForParticles(particle) // count beads on 'cis' and 'trans' sides
	{
	  if ( position(particle,Direction) < domain_min(Direction))
	    cis++;
	  else if ( position(particle,Direction) > domain_max(Direction))
	    trans++;
 	  else
 	    break;
	}

	if ( cis == NoOfParticles ) // if all beads on cis side
	  {TransStatus=0; TRANSLOC(sim-1)=0;} // reset timer
	else if ( trans == NoOfParticles ) // if all beads on trans side
	  {TransStatus=2;} // set exit flag
	else // otherwise
	  {TransStatus=1; TRANSLOC(sim-1)++;} // increment timer
	
      }

}

void ppAverage(doubleArray & PPD, doubleArray & PPS, const double dt_nondim, const doubleArray & norm)
{
  using namespace Data_ini;
  using namespace Main_misc;
  using namespace PP;

  if(MsdVaf)
    {
      ForProdSteps(prodstep)
      {
	MSD(prodstep)/=norm(prodstep);
	VAF(prodstep)/=norm(prodstep);
      }
      // Calculate D of center of mass
      // NOTE: For BD (integrated over velocity decay time scale), <VAF> will always be zero and therefore will yield nonsensical DIFF. Use MSD result instead.
      if(!BD)
	{
	  for(i=1; i<NoOfProdSteps; i++)
	    DIFF_VAF+=dt_nondim*0.5*(VAF(i)+VAF(i-1));
	  DIFF_VAF/=Dim;
	}
      // D=lim(t->inf) MSD(t)/(2*Dim*t)
      DIFF_MSD=MSD(NoOfProdSteps-1)/(2.*Dim*NoOfProdSteps*ExportFreq*dt_nondim); // *** 3D D
//       DIFF_MSD=MSD(NoOfProdSteps-1)/(2.*(Dim-1)*NoOfProdSteps*ExportFreq*dt_nondim); // *** 2D D
//       DIFF_MSD=MSD(NoOfProdSteps-1)/(2.*(Dim-2)*NoOfProdSteps*ExportFreq*dt_nondim); // *** 1D D
    }

  int NoOfActProdSteps=step-equil_frac*NoOfTimeSteps; // actual NoOfProdSteps = actual NoOfTimeSteps - NoOfEquilSteps(takes into account premature exits from timeloop)
  if(Com)
    {
    ForDims(dim) 
      VCOM_AVG(dim)*=u_0/NoOfActProdSteps; // vel. COM average [m/s] (for entropic trapping)
//       //*** MSV *** (START)
//       VCOM_AVG(dim)*=u_0*u_0/NoOfActProdSteps; // MSV (for VAF decay test)
//       //*** MSV *** (END)
    }

}

void ppExport(doubleArray & PPD, doubleArray & PPS, doubleArray & PPSIM, const doubleArray & time, const double dt_nondim, aString nameOfHDFFile, double std_diff)
{
  using namespace Data_ini;
  using namespace Main_misc; 
  using namespace PP;

  // *** PRINT-OUT A SUMMARY ***
//   cout.unsetf(ios_base::scientific); // format scientific -> fixed
//   cout.setf(ios_base:: fixed); cout.setf(ios_base::showpoint);
  cout << "\n\n *********** NORMALIZED POLYMER PROPERTIES - SUMMARY *************";

  // Average & normalize where needed, print out some averages
  if(Com)
    {
      //MSD(all).display("MSD:");
      cout << "\n Mean COM velocity [m/s]:    " << setw(10);
      ForDims(dim) printf("V(%i)=%e \t", dim, VCOM_AVG(dim));
//       //*** MSV *** (START)
//       cout << "\n Temperature [K]:    " << setw(10); // use MSV to check temperature T
//       printf("%f", (VCOM_AVG(0)+VCOM_AVG(1)+VCOM_AVG(2))*mass_0/3./KB);
//       //*** MSV *** (END)
    }

  if(Msrgyr)
    {
      MSRGYR(all)/=NoOfParticles;
      MSRGYR(all)/=(q0*q0/L/L); // normalize by nondim q0 (or msrgyr)
      cout << "\n Mean square Rgyr:         " << setw(10) << sum(MSRGYR(all))/NoOfProdSteps;
    }
  
  if(Mseed)
    {
      MSEED(all)/=(q0*q0/L/L); // normalize by nondim q0 (or mseed)
      cout << "\n Mean square Re-e:         " << setw(10) << sum(MSEED(all))/NoOfProdSteps;
    }
  
  if(Stretch)
    {
      STRETCH(all)/=(NoOfParticles-1);
      STRETCH(all)/=(q0/L); // normalize by nondim contour length
      cout << "\n Stretch at final time:    " << setw(10) << STRETCH(NoOfProdSteps-1);
    }
  
  if(MsdVaf)
    {
      DIFF_MSD/=(Diff/NoOfParticles/L/L*Tau); // normalize by Rouse diffusivity
      DIFF_VAF/=(Diff/NoOfParticles/L/L*Tau);
      std_diff/=(Diff/NoOfParticles/L/L*Tau);
      if(!BD) cout << "\n Diff (VAF):               " << setw(10) << DIFF_VAF;
      cout << "\n Diff (MSD):               " << setw(10) << DIFF_MSD << " (std=" << std_diff << ")";

      double // Usta's xi_eff VS xi_0; assumes single bead & uniform grid spacing!
	D_chain=1e-12, // input chain diffusivity (experimental data) [m2/s]
	spacing_min=cg[0].gridSpacing(axis2),
	FRICTION=6*Pi*rad_p*eta*friction_corr, // Stokes friction constant (tuning parameter) [kg/s]
	FRICTION_CHAIN=KB*temp/(DIFF_MSD*Diff),
	x_usta=eta*spacing_min*L/FRICTION,
	y_usta=eta*spacing_min*L*DIFF_MSD*Diff/KB/temp,
        g_usta=y_usta-x_usta,
        zeta_bare_inv=D_chain*NoOfParticles/(KB*temp)-g_usta/(eta*spacing_min*L),
	fric_corr=pow( 6*Pi*rad_p*eta * zeta_bare_inv,-1);
      printf("\n (friction_corr (in), friction_corr (out) = ( %e, %e )", friction_corr, fric_corr);
      printf("\n Usta's friction relation: (x,y) = \n %e, %e, 'o', \n g = %e", x_usta, y_usta, g_usta);
      printf("\n A = %e, B = %e", 6*Pi*rad_p*eta * D_chain*NoOfParticles/(KB*temp), -6*Pi*rad_p*eta * g_usta/(eta*spacing_min) );

    }

  if(1)
    {
      TRANSLOC(all)*=ExportFreq*dt_nondim;
      //TRANSLOC(all).display("\nTranslocation times:");
      cout << "\n Mean T_transloc:          " << setw(10) << sum(TRANSLOC(all))/NoOfSims;
    }

  // Export to HDF
  printf("\n\n Exporting to HDF file...");
  ios::sync_with_stdio();
  HDF_DataBase root;
  root.mount(nameOfHDFFile,"I");

  root.put(L, "L"); // export some scales
  root.put(Tau, "Tau");
  root.put(u_0, "u_0");

  root.put(time, "Time"); // export time array [s]
  root.put(PPD, "PPD"); // exoport dynamic polymer properties matrix [normalized]
  root.put(PPS, "PPS"); // exoport static polymer properties matrix [normalized]
  root.put(PPSIM, "PPSIM");
  root.put(PPDrows, "PPDrows"); // export indices of individual properties so that we know what PPD row corresponds to which property
  root.put(PPSrows, "PPSrows"); // export indices of individual properties so that we know what PPS row corresponds to which property
  root.put(PPSIMrows, "PPSIMrows");

  root.unmount();  

}

//******************* FALSE-TRANSIENTS: general  *******************
int 
timeSteppingError(const realCompositeGridFunction & u, 
		  realCompositeGridFunction & uold,
		  const double epsilon)
// ================================================================================
// /Description:
//  Compute relative error in time-stepping integration
//=================================================================================
{
  using namespace Main_misc; // use misc vars

  double error=max(fabs(u-uold)); // stopping criterion (max norm)
  uold=u;
  if (error < epsilon) 
    {
      cout << "\nError within tolerance.\n";
      return 1;
    }
  printf("\nMax error = %e. Continuing...", error);

  return 0;
}

//******************* FALSE-TRANSIENTS: Navier-Stokes  *******************
int 
applyBoundaryConditionsNS( realCompositeGridFunction & u, real t)
// =================================================================================================
// /Description:
//      Apply boundary conditions for an overlapping grids with refinements.
// =================================================================================================
{
  using namespace Main_misc; // use misc vars
  
  CompositeGrid & cg = *u.getCompositeGrid();

  ForGrids(grid)
    u[grid].updateGhostBoundaries();  // update parallel ghost boundaries
  
  //checkArrayIDs("applyBC: before interpolate"); 
  real time0=getCPU();
  u.interpolate();        // this includes AMR interpolation.
  timeForSolveInterpolate+=getCPU()-time0;
  //checkArrayIDs("applyBC: after interpolate"); 

  // apply true boundary conditions.
  time0=getCPU();

	      ForGrids(grid) // loop over the component grids
	      {
		MappedGrid & mg = cg[grid]; // make a reference (alias) to save typing
		getIndex(mg.indexRange(),I1,I2,I3);

		// *** apply Boundary Conditions ***
		ForBoundary(side,axis)
		  {
		    if( mg.boundaryCondition()(side,axis)==inflow2 )
		      {
			u.applyBoundaryCondition(R1D,neumann,BCTypes::boundary,0.,t);  // R1D = index 1..Dim = [uc vc wc]
			u.applyBoundaryCondition(R0,dirichlet,BCTypes::boundary,BC_NS.p_inflow2,t); // R0 = index 0 = [pc]
			u.applyBoundaryCondition(R0,extrapolate,BCTypes::boundary,0.,t);
		      }
		    else if( mg.boundaryCondition()(side,axis)==inflow1 ) 
		      {
			u.applyBoundaryCondition(R1D,neumann,BCTypes::boundary,0.,t);
			u.applyBoundaryCondition(R0,dirichlet,BCTypes::boundary,BC_NS.p_inflow1,t);
			u.applyBoundaryCondition(R0,extrapolate,BCTypes::boundary,0.,t);
		      }
		    else if( mg.boundaryCondition()(side,axis)==outflow1 )
		      {
			u.applyBoundaryCondition(R1D,neumann,BCTypes::boundary,0.,t);
			u.applyBoundaryCondition(R0,dirichlet,BCTypes::boundary,BC_NS.p_outflow1,t);
			u.applyBoundaryCondition(R0,extrapolate,BCTypes::boundary,0.,t);
		      }
		    else if( mg.boundaryCondition()(side,axis)==outflow2 )
		      {
			u.applyBoundaryCondition(R1D,neumann,BCTypes::boundary,0.,t);
			u.applyBoundaryCondition(R0,dirichlet,BCTypes::boundary,BC_NS.p_outflow2,t);
			u.applyBoundaryCondition(R0,extrapolate,BCTypes::boundary,0.,t);
		      }
		    else if( mg.boundaryCondition()(side,axis)==noSlipWall  )
		      {
			u.applyBoundaryCondition(R1D,dirichlet,BCTypes::boundary,-BC_NS.v_wall,t);
			u.applyBoundaryCondition(R1D,extrapolate,BCTypes::boundary,0.,t);
			u.applyBoundaryCondition(R0,neumann,BCTypes::boundary,0.,t);
		      }
		    else if( mg.boundaryCondition()(side,axis)==noSlipWall2 || mg.boundaryCondition()(side,axis)==noSlipWall3 )
		      {
			u.applyBoundaryCondition(R1D,dirichlet,BCTypes::boundary,BC_NS.v_wall,t);
			u.applyBoundaryCondition(R1D,extrapolate,BCTypes::boundary,0.,t);
			u.applyBoundaryCondition(R0,neumann,BCTypes::boundary,0.,t);
		      }
// 		    //////////// NEUMANN VELOCITY WALLS

// 		    else if( mg.boundaryCondition()(side,axis)==noSlipWall  )
// 		      {
// 			u.applyBoundaryCondition(R1D,neumann,BCTypes::boundary,0.,t);
// 			u.applyBoundaryCondition(R0,dirichlet,BCTypes::boundary,BC_NS.p_outflow2,t);
// 			u.applyBoundaryCondition(R0,extrapolate,BCTypes::boundary,0.,t);
// 		      }
// 		    else if( mg.boundaryCondition()(side,axis)==noSlipWall2 || mg.boundaryCondition()(side,axis)==noSlipWall3 )
// 		      {

// 			u.applyBoundaryCondition(R1D,dirichlet,BCTypes::boundary,-BC_NS.v_wall,t);
// 			u.applyBoundaryCondition(R1D,extrapolate,BCTypes::boundary,0.,t);
// 			u.applyBoundaryCondition(R0,neumann,BCTypes::boundary,0.,t);
// 		      }
		    //////////// GENERALIZED DIVERGENCE
// 		    else if( mg.boundaryCondition()(side,axis)==noSlipWall  )
// 		      {
// 			u.applyBoundaryCondition(R1D,dirichlet,BCTypes::boundary,-BC_NS.v_wall);
// 			u.applyBoundaryCondition(R1D,extrapolate,BCTypes::boundary,0.);
// 			u.applyBoundaryCondition(R1D,generalizedDivergence, allBoundaries);
// 		      }
// 		    else if( mg.boundaryCondition()(side,axis)==noSlipWall2 || mg.boundaryCondition()(side,axis)==noSlipWall3 )
// 		      {
// 			u.applyBoundaryCondition(R1D,dirichlet,BCTypes::boundary,BC_NS.v_wall);
// 			u.applyBoundaryCondition(R1D,extrapolate,BCTypes::boundary,0.);
// 			u.applyBoundaryCondition(R1D,generalizedDivergence, allBoundaries);
// 		      }
		  }
	      } // end ForGrids

	      // fix up corners, periodic update:
	      u.finishBoundaryConditions();

	      timeForBoundaryConditions+=getCPU()-time0;
	      return 0;
}

int 
dudt( realCompositeGridFunction & u, realCompositeGridFunction & ut, realCompositeGridFunction & f, real t )
// ================================================================================
// /Description:
//  Compute derivatives
//=================================================================================
{
  using namespace Main_misc; // use misc vars

  real time0=getCPU();
  
  CompositeGrid & cg = *u.getCompositeGrid();
  CompositeGridOperators & op = *u.getOperators();
  Index I1,I2,I3;

  // pt = p.laplacian() - damping * u.divergence() - fg.divergence();
  // ut = u.laplacian() - p.gradient() + fg;
  
    int grid;
    ForGrids(grid)
    {
      MappedGrid & mg = cg[grid];
      getIndex(mg.gridIndexRange(),I1,I2,I3);

      // set up required arrays for serial & parallel version
      #ifdef USE_PPP
	realSerialArray utLocal; getLocalArrayWithGhostBoundaries(ut[grid],utLocal);
	realSerialArray uLocal;  getLocalArrayWithGhostBoundaries(u[grid],uLocal);
	realSerialArray fLocal;  getLocalArrayWithGhostBoundaries(f[grid],fLocal);

	bool ok=ParallelUtility::getLocalArrayBounds(u[grid],uLocal,I1,I2,I3);
	if( !ok ) continue;
      #else
        realSerialArray & utLocal = ut[grid];
        realSerialArray & uLocal = u[grid]; 
        realSerialArray & fLocal = f[grid]; 
      #endif

	realArray ulap(uLocal);
	realArray ux(uLocal);
	realArray uy(uLocal);
	realArray uz(uLocal);
 	realArray fg(fLocal);

	// evaluate required derivatives EFFICIENTLY
	op[grid].derivative(MappedGridOperators::laplacianOperator,uLocal,ulap,I1,I2,I3);
	op[grid].derivative(MappedGridOperators::xDerivative,uLocal,ux,I1,I2,I3);
	op[grid].derivative(MappedGridOperators::yDerivative,uLocal,uy,I1,I2,I3);
	if (Dim==3) op[grid].derivative(MappedGridOperators::zDerivative,uLocal,uz,I1,I2,I3);

	// compute dudt; u=[p vx vy vz]
	utLocal(I1,I2,I3,pc) = ulap(I1,I2,I3,pc) - damping*(ux(I1,I2,I3,uc) + uy(I1,I2,I3,vc)) - fg(I1,I2,I3,pc);
	utLocal(I1,I2,I3,uc) = ulap(I1,I2,I3,uc) - ux(I1,I2,I3,pc) + fg(I1,I2,I3,uc);
	utLocal(I1,I2,I3,vc) = ulap(I1,I2,I3,vc) - uy(I1,I2,I3,pc) + fg(I1,I2,I3,vc);
	if (Dim==3) 
	  {
	    utLocal(I1,I2,I3,pc)+= -damping*uz(I1,I2,I3,wc);
	    utLocal(I1,I2,I3,wc) = ulap(I1,I2,I3,wc) - uz(I1,I2,I3,pc) + fg(I1,I2,I3,wc);
	  }
    }

    timeForDudt+=getCPU()-time0;
    return 0;
}

int 
linearMultiStep(real & t, 
		real dt,
		realCompositeGridFunction & f,
		realCompositeGridFunction & u,  
		realCompositeGridFunction & u0, 
		realCompositeGridFunction & u1, 
		realCompositeGridFunction & u2)
// ================================================================================
// /Description:
//  Driver for a Linear Multistep Method
//=================================================================================
{
  using namespace Main_misc; 

  if (ft_step<3) // use Runge-Kutta to get first 3 iterations...
    {
      // Take a time step (Runge-Kutta, Euler)
      rungeKutta4(t,dt,f,u,u0,u1,u2);
      Deque.push_back(u0); // push u0=f(tn,yn) => fill in the queue iteratively
    }
  else // ...and continue with 4-step Adams-Bashforth
    { 
      adamsBashforth(t,dt,f,u,Deque[0],Deque[1],Deque[2],u2); // -> u, u2
      Deque.pop_front();         // remove from front
      Deque.push_back(u2);       // insert at back
    }
  return 0;
}

int 
adamsBashforth(real & t, 
	       real dt,
	       realCompositeGridFunction & f,
	       realCompositeGridFunction & u,  // y(n+3) -> y(n+4)
	       realCompositeGridFunction & u0, // f(tn,yn) 
	       realCompositeGridFunction & u1, // f(t(n+1),y(n+1))
	       realCompositeGridFunction & u2, // f(t(n+2),y(n+2))
	       realCompositeGridFunction & u3) // -> f(t(n+3),y(n+3))
// ================================================================================
// /Description:
//  4-step Adams-Bashforth
//
//       y(n+4) = y(n+3) + dt * ( 
//                                55/24*f(t(n+3),y(n+3)) 
//                               -59/24*f(t(n+2),y(n+2)) 
//                                37/24*f(t(n+1),y(n+1)) 
//                               -3/8*f(tn,yn) 
//                              )
//
//   /t (input) : current time
//   /dt (input) : time step.
//   /u (input/output) : y(n+3) on input, y(n+4) at output
//   /u0,u1,u2  : work spaces
//   /u3 (input/output) : work space
//=================================================================================
{
  using namespace Adams_Bashforth; // load constants

  CompositeGrid & cg = *u.getCompositeGrid();
  
  dudt( u,u3,f,t );  // ... u3 <- f(t(n+3),y(n+3))
  
  int grid;
  for( grid=0; grid<cg.numberOfComponentGrids(); grid++)
  {
    #ifdef USE_PPP
      realSerialArray ug; getLocalArrayWithGhostBoundaries(u[grid],ug);
      realSerialArray u0g; getLocalArrayWithGhostBoundaries(u0[grid],u0g);
      realSerialArray u1g; getLocalArrayWithGhostBoundaries(u1[grid],u1g);
      realSerialArray u2g; getLocalArrayWithGhostBoundaries(u2[grid],u2g);
      realSerialArray u3g; getLocalArrayWithGhostBoundaries(u3[grid],u3g);
    #else
      realSerialArray & ug = u[grid];
      realSerialArray & u0g = u0[grid];
      realSerialArray & u1g = u1[grid];
      realSerialArray & u2g = u2[grid];
      realSerialArray & u3g = u3[grid];
    #endif
      ug+=dt*(b3*u3g+b2*u2g+b1*u1g+b0*u0g);   // ...u <- y(n+4)
  }
  
  applyBoundaryConditionsNS( u,t+dt );

  t+=dt;

  return 0;
}

int 
rungeKutta4(real & t, 
	    real dt,
	    realCompositeGridFunction & f,
	    realCompositeGridFunction & u1, 
	    realCompositeGridFunction & u2, 
	    realCompositeGridFunction & u3, 
	    realCompositeGridFunction & u4)
// ================================================================================
// /Description:
//  Advance some time steps - Fourth Order Runge-Kutta
//
//       y(n+1) = yn + 1/6( k1 + 2*k2 + 2*k3 + k4 )
//           k1 = dt*f(t,yn)
//           k2 = dt*f(t+.5*h,yn+.5*k1)
//           k3 = dt*f(t+.5*h,yn+.5*k2)
//           k4 = dt*f(t+h,yn+k3)
//
//   /t (input) : current time
//   /dt (input) : time step.
//   /u1 (input/output) : solution at time t on input, solution at time t+dt at output
//   /u2,u3,u4  : work spaces
//=================================================================================
{
  CompositeGrid & cg = *u1.getCompositeGrid();

  doubleCompositeGridFunction Ftnyn;

  dudt( u1,u2,f,t );  // ... u2 <- k1=du1/dt(t)

  Ftnyn=u2; // store (for Adams-Bashforth)

  real dtb2=dt*.5;
  real dtb3=dt/3.;
  real dtb6=dt/6.;
  
  int grid;
  for( grid=0; grid<cg.numberOfComponentGrids(); grid++)
  {
    #ifdef USE_PPP
      realSerialArray u1g; getLocalArrayWithGhostBoundaries(u1[grid],u1g);
      realSerialArray u2g; getLocalArrayWithGhostBoundaries(u2[grid],u2g);
      realSerialArray u3g; getLocalArrayWithGhostBoundaries(u3[grid],u3g);
      realSerialArray u4g; getLocalArrayWithGhostBoundaries(u4[grid],u4g);
    #else
      realSerialArray & u1g = u1[grid];
      realSerialArray & u2g = u2[grid];
      realSerialArray & u3g = u3[grid];
      realSerialArray & u4g = u4[grid];
    #endif
    u3g=u1g+dtb2*u2g;   // ...u3 <- yn+.5*k1
    u4g=u1g+dtb6*u2g;   // ...u4 <- yn+1/6( k1 )   keep a running sum of the result (saves space)
  }
  
  applyBoundaryConditionsNS( u3,t+dtb2 );
  dudt( u3,u2,f,t+dtb2 );  //  ...u2 <- k2 = f(u3)
  
  for( grid=0; grid<cg.numberOfComponentGrids(); grid++)
  {
    #ifdef USE_PPP
      realSerialArray u1g; getLocalArrayWithGhostBoundaries(u1[grid],u1g);
      realSerialArray u2g; getLocalArrayWithGhostBoundaries(u2[grid],u2g);
      realSerialArray u3g; getLocalArrayWithGhostBoundaries(u3[grid],u3g);
      realSerialArray u4g; getLocalArrayWithGhostBoundaries(u4[grid],u4g);
    #else
      realSerialArray & u1g = u1[grid];
      realSerialArray & u2g = u2[grid];
      realSerialArray & u3g = u3[grid];
      realSerialArray & u4g = u4[grid];
    #endif
    u3g=u1g+dtb2*u2g;   // ...yn+.5*k2
    u4g+=dtb3*u2g;     // ...yn+1/6( k1 +2*k2 )
  }
  
  applyBoundaryConditionsNS( u3,t+dtb2 );
  dudt( u3,u2,f,t+dtb2 ); // ...u2 <- k3 = f(u3)

  for( grid=0; grid<cg.numberOfComponentGrids(); grid++)
  {
    #ifdef USE_PPP
      realSerialArray u1g; getLocalArrayWithGhostBoundaries(u1[grid],u1g);
      realSerialArray u2g; getLocalArrayWithGhostBoundaries(u2[grid],u2g);
      realSerialArray u3g; getLocalArrayWithGhostBoundaries(u3[grid],u3g);
      realSerialArray u4g; getLocalArrayWithGhostBoundaries(u4[grid],u4g);
    #else
      realSerialArray & u1g = u1[grid];
      realSerialArray & u2g = u2[grid];
      realSerialArray & u3g = u3[grid];
      realSerialArray & u4g = u4[grid];
    #endif
    u3g=u1g+dt*u2g;    // ...yn+k3
    u4g+=dtb3*u2g;    // ...yn+1/6( k1 +2*k2 +2*k3 )
  }
 
  applyBoundaryConditionsNS( u3,t+dt );
  dudt( u3,u2,f,t+dt ); //  ...u2 <- k4 = f(u3)

 
  for( grid=0; grid<cg.numberOfComponentGrids(); grid++)
  {
    #ifdef USE_PPP
      realSerialArray u1g; getLocalArrayWithGhostBoundaries(u1[grid],u1g);
      realSerialArray u2g; getLocalArrayWithGhostBoundaries(u2[grid],u2g);
      realSerialArray u4g; getLocalArrayWithGhostBoundaries(u4[grid],u4g);
    #else
      realSerialArray & u1g = u1[grid];
      realSerialArray & u2g = u2[grid];
      realSerialArray & u4g = u4[grid];
    #endif
    u1g=u4g+dtb6*u2g;
  }
  
  applyBoundaryConditionsNS( u1,t+dt );

  t+=dt;

  u2=Ftnyn; // retrieve (for Adams-Bashforth)

  return 0;
}

int 
euler(real & t, 
      real dt,
      realCompositeGridFunction & f,
      realCompositeGridFunction & u1, 
      realCompositeGridFunction & u2)
// ================================================================================
// /Description:
//  Advance some time steps - First Order Euler
//
//       y(n+1) = yn + k1
//           k1 = dt*f(t,yn)
//
//   /t (input) : current time
//   /dt (input) : time step.
//   /u1 (input/output) : solution at time t on input, solution at time t+dt at output
//   /u2   : work space
//=================================================================================
{
  CompositeGrid & cg = *u1.getCompositeGrid();
  
  dudt( u1,u2,f,t );  // ... u2 <- k1=du1/dt(t)
  
  int grid;
  for( grid=0; grid<cg.numberOfComponentGrids(); grid++)
  {
    #ifdef USE_PPP
      realSerialArray u1g; getLocalArrayWithGhostBoundaries(u1[grid],u1g);
      realSerialArray u2g; getLocalArrayWithGhostBoundaries(u2[grid],u2g);
    #else
      realSerialArray & u1g = u1[grid];
      realSerialArray & u2g = u2[grid];
    #endif
    u1g+=dt*u2g;   // ...u1 <- yn+k1
  }
  
  applyBoundaryConditionsNS( u1,t+dt );

  t+=dt;

  return 0;
}

//******************* FALSE-TRANSIENTS: Poisson-Boltzmann  *******************
//*** (only dudt applyBCs different, the rest of them just exclude 'f') ***
int 
applyBoundaryConditionsPB( realCompositeGridFunction & u, real t)
// =================================================================================================
// /Description:
//      Apply boundary conditions for an overlapping grids with refinements.
// =================================================================================================
{
  using namespace Main_misc; // use misc vars
  using namespace Data_ini;
  
  CompositeGrid & cg = *u.getCompositeGrid();

  ForGrids(grid)
    u[grid].updateGhostBoundaries();  // update parallel ghost boundaries
  
  //checkArrayIDs("applyBC: before interpolate"); 
  real time0=getCPU();
  u.interpolate();        // this includes AMR interpolation.
  timeForSolveInterpolate+=getCPU()-time0;
  //checkArrayIDs("applyBC: after interpolate"); 

  // apply true boundary conditions.
  time0=getCPU();

	      ForGrids(grid) // loop over the component grids
	      {
		MappedGrid & mg = cg[grid]; // make a reference (alias) to save typing
		getIndex(mg.indexRange(),I1,I2,I3);

		// *** apply Boundary Conditions ***
		ForBoundary(side,axis)
		  {
		    if( mg.boundaryCondition()(side,axis)==inflow2 )
		      {
			u.applyBoundaryCondition(c0,neumann,BCTypes::boundary,0.,t);
			u.applyBoundaryCondition(c1,dirichlet,BCTypes::boundary,a_inflow2,t);
			u.applyBoundaryCondition(c1,extrapolate,BCTypes::boundary,0.,t);
		      }
		    else if( mg.boundaryCondition()(side,axis)==inflow1 ) 
		      {
			u.applyBoundaryCondition(c0,neumann,BCTypes::boundary,0.,t);
			u.applyBoundaryCondition(c1,dirichlet,BCTypes::boundary,a_inflow1,t);
			u.applyBoundaryCondition(c1,extrapolate,BCTypes::boundary,0.,t);
		      }
		    else if( mg.boundaryCondition()(side,axis)==outflow1 )
		      {
			u.applyBoundaryCondition(c0,neumann,BCTypes::boundary,0.,t);
			u.applyBoundaryCondition(c1,dirichlet,BCTypes::boundary,a_outflow1,t);
			u.applyBoundaryCondition(c1,extrapolate,BCTypes::boundary,0.,t);
		      }
		    else if( mg.boundaryCondition()(side,axis)==outflow2 )
		      {
			u.applyBoundaryCondition(c0,neumann,BCTypes::boundary,0.,t);
			u.applyBoundaryCondition(c1,dirichlet,BCTypes::boundary,a_outflow2,t);
			u.applyBoundaryCondition(c1,extrapolate,BCTypes::boundary,0.,t);
		      }
		    else if( mg.boundaryCondition()(side,axis)==noSlipWall || mg.boundaryCondition()(side,axis)==noSlipWall2 )
		      {
			u.applyBoundaryCondition(c0,dirichlet,BCTypes::boundary,i_wall,t);
			u.applyBoundaryCondition(c0,extrapolate,BCTypes::boundary,0.,t);
			u.applyBoundaryCondition(c1,neumann,BCTypes::boundary,0.,t);
		      }
		    else if( mg.boundaryCondition()(side,axis)==noSlipWall3)
		      {
			u.applyBoundaryCondition(c0,dirichlet,BCTypes::boundary,i_wall2,t); // insulated
			u.applyBoundaryCondition(c0,extrapolate,BCTypes::boundary,0.,t);
			u.applyBoundaryCondition(c1,neumann,BCTypes::boundary,0.,t);
		      }	
		  }
	      } // end ForGrids

	      // fix up corners, periodic update:
	      u.finishBoundaryConditions();

	      timeForBoundaryConditions+=getCPU()-time0;
	      return 0;
}

int 
dudt( realCompositeGridFunction & u, realCompositeGridFunction & ut, real t )
{
  using namespace Main_misc; // use misc vars
  using namespace Data_ini;

  real time0=getCPU();
  
  CompositeGrid & cg = *u.getCompositeGrid();
  CompositeGridOperators & op = *u.getOperators();
  Index I1,I2,I3;

  // ut_i = u.laplacian() - ksqr * u;
  // ut_a = u.laplacian();
  double ksqr=L*L/debye/debye;
  
    int grid;
    ForGrids(grid)
    {
      MappedGrid & mg = cg[grid];
      getIndex(mg.gridIndexRange(),I1,I2,I3);

      // set up required arrays for serial & parallel version
      #ifdef USE_PPP
	realSerialArray utLocal; getLocalArrayWithGhostBoundaries(ut[grid],utLocal);
	realSerialArray uLocal;  getLocalArrayWithGhostBoundaries(u[grid],uLocal);

	bool ok=ParallelUtility::getLocalArrayBounds(u[grid],uLocal,I1,I2,I3);
	if( !ok ) continue;
      #else
        realSerialArray & utLocal = ut[grid];
        realSerialArray & uLocal = u[grid]; 
      #endif

	realArray ulap(uLocal);

	// evaluate required derivatives EFFICIENTLY
	op[grid].derivative(MappedGridOperators::laplacianOperator,uLocal,ulap,I1,I2,I3);

	// compute dudt; u=[phi_i phi_a]
	utLocal(I1,I2,I3,ic) = ulap(I1,I2,I3,ic) - ksqr* uLocal(I1,I2,I3,ic);
	utLocal(I1,I2,I3,ac) = ulap(I1,I2,I3,ac);
    }

    timeForDudt+=getCPU()-time0;
    return 0;
}

int 
linearMultiStep(real & t, 
		real dt,
		realCompositeGridFunction & u,  
		realCompositeGridFunction & u0, 
		realCompositeGridFunction & u1, 
		realCompositeGridFunction & u2)
// ================================================================================
// /Description:
//  Driver for a Linear Multistep Method
//=================================================================================
{
  using namespace Main_misc; 

  if (ft_step<3) // use Runge-Kutta to get first 3 iterations...
    {
      // Take a time step (Runge-Kutta, Euler)
      rungeKutta4(t,dt,u,u0,u1,u2);
      Deque.push_back(u0); // push u0=f(tn,yn) => fill in the queue iteratively
    }
  else // ...and continue with 4-step Adams-Bashforth
    { 
      adamsBashforth(t,dt,u,Deque[0],Deque[1],Deque[2],u2); // -> u, u2
      Deque.pop_front();         // remove from front
      Deque.push_back(u2);       // insert at back
    }
  return 0;
}

int 
adamsBashforth(real & t, 
	       real dt,
	       realCompositeGridFunction & u,  // y(n+3) -> y(n+4)
	       realCompositeGridFunction & u0, // f(tn,yn) 
	       realCompositeGridFunction & u1, // f(t(n+1),y(n+1))
	       realCompositeGridFunction & u2, // f(t(n+2),y(n+2))
	       realCompositeGridFunction & u3) // -> f(t(n+3),y(n+3))
// ================================================================================
// /Description:
//  4-step Adams-Bashforth
//
//       y(n+4) = y(n+3) + dt * ( 
//                                55/24*f(t(n+3),y(n+3)) 
//                               -59/24*f(t(n+2),y(n+2)) 
//                                37/24*f(t(n+1),y(n+1)) 
//                               -3/8*f(tn,yn) 
//                              )
//
//   /t (input) : current time
//   /dt (input) : time step.
//   /u (input/output) : y(n+3) on input, y(n+4) at output
//   /u0,u1,u2  : work spaces
//   /u3 (input/output) : work space
//=================================================================================
{
  using namespace Adams_Bashforth; // load constants

  CompositeGrid & cg = *u.getCompositeGrid();
  
  dudt( u,u3,t );  // ... u3 <- f(t(n+3),y(n+3))
  
  int grid;
  for( grid=0; grid<cg.numberOfComponentGrids(); grid++)
  {
    #ifdef USE_PPP
      realSerialArray ug; getLocalArrayWithGhostBoundaries(u[grid],ug);
      realSerialArray u0g; getLocalArrayWithGhostBoundaries(u0[grid],u0g);
      realSerialArray u1g; getLocalArrayWithGhostBoundaries(u1[grid],u1g);
      realSerialArray u2g; getLocalArrayWithGhostBoundaries(u2[grid],u2g);
      realSerialArray u3g; getLocalArrayWithGhostBoundaries(u3[grid],u3g);
    #else
      realSerialArray & ug = u[grid];
      realSerialArray & u0g = u0[grid];
      realSerialArray & u1g = u1[grid];
      realSerialArray & u2g = u2[grid];
      realSerialArray & u3g = u3[grid];
    #endif
      ug+=dt*(b3*u3g+b2*u2g+b1*u1g+b0*u0g);   // ...u <- y(n+4)
  }
  
  applyBoundaryConditionsPB( u,t+dt );

  t+=dt;

  return 0;
}

int 
rungeKutta4(real & t, 
	    real dt,
	    realCompositeGridFunction & u1, 
	    realCompositeGridFunction & u2, 
	    realCompositeGridFunction & u3, 
	    realCompositeGridFunction & u4)
// ================================================================================
// /Description:
//  Advance some time steps - Fourth Order Runge-Kutta
//
//       y(n+1) = yn + 1/6( k1 + 2*k2 + 2*k3 + k4 )
//           k1 = dt*f(t,yn)
//           k2 = dt*f(t+.5*h,yn+.5*k1)
//           k3 = dt*f(t+.5*h,yn+.5*k2)
//           k4 = dt*f(t+h,yn+k3)
//
//   /t (input) : current time
//   /dt (input) : time step.
//   /u1 (input/output) : solution at time t on input, solution at time t+dt at output
//   /u2,u3,u4  : work spaces
//=================================================================================
{
  CompositeGrid & cg = *u1.getCompositeGrid();

  doubleCompositeGridFunction Ftnyn;

  dudt( u1,u2,t );  // ... u2 <- k1=du1/dt(t)

  Ftnyn=u2; // store (for Adams-Bashforth)

  real dtb2=dt*.5;
  real dtb3=dt/3.;
  real dtb6=dt/6.;
  
  int grid;
  for( grid=0; grid<cg.numberOfComponentGrids(); grid++)
  {
    #ifdef USE_PPP
      realSerialArray u1g; getLocalArrayWithGhostBoundaries(u1[grid],u1g);
      realSerialArray u2g; getLocalArrayWithGhostBoundaries(u2[grid],u2g);
      realSerialArray u3g; getLocalArrayWithGhostBoundaries(u3[grid],u3g);
      realSerialArray u4g; getLocalArrayWithGhostBoundaries(u4[grid],u4g);
    #else
      realSerialArray & u1g = u1[grid];
      realSerialArray & u2g = u2[grid];
      realSerialArray & u3g = u3[grid];
      realSerialArray & u4g = u4[grid];
    #endif
    u3g=u1g+dtb2*u2g;   // ...u3 <- yn+.5*k1
    u4g=u1g+dtb6*u2g;   // ...u4 <- yn+1/6( k1 )   keep a running sum of the result (saves space)
  }
  
  applyBoundaryConditionsPB( u3,t+dtb2 );
  dudt( u3,u2,t+dtb2 );  //  ...u2 <- k2 = f(u3)
  
  for( grid=0; grid<cg.numberOfComponentGrids(); grid++)
  {
    #ifdef USE_PPP
      realSerialArray u1g; getLocalArrayWithGhostBoundaries(u1[grid],u1g);
      realSerialArray u2g; getLocalArrayWithGhostBoundaries(u2[grid],u2g);
      realSerialArray u3g; getLocalArrayWithGhostBoundaries(u3[grid],u3g);
      realSerialArray u4g; getLocalArrayWithGhostBoundaries(u4[grid],u4g);
    #else
      realSerialArray & u1g = u1[grid];
      realSerialArray & u2g = u2[grid];
      realSerialArray & u3g = u3[grid];
      realSerialArray & u4g = u4[grid];
    #endif
    u3g=u1g+dtb2*u2g;   // ...yn+.5*k2
    u4g+=dtb3*u2g;     // ...yn+1/6( k1 +2*k2 )
  }
  
  applyBoundaryConditionsPB( u3,t+dtb2 );
  dudt( u3,u2,t+dtb2 ); // ...u2 <- k3 = f(u3)

  for( grid=0; grid<cg.numberOfComponentGrids(); grid++)
  {
    #ifdef USE_PPP
      realSerialArray u1g; getLocalArrayWithGhostBoundaries(u1[grid],u1g);
      realSerialArray u2g; getLocalArrayWithGhostBoundaries(u2[grid],u2g);
      realSerialArray u3g; getLocalArrayWithGhostBoundaries(u3[grid],u3g);
      realSerialArray u4g; getLocalArrayWithGhostBoundaries(u4[grid],u4g);
    #else
      realSerialArray & u1g = u1[grid];
      realSerialArray & u2g = u2[grid];
      realSerialArray & u3g = u3[grid];
      realSerialArray & u4g = u4[grid];
    #endif
    u3g=u1g+dt*u2g;    // ...yn+k3
    u4g+=dtb3*u2g;    // ...yn+1/6( k1 +2*k2 +2*k3 )
  }
 
  applyBoundaryConditionsPB( u3,t+dt );
  dudt( u3,u2,t+dt ); //  ...u2 <- k4 = f(u3)

 
  for( grid=0; grid<cg.numberOfComponentGrids(); grid++)
  {
    #ifdef USE_PPP
      realSerialArray u1g; getLocalArrayWithGhostBoundaries(u1[grid],u1g);
      realSerialArray u2g; getLocalArrayWithGhostBoundaries(u2[grid],u2g);
      realSerialArray u4g; getLocalArrayWithGhostBoundaries(u4[grid],u4g);
    #else
      realSerialArray & u1g = u1[grid];
      realSerialArray & u2g = u2[grid];
      realSerialArray & u4g = u4[grid];
    #endif
    u1g=u4g+dtb6*u2g;
  }
  
  applyBoundaryConditionsPB( u1,t+dt );

  t+=dt;

  u2=Ftnyn; // retrieve (for Adams-Bashforth)

  return 0;
}

int 
euler(real & t, 
      real dt,
      realCompositeGridFunction & u1, 
      realCompositeGridFunction & u2)
// ================================================================================
// /Description:
//  Advance some time steps - First Order Euler
//
//       y(n+1) = yn + k1
//           k1 = dt*f(t,yn)
//
//   /t (input) : current time
//   /dt (input) : time step.
//   /u1 (input/output) : solution at time t on input, solution at time t+dt at output
//   /u2   : work space
//=================================================================================
{
  CompositeGrid & cg = *u1.getCompositeGrid();
  
  dudt( u1,u2,t );  // ... u2 <- k1=du1/dt(t)
  
  int grid;
  for( grid=0; grid<cg.numberOfComponentGrids(); grid++)
  {
    #ifdef USE_PPP
      realSerialArray u1g; getLocalArrayWithGhostBoundaries(u1[grid],u1g);
      realSerialArray u2g; getLocalArrayWithGhostBoundaries(u2[grid],u2g);
    #else
      realSerialArray & u1g = u1[grid];
      realSerialArray & u2g = u2[grid];
    #endif
    u1g+=dt*u2g;   // ...u1 <- yn+k1
  }
  
  applyBoundaryConditionsPB( u1,t+dt );

  t+=dt;

  return 0;
}

int
NaN(realCompositeGridFunction & u)
// ================================================================================
// /Description:
//  Test for NaNs
//
// Note: NaNs are not equal to anything - not even themselves.
//=================================================================================
{
 
  using namespace Main_misc;
  CompositeGrid & cg = *u.getCompositeGrid();
  
  ForGrids(grid) // loop over the component grids
  {
    MappedGrid & mg = cg[grid];
    getIndex(mg.indexRange(),I1,I2,I3);
    for(int ii=I1.getBase(); ii<=I1.getBound();ii++) 
      for(int jj=I2.getBase(); jj<=I2.getBound();jj++) 
	for(int kk=I3.getBase(); kk<=I3.getBound();kk++) 
	  ForDims(dim) 
	    if(u[grid](ii,jj,kk,dim)!=u[grid](ii,jj,kk,dim))
	      {
		cerr << "\nNaN encountered!";
		return 1;
	      }
  }

  return 0;
}

void forceEval(const doubleArray & p, doubleArray & f, fe & fed)
//
// === Evaluate MD (inter-particle) forces ===
// Arguments:
// p = position array
// f = MD fores
// fed = forceEval data structure
{
  using namespace Data_ini;
  using namespace Main_misc;

  double rab;
  doubleArray rab_v(dim);
  f=0.; // initialize

  // Bonded forces (yes/no)
  if(Molecular_dynamics==1)  // Marko-Siggia
    {
      // 1) Marko-Siggia spring force (supply: f_MS)
      double q;
      for (int n=0; n<NoOfParticles-1; n++) // beads (0..N-1)
	{
	  ForDims(dim)
	    rab_v(dim)=L*(p(n+1,dim)-p(n,dim)); // r_ab (dimensional)
      
	  rab=sqrt(sum(rab_v*rab_v)); // |r_ab|
	  q=rab/q0;
	  if (q>1.0)
	    printf("\nSpring overextension! (step=%i, bead=%d, bond extension=%f%%", step, n, q*100);

	  ForDims(dim)
	    f(n,dim)=fed.f_MS*(pow(1-q,-2)-1+4*q)*rab_v(dim)/rab; // Marko-Siggia spring law (multiply by unit bond vector)
	    //f(n,dim)=fed.f_H*q*rab_v(dim)/rab; // Hookean spring
	}
      for (int n=NoOfParticles-1; n>0; n--) // correct beads (1..N), need reverse loop
	ForDims(dim)
	  f(n,dim)-=f(n-1,dim);
    } // end if(Molecular_dynamics)

  if(Molecular_dynamics==2) // Harmonic bond & angle potentials
    {
      // 2) Harmonic bond potential (supply: k_B, r0)
      double q;
      for (int n=0; n<NoOfParticles-1; n++) // beads (0..N-1)
	{
	  ForDims(dim)
	    rab_v(dim)=L*(p(n+1,dim)-p(n,dim)); // r_ab (dimensional)
      
	  rab=sqrt(sum(rab_v*rab_v)); // |r_ab|
	  q=rab/q0;
	  if (q>1.0)
	    printf("\nSpring overextension! (step=%i, bead=%d, bond extension=%f%%", step, n, q*100);

	  ForDims(dim)
	    f(n,dim)=2*fed.k_B*(rab-fed.r0)*rab_v(dim)/rab; // bond force
	}
      for (int n=NoOfParticles-1; n>0; n--) // correct beads (1..N), need reverse loop
	ForDims(dim)
	  f(n,dim)-=f(n-1,dim);


      // 3) Harmonic angle potential (supply: k_A, theta0)
      double rij, rik;
      doubleArray rij_v(dim), rik_v(dim);
      for (int n=1; n<NoOfParticles-1; n++) // beads (1..N-1)
	{
	  ForDims(dim)
	  {
	    rij_v(dim)=L*(p(n-1,dim)-p(n,dim));
	    rik_v(dim)=L*(p(n+1,dim)-p(n,dim));
	  }

      	  rij=sqrt(sum(rij_v*rij_v)); // |r_ij|
      	  rik=sqrt(sum(rik_v*rik_v)); // |r_ik|

	  double 
	    theta=acos( sqrt(sum(rij_v*rik_v)) / (rij*rik) ),
	    pref=1/sin(theta)*2*fed.k_A*(theta-fed.theta0),
	    cost=cos(theta),
	    rijrik=rij*rik, 
	    rij2=rij*rij,
	    rik2=rik*rik;

	  ForDims(dim) // angle force
	  {
	    f(n-1,dim)=pref*( (1)*rik_v(dim)/rijrik - cost*( (1)*rij_v(dim)/rij2 ) )  ; // j
	    f(n  ,dim)=pref*( (-1)*rik_v(dim)/rijrik + (-1)*rij_v(dim)/rijrik - cost*( (-1)*rij_v(dim)/rij2 + (-1)*rik_v(dim)/rik2 ) )  ; // i
	    f(n+1,dim)=pref*( (1)*rij_v(dim)/rijrik - cost*( (1)*rik_v(dim)/rik2 ) )  ; // k
	  }
	}

    } // end if(Molecular_dynamics)

  // 2) Excluded-volume force (VDW)
  if(Vdw==1) // Jendrejack
    {
      doubleArray ftmp(Dim);

      for(int i=0; i<NoOfParticles; i++)
	{
	  for(int j=i+2; j<NoOfParticles; j++) //j=i+1: include bonded pairs, j=i+2: exclude bonded pairs
	    {
	      ForDims(dim)
		rab_v(dim)=L*(p(j,dim)-p(i,dim)); // r_ab (dimensional)
      
	      rab=sqrt(sum(rab_v*rab_v)); // |r_ab|
	      ForDims(dim)
	      {
		ftmp(dim)=exp(fed.exp_EV*rab*rab)*rab_v(dim); // Jendrejack's e-v force
		f(i,dim)-=fed.f_EV1*ftmp(dim);
		f(j,dim)+=fed.f_EV1*ftmp(dim);
	      }
	    }
	}

    }

  if(Vdw==2) // Lennard-Jones
    {
      doubleArray ftmp(Dim);

      for(int i=0; i<NoOfParticles; i++)
	{
	  for(int j=i+1; j<NoOfParticles; j++) //j=i+1: include bonded pairs, j=i+2: exclude bonded pairs
	    {
	      ForDims(dim)
		rab_v(dim)=L*(p(j,dim)-p(i,dim)); // r_ab (dimensional)
      
	      rab=sqrt(sum(rab_v*rab_v)); // |r_ab|
	      ForDims(dim)
	      {
		//ftmp(dim)=(2*pow(fed.vdw_s/rab,13)-pow(fed.vdw_s/rab,7))*rab_v(dim)/rab; // L-J force (full)
		ftmp(dim)=(2*pow(fed.vdw_s/rab,13))*rab_v(dim)/rab; // L-J force (repulsive part only)
		f(i,dim)-=fed.f_EV2*ftmp(dim);
		f(j,dim)+=fed.f_EV2*ftmp(dim);
	      }
	    }
	}

    }

  if(Electrostatics) // electrostatic forces (yes/no)
    {
      // 2) Electrostatic (Debye-Huckel) force for dielectric medium
      doubleArray ftmp(Dim); 

      for(int i=0; i<NoOfParticles; i++)
	{
	  for(int j=i+1; j<NoOfParticles; j++) //j=i+1: include bonded pairs, j=i+2: exclude bonded pairs
	    {
	      ForDims(dim)
		rab_v(dim)=L*(p(j,dim)-p(i,dim)); // r_ab (dimensional)
      
	      rab=sqrt(sum(rab_v*rab_v)); // |r_ab|
	      ForDims(dim)
	      {
		//ftmp(dim)=(1+rab/debye)*exp(-rab/debye)/(rab*rab*rab) *rab_v(dim); // screened force
		ftmp(dim)=1./(rab*rab*rab) *rab_v(dim); // unscreened force
		f(i,dim)+=fed.f_ES*ftmp(dim);
		f(j,dim)-=fed.f_ES*ftmp(dim);
	      }
	    }
	}

    }

  // convert force units [N] -> [Da*A*ps^-2] to mimic output from dlpoly
  f/=(0.001/AVOG*1e-10*1e24); // f_0/Fmd

}

void rndEval(doubleArray & Random, const doubleArray & Sigma, const double Sigma_reduced)
// ================================================================================
// /Description:
//    Evaluate random forces
//
// ================================================================================
{
  using namespace Data_ini;
  using namespace Main_misc;

  // Update RND forces
  // 1) Generate rnd numbers (use Box-Muller transform: uniform->Gaussian(0,1))
  ForParticlesAndDims(particle,dim)
    {
      Random(particle,dim)=ranf(); //box_muller(0,1); // ranf() or box_muller(0,1) (Gaussian requires smaller time steps!)
      //cout << endl << Random(particle,dim); // print them out so we can check if the distribution is correct
    }

  // 2) Calculate Brownian velocity contribution (sigma term in BD equation), overwrite Random array to save space
  if (Momentum_transfer && step % MTFreq != 0 && P_Oseen) // if not resolving Navier-Stokes (and P_Oseen is ON)
    ForParticlesReverse(particle) // use reverse loop over particles so that Random array can be overwritten
      ForDims(dim)
      {
	Random(particle,dim)=Sigma(particle,particle)*Random(particle,dim); // diagonal term ...
	for (particle2=0; particle2<particle; particle2++)
	  Random(particle,dim)+=Sigma(particle,particle2)*Random(particle2,dim); // ... plus off-diagonal terms
      }
  else
    Random*=Sigma_reduced;
}

bool FileExists(string strFilename) {
  // Check if file exists
  struct stat stFileInfo;
  bool blnReturn;
  int intStat;

  // Attempt to get the file attributes
  intStat = stat(strFilename.c_str(),&stFileInfo);
  if(intStat == 0) {
    // We were able to get the file attributes
    // so the file obviously exists.
    blnReturn = true;
  } else {
    // We were not able to get the file attributes.
    // This may mean that we don't have permission to
    // access the folder which contains this file. If you
    // need to do that level of checking, lookup the
    // return values of stat which will give you
    // more details on why stat failed.
    blnReturn = false;
  }
  
  return(blnReturn);
}

int getNumberOfPoints(CompositeGrid & gc)
// ================================================================================
// /Description:
//    Determine the total number of grid points in a CompositeGrid.
//
//  /gc (input) 
//  /totalNumberOfGridPoints (output): total number of grid points in gc
//  /maskOption (input) : indicates which points to include.
// ================================================================================
{
    int totalNumberOfGridPoints=0;
    for( int grid=0; grid<gc.numberOfComponentGrids(); grid++ )
    {
        MappedGrid & mg = gc[grid];
        const IntegerArray & dim = mg.indexRange();
        totalNumberOfGridPoints+=(dim(1,0)-dim(0,0)+1)*(dim(1,1)-dim(0,1)+1)*(dim(1,2)-dim(0,2)+1);
    }
    return(totalNumberOfGridPoints);
}
void DrawProgressBar(int len, double percent) {
  cout << "\x1B[2K"; // Erase the entire current line.
  cout << "\x1B[0E"; // Move to the beginning of the current line.
  string progress;
  for (int i = 0; i < len; ++i) {
    if (i < static_cast<int>(len * percent)) {
      progress += "=";
    } else {
      progress += " ";
    }
  }
  cout << "[" << progress << "] " << (static_cast<int>(100 * percent)) << "%";
  flush(cout); // Required.
}

void getDomainBounds(realArray & min, realArray & max)
// ================================================================================
// /Description:
//    Get bounds of the domain, check only background grid cg[0]
//
// ================================================================================
{
  using namespace Main_misc;
  using namespace Data_ini;

 //    for( int grid=0; grid<gc.numberOfComponentGrids(); grid++ )
  int grid=0;
  {
    MappedGrid & mg = cg[grid];
    realMappedGridFunction & vertex = mg.vertex();
    ForDims(dim)
    {
      min(dim) = vertex(mg.gridIndexRange(0,0),mg.gridIndexRange(0,1),mg.gridIndexRange(0,2),dim);
      max(dim) = vertex(mg.gridIndexRange(1,0),mg.gridIndexRange(1,1),mg.gridIndexRange(1,2),dim);
    }
    //max(0)-=0.5; // substract half the trap length => exit plane moved to the middle of the last trap
  }
  if(Debug & 1) 
    {
      min.display("\nMin :");
      max.display("\nMax :");
    }
}

void rescale_position(doubleArray & position, doubleArray & velocity,  doubleArray & position_ini, double Position_rescale, double Velocity_rescale)
{
  using namespace Main_misc;
  using namespace Data_ini;

 // Rescale (dlpoly -> overture format)
  position*=Position_rescale;  // rescale position, 
  velocity*=Velocity_rescale;     // rescale velocity

  position_ini=position; // make a copy

  if(Debug & 1) 
    {
      position.display("\n\n Initial position of particles: ");
      velocity.display("\n\n Initial velocity of particles: ");
    }
}

void run_Sarw(sarw_data & dat, int sim)
{
  using namespace Main_misc;
  using namespace Data_ini;

	int try_max = 100; // max number of random walk attempts
	for (int seed_try=1; seed_try <= try_max; seed_try++) // attempt different seeds
	  {
	    try 
	      {
		sarw(Seed+sim,dat,sim); // call C subroutine (sarw.C)
	      }
	    catch (const char *s)  // exception handler (random walk failed)
	      {
		Seed+=1000; // try a different seed
		printf("%s Trying seed = %li (attempt %i out of %i)...\n", s, Seed, seed_try, try_max);
		continue;
	      }
	    break;
	  }
}



void eval_Quadrupole(realCompositeGridFunction & E, realCompositeGridFunction & Q)
{
  using namespace Main_misc;
  using namespace Data_ini;

  int ncomp = (Dim*Dim + Dim)/2.; // # components of a symmetric matrix

  CompositeGridOperators op(cg); // create differential operators
  op.setOrderOfAccuracy(OrderOfAccuracy);
  E.setOperators(op);

  realCompositeGridFunction  
    J(cg,all,all,all,ncomp), 
    Hx(cg,all,all,all,ncomp),
    Hy(cg,all,all,all,ncomp),
    Hz(cg,all,all,all,ncomp);
  J=Hx=Hy=Hz=0.;

  if(Dim==3)
    realCompositeGridFunction  
      Hz(cg,all,all,all,ncomp);
  Range R3(3,3), R4(4,4), R5(5,5); // add some ranges

  cout << "\nCalculating Jacobian matrix...";
  // Jacobian matrix (symmetric, del x E = 0)
  J(R0)=E(R0).x();
  J(R1)=E(R1).y();
  J(R2)=E(R0).y();
  if(Dim==3)
    {
      J(R3)=E(R0).z();
      J(R4)=E(R1).z();
      J(R5)=E(R2).z();
    }

  cout << "\nCalculating Hessian matrix...";
  // Hesian matrices (symmetric, partials continous)
  // Hx for Ex
  Hx(R0)=E(R0).xx(); 
  Hx(R1)=E(R0).yy(); 
  Hx(R2)=E(R0).xy(); 
  if(Dim==3)
    {
      Hx(R3)=E(R0).xz(); 
      Hx(R4)=E(R0).xy(); 
      Hx(R5)=E(R0).zz(); 
    }

  // Hy for Ey
  Hy(R0)=E(R1).xx(); 
  Hy(R1)=E(R1).yy(); 
  Hy(R2)=E(R1).xy(); 
  if(Dim==3)
    {
      Hy(R3)=E(R1).xz(); 
      Hy(R4)=E(R1).xy(); 
      Hy(R5)=E(R1).zz(); 
    }

  // Hz for Ez
  if(Dim==3)
    {
      Hz(R0)=E(R2).xx(); 
      Hz(R1)=E(R2).yy(); 
      Hz(R2)=E(R2).xy(); 
      
      Hz(R3)=E(R2).xz(); 
      Hz(R4)=E(R2).xy(); 
      Hz(R5)=E(R2).zz(); 
    }

//   J.interpolate();
//   Hx.interpolate();
//   Hy.interpolate();
//   Hz.interpolate();

  cout << "\nCalculating quadrupole tensor components...";
  if(Dim==2) // 2-D
    {
      // Q, x-component
      Q(R0) = 
	J(R0)*Hx(R0) + J(R2)*Hx(R2) +
	J(R2)*Hx(R2) + J(R1)*Hx(R1);
      // Q, y-component
      Q(R1) = 
	J(R0)*Hy(R0) + J(R2)*Hy(R2) +
	J(R2)*Hy(R2) + J(R1)*Hy(R1);
     }
  else // 3-D
    {
      // Q, x-component
      Q(R0) = 
	J(R0)*Hx(R0) + J(R2)*Hx(R2) + J(R3)*Hx(R3) +
	J(R2)*Hx(R2) + J(R1)*Hx(R1) + J(R4)*Hx(R4) +
	J(R3)*Hx(R3) + J(R4)*Hx(R4) + J(R5)*Hx(R5);
      // Q, y-component
      Q(R1) = 
	J(R0)*Hy(R0) + J(R2)*Hy(R2) + J(R3)*Hy(R3) +
	J(R2)*Hy(R2) + J(R1)*Hy(R1) + J(R4)*Hy(R4) +
	J(R3)*Hy(R3) + J(R4)*Hy(R4) + J(R5)*Hy(R5);
      // Q, z-component
      Q(R2) = 
	J(R0)*Hz(R0) + J(R2)*Hz(R2) + J(R3)*Hz(R3) +
	J(R2)*Hz(R2) + J(R1)*Hz(R1) + J(R4)*Hz(R4) +
	J(R3)*Hz(R3) + J(R4)*Hz(R4) + J(R5)*Hz(R5);
    }

  Q.interpolate();
  cout << "done.\n";

  printf("\nE:  Min=%e, Max=%e", min(E), max(E));
  printf("\nJ:  Min=%e, Max=%e", min(J), max(J));
  printf("\nHx: Min=%e, Max=%e", min(Hx), max(Hx));
  printf("\nHy: Min=%e, Max=%e", min(Hy), max(Hy));
  printf("\nHz: Min=%e, Max=%e", min(Hz), max(Hz));
  printf("\nQ:  Min=%e, Max=%e", min(Q), max(Q));

}

void CM_NS_Build(realCompositeGridFunction & coeff2, int matrix, real dt)
{
  //=========================================================================================
  // Build coefficient matrix for equlibrium Navier-Stokes
  //
  // Format of the matrix:
  // 1 ... equilibrium
  // 2 ... backward Euler
  //=========================================================================================

  using namespace Main_misc;
  using namespace Data_ini;

  cout << " \nBuilding coefficient matrix for equilibrium Navier-Stokes...\n";

  int width=OrderOfAccuracy+1; // 4th order accuracy => 5-point stencil
  Dim = cg.numberOfDimensions(); // Determine dimension of the grid
  int numberOfGhostLines=OrderOfAccuracy/2;

  int numberOfComponents2 = Dim+1; // velocity components + pressure (v_x=u, v_y=v, v_z=w, P)
  int stencilSize2=int(pow(width,cg.numberOfDimensions())+1);
  int stencilDimension2=stencilSize2*SQR(numberOfComponents2);
  //realCompositeGridFunction coeff2(cg,stencilDimension2,all,all,all);
  coeff2.updateToMatchGrid(cg,stencilDimension2,all,all,all); // resize
 
  // make this grid function a coefficient matrix:
  coeff2.setIsACoefficientMatrix(TRUE,stencilSize2,numberOfGhostLines,numberOfComponents2);
  //coeff2.setIsACoefficientMatrix(TRUE,stencilSize2);//,numberOfGhostLines,numberOfComponents2);
  
  CompositeGridOperators op2(cg); // create some operators
  op2.setStencilSize(stencilSize2);
  op2.setNumberOfComponentsForCoefficients(numberOfComponents2);
  op2.setOrderOfAccuracy(OrderOfAccuracy);
  coeff2.setOperators(op2);
  ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  // Form a system of equations for (P,u,v,w), Pressure-Poisson system, Henshaw's 4th order pressure correction method with damping//
  // (1) delta(p)-damping*div(U)-div(F)=0  => -delta(p)+damping*div(U)=-div(F)                                                       //
  // (2) grad(p)-delta(U)-F=0              => -delta(U)+grad(p)=F, F=ksqr*phi_i*grad(phi_a)                                       //
  //                                                                                                                               //
  // 1. -Delta(p) + damping (u.x + v.y (+ w_z)) = f2_0                                                                             //
  // 2. -Delta(u) + p.x = f2_1                                                                                                     //
  // 3. -Delta(v) + p.y = f2_2                                                                                                     //
  // (4. -Delta(w) + p.z = f2_3)                                                                                                   //
  ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  Overture::printMemoryUsage("Before filling coefficient matrix");

  coeff2 = 
    - op2.laplacianCoefficients(eq1,pc) + damping*(op2.xCoefficients(eq1,uc)+op2.yCoefficients(eq1,vc)) // eqn1
    - op2.laplacianCoefficients(eq2,uc) + op2.xCoefficients(eq2,pc)  // eqn 2
    - op2.laplacianCoefficients(eq3,vc) + op2.yCoefficients(eq3,pc); // eqn 3
  if (Dim==3) coeff2 += damping*op2.zCoefficients(eq1,wc) - op2.laplacianCoefficients(eq4,wc) + op2.zCoefficients(eq4,pc); // 3-D case

  if(matrix==2)  // modify CM for backward Euler
    {
      coeff2 = dt*coeff2
	+ op2.identityCoefficients(eq1,pc)
	+ op2.identityCoefficients(eq2,uc)
	+ op2.identityCoefficients(eq3,vc);
      if (Dim==3) coeff2 += op2.identityCoefficients(eq4,wc);
    }
  Overture::printMemoryUsage("After filling coefficient matrix");

  //=========================================================================================
  //    Apply BCs to the coefficient matrix for Navier-Stokes
  //=========================================================================================


  cout << " \nApplying BCs to the coefficient matrix for Navier-Stokes...\n";
  Dim = cg.numberOfDimensions(); // Determine dimension of the grid

  // ----------------------- Apply BCs -----------------------

  // Process all velocity components (eqn no. = component index = i)
  for(i=1; i<Dim+1; i++) 
    {
      coeff2.applyBoundaryConditionCoefficients(i,i,neumann,    inflow2);
      coeff2.applyBoundaryConditionCoefficients(i,i,neumann,    inflow1);
      coeff2.applyBoundaryConditionCoefficients(i,i,neumann,    outflow1);
      coeff2.applyBoundaryConditionCoefficients(i,i,neumann,    outflow2);
      coeff2.applyBoundaryConditionCoefficients(i,i,dirichlet,  noSlipWall);
      coeff2.applyBoundaryConditionCoefficients(i,i,extrapolate,noSlipWall);
      coeff2.applyBoundaryConditionCoefficients(i,i,dirichlet,  noSlipWall2);
      coeff2.applyBoundaryConditionCoefficients(i,i,extrapolate,noSlipWall2);
      coeff2.applyBoundaryConditionCoefficients(i,i,dirichlet,  noSlipWall3);
      coeff2.applyBoundaryConditionCoefficients(i,i,extrapolate,noSlipWall3);
    }
  // Process pressure
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,dirichlet,  inflow2);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,extrapolate,inflow2);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,dirichlet,  inflow1);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,extrapolate,inflow1);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,dirichlet,  outflow1);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,dirichlet,  outflow2);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,extrapolate,outflow1);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,extrapolate,outflow2);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,neumann,    noSlipWall);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,neumann,    noSlipWall2);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,neumann,    noSlipWall3);
   
  if( OrderOfAccuracy==4 )
    {
      BoundaryConditionParameters bcParams;
      bcParams.ghostLineToAssign=2; // assign BC also to the 2nd ghost line
      for(i=0; i<Dim+1; i++) 
	{
	  coeff2.applyBoundaryConditionCoefficients(i,i,extrapolate,allBoundaries,bcParams); // extrap 2nd ghost line
	}
    }

  coeff2.finishBoundaryConditions();

  // --------------------- BCs applied -----------------------
}

void CM_NS_ApplyBCs(realCompositeGridFunction & coeff2)
{
  //=========================================================================================
  //    Apply BCs to the coefficient matrix for Navier-Stokes
  //=========================================================================================

  using namespace Main_misc;
  using namespace Data_ini;

  cout << " \nApplying BCs to the coefficient matrix for Navier-Stokes...\n";
  Dim = cg.numberOfDimensions(); // Determine dimension of the grid

  // ----------------------- Apply BCs -----------------------

  // Process all velocity components (eqn no. = component index = i)
  for(i=1; i<Dim+1; i++) 
    {
      coeff2.applyBoundaryConditionCoefficients(i,i,neumann,    inflow2);
      coeff2.applyBoundaryConditionCoefficients(i,i,neumann,    inflow1);
      coeff2.applyBoundaryConditionCoefficients(i,i,neumann,    outflow1);
      coeff2.applyBoundaryConditionCoefficients(i,i,neumann,    outflow2);
      coeff2.applyBoundaryConditionCoefficients(i,i,dirichlet,  noSlipWall);
      coeff2.applyBoundaryConditionCoefficients(i,i,extrapolate,noSlipWall);
      coeff2.applyBoundaryConditionCoefficients(i,i,dirichlet,  noSlipWall2);
      coeff2.applyBoundaryConditionCoefficients(i,i,extrapolate,noSlipWall2);
      coeff2.applyBoundaryConditionCoefficients(i,i,dirichlet,  noSlipWall3);
      coeff2.applyBoundaryConditionCoefficients(i,i,extrapolate,noSlipWall3);
    }
  // Process pressure
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,dirichlet,  inflow2);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,extrapolate,inflow2);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,dirichlet,  inflow1);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,extrapolate,inflow1);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,dirichlet,  outflow1);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,dirichlet,  outflow2);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,extrapolate,outflow1);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,extrapolate,outflow2);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,neumann,    noSlipWall);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,neumann,    noSlipWall2);
  coeff2.applyBoundaryConditionCoefficients(eq1,pc,neumann,    noSlipWall3);
   
  if( OrderOfAccuracy==4 )
    {
      BoundaryConditionParameters bcParams;
      bcParams.ghostLineToAssign=2; // assign BC also to the 2nd ghost line
      for(i=0; i<Dim+1; i++) 
	{
	  coeff2.applyBoundaryConditionCoefficients(i,i,extrapolate,allBoundaries,bcParams); // extrap 2nd ghost line
	}
    }

  coeff2.finishBoundaryConditions();

  // --------------------- BCs applied -----------------------
}

void checkIfPenetration(const int & step, const doubleArray & position, realArray & domain_min, realArray & domain_max)
// ================================================================================
// /Description:
//    Inform of any wall penetration
// Note: Checks only penetration of the outside rectangular box. Assumes no walls
// in x-direction
//
// ================================================================================
{
  using namespace Main_misc;
  using namespace Data_ini;

  float overlap, tolerance=0.1;

  ForParticlesAndDims(particle,dim)
    for (dim=1; dim<Dim; dim++) // skip dim=0 (asumming no walls in x-direction)
      if ( position(particle,dim) < domain_min(dim) || position(particle,dim) > domain_max(dim)) // outside the box...
	{
	  overlap=min( fabs(position(particle,dim) - domain_min(dim)), fabs(domain_max(dim) - position(particle,dim)) ); // ... and outside tolerance?
	  if (overlap > tolerance)
	    printf("\nPENETRATION ALERT: %i-component of particle no. %i at position [%f, %f, %f] is outside the domain by %f at step=%i.",
		   dim, particle, position(particle, 0), position(particle, 1), position(particle, 2), overlap, step);
	}
}

int checkIfInside(const int & step, const doubleArray & position, realArray & domain_min, realArray & domain_max)
// ================================================================================
// /Description:
//    Check if all the beads are within the confining geometry
// OUTPUT: 1=INSIDE, 0=OUTSIDE
//
// ================================================================================
{
  using namespace Main_misc;
  using namespace Data_ini;

  float overlap, tolerance=0.1;

  ForParticlesAndDims(particle,dim)
      if ( position(particle,dim) < domain_min(dim) || position(particle,dim) > domain_max(dim)) // outside the box...
	{
	  overlap=min( fabs(position(particle,dim) - domain_min(dim)), fabs(domain_max(dim) - position(particle,dim)) ); // ... and outside tolerance?
	  if (overlap > tolerance)
	    printf("\nPENETRATION ALERT: %i-component of particle no. %i at position [%f, %f, %f] is outside the domain by %f at step=%i.",
		   dim, particle, position(particle, 0), position(particle, 1), position(particle, 2), overlap, step);
	}
}

void NDist(const float x, const float y, const float z, float Length, float H, float W, double & D, double & Dx, double & Dy, double & Dz, bool ProductionPhase)
// Calcuate distance from bead to nearest wall in a pipe (of rectangular cross-section)
// - Assume walls in 'y' and 'z' directions only => Dx=0
// - Assume 0 < y < H, 0 < z < W
//
// IN:
// [y z] ... position
// [H W] ... height, width
//
// OUT:
// D=[D Dx Dy Dz]

{

  D=Dx=Dy=Dz=0; //init

  // 1) pipe

  bool Inside= ( x > 0 && x < Length ); // check if bead inside domain (relevant for translocation simulations only)

  if (ProductionPhase && Inside)
    {
      double dist[4]= {y, H-y, z, W-z}; // 4 distances btw bead and its orthogonal projections: (y,H-y,z,W-z)
      D=find_minmax(dist,4,0); // find minimum distance

      if (D == dist[0])
	Dy=1;
      else if (D == dist[1])
	Dy=-1;
      else if (D == dist[2])
	Dz=1;
      else if (D == dist[3])
	Dz=-1;
    }
  else if (ProductionPhase && !Inside)
    {
      D=Dx=Dy=Dz=0; //init
    }
  else // impose artificial wall at pore entrance (assume translocation in positive x-dir) DURING EQUILIBRATION to keep the PORE CLOSED
    {
      D=fabs(x);
      Dx=-1;
    }

//   // 2) slit
//   double dist[2]= {y, H-y};
//   D=find_minmax(dist,2,0);

//   if (D == dist[0])
//     Dy=1;
//   else if (D == dist[1])
//     Dy=-1;

}


int add_walls_to_CONFIG(const char * xyz_file, char * config_file, double Position_rescale)
{
  // INPUT: 
  //   - XYZ file with coordinates of wall atoms (use 'genwall' to generate), assume coordinates scaled by L!
  //   - CONFIG without wall atoms
  // OUTPUT: 
  //   - CONFIG with wall atoms
  //   - NoOfWallAtoms

  using namespace Main_misc;
  using namespace Data_ini;
  // For convenience, use C functions fprintf & fscanf since dlpoly input files require pedantic formatting.

  char * tmp_file= "atoms.tmp";
  float x=0.,y=0.,z=0.;
  FILE *xyz, *cfg, *fld, *ctr;
  int levcfg=0, imcon=0; // dlpoly params for the 2nd line of CONFIG file

  xyz = fopen(xyz_file, "r");
  cfg = fopen(config_file, "a");

  if(xyz == NULL) {
    printf("\nFailed to open %s. Aborting now...", xyz_file);
    abort();
  }

  int i=0;
  while (fscanf(xyz, "%f%f%f", &x,&y,&z) != EOF) // add wall atoms imported from XYZ file
    {
      i++;
      fprintf(cfg, "wall %10i\n %19.10f %19.10f %19.10f\n", NoOfParticles+i, x/Position_rescale, y/Position_rescale, z/Position_rescale); // append to CONFIG, (x,y,z) scaled by L -> convert to Angstroms
    }  

  fclose(cfg);
  fclose(xyz);

  string line;
  ifstream in(config_file);
  ofstream out(tmp_file);
  int j=0;
  while( getline(in,line) ) // change CONFIG records as needed
    {
      j++;
      if(j==2) // 2nd line => change NoOfParticles
	out << setw(10) << levcfg << setw(10) << imcon << setw(10) << NoOfParticles+i << endl;
      else
	out << line << endl;
    }
  
  in.close(); out.close();
  remove(config_file);
  rename(tmp_file, config_file);

  return (i);
}

int shift_position(doubleArray & position, double Position_rescale)
// Shifts the center of mass of the chain to the required location. 
// All length units are in Angstroms.
{
  // INPUTS:
  // - position
  // OUTPUTS:
  // - shifted position

  using namespace Main_misc;
  using namespace Data_ini;

  doubleArray com(Dim); com=0.;

  // Calculate Center Of Mass (com)...
  ForParticlesAndDims(particle,dim)
    com(dim)+=position(particle,dim);
  com/=NoOfParticles;

  // ... and shift it to the reference position com_ref
  double com_ref[3]={com_ref_x/Position_rescale, com_ref_y/Position_rescale, com_ref_z/Position_rescale}; // convert length [L] -> [Angstroms]

  ForParticlesAndDims(particle,dim)
    if(Mode)
      position(particle,dim)=position(particle,dim)+(com_ref[dim]-com(dim));
    else // bounded SARW => com_ref denotes starting point
      position(particle,dim)=position(particle,dim)+com_ref[dim];
}

void write_position_to_CONFIG(doubleArray & position, char * config_file)
// Write shifted position to CONFIG file (CONFIG is overwritten)
// That way the CONFIG can be run independently in dlpoly (correctly shifted coors esp important in the presence of wall atoms).
// All length units are in Angstroms.
{
  // INPUTS:
  // - position
  // OUTPUTS:
  // - shifted CONFIG file

  using namespace Main_misc;
  using namespace Data_ini;

  FILE *cfg;
  int levcfg=0, imcon=0; // dlpoly params for the 2nd line of CONFIG file

  cfg = fopen(config_file, "w");

  fprintf(cfg, "POLYMER CHAIN - SHIFTED CONFIGURATION\n");
  fprintf(cfg, "%10i %10i %10i \n", levcfg, imcon, NoOfParticles);

  ForParticles(particle)
    fprintf(cfg, "bead %10i\n %19.10f %19.10f %19.10f\n", particle+1, position(particle,0), position(particle,1), position(particle,2)); // length units in [A]

  fclose(cfg);
}

double variance( const doubleArray & arr, int no )
/***************************************************************************
 ROUTINE
               variance
 FUNCTION
               calculates variance of the input array
 SYNOPSIS
               subroutine variance(arr,no)
   formal 
                       data    I/O
       name            type    type    function
       -------------------------------------------------------------------
       arr		real	i	input data array
       no		int	i	number of elements in array
**************************************************************************/
{
  int i;
  double sum = 0.0, sum2 = 0.0, tavg;

  for (i = 0; i < no; i++)
    sum += arr(i);
  tavg = sum / (double) no; // average

  for (i = 0; i < no; i++)
    sum2 += (tavg - arr(i)) * (tavg - arr(i));

  if (no==1) no++;
  return( sum2 / (double) (no - 1) ); // variance
}



//************************************** END-OF-FILE  **************************************//
